﻿using System;
using REBORN.Common.Plugin.Redis;
using REBORN.CacheManager.DataStatistic;
using REBORN.Common.Plugin;
using REBORN.Common;
using Property.Repository;
using Property.Model.Extend;
using Property.Model.CacheEntity;

namespace Property.CacheManager.User
{
    /// <summary>
    /// redis缓存（只适用于数据统计）
    /// </summary>
    public class UserReidsCache 
    {
        static RedisHelper redis = new RedisHelper(DataConstant.REDIS_DB2);
        /// <summary>
        /// 设置缓存
        /// </summary>
        /// <param name="model"></param>
        public static void UserInfoSet(string cacheKey, UserInfo model, int JwtExpirTime)
        {
            try
            {
                TimeSpan ts = GetExpirTime(JwtExpirTime);
                redis.StringSet<UserInfo>(cacheKey, model, ts);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "UserInfoSet缓存设置失败");
            }
        }

        /// <summary>
        /// 获取缓存时长
        /// </summary>
        /// <param name="JwtExpirTime"></param>
        /// <returns></returns>
        private static TimeSpan GetExpirTime(int JwtExpirTime)
        {
            DateTime dt = DateTime.Now;
            DateTime dt2 = DateTime.Now;
            TimeSpan ts = dt.AddSeconds(JwtExpirTime) - dt2;
            return ts;
        }

        /// <summary>
        /// 获取缓存
        /// </summary>
        /// <returns></returns>
        public static UserInfo UserInfoGet(string cacheKey)
        {
            UserInfo model = null;
            try
            {
                model = redis.StringGet<UserInfo>(cacheKey);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "UserInfoSet缓存获取失败");
                return null;
            }            
            return model;
        }

        /// <summary>
        /// 获取用户登录信息
        /// </summary>
        /// <param name="uid"></param>
        /// <returns></returns>
        public static UserInfo GetUserLoginInfo(string uid)
        {

            string cacheKey = CacheKey.UserModuleCacheKeyConfig.USER_Login_Info + uid;
            UserInfo userInfo = null;
            try
            {
                userInfo = redis.StringGet<UserInfo>(cacheKey);
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "GetUserLoginInfo");
            }
            if (userInfo == null)
            {
                Rb_employeeRepository repository = new Repository.Rb_employeeRepository();
                Rb_Group_Repository rb_Group_Repository = new Repository.Rb_Group_Repository();
                Rb_branchRepository rb_branchRepository = new Repository.Rb_branchRepository();
                Rb_departmentRepository rb_departmentRepository = new Repository.Rb_departmentRepository();
                Rb_postRepository rb_postRepository = new Repository.Rb_postRepository();
                int emoloyeeId = 0;
                Int32.TryParse(uid, out emoloyeeId);
                RB_Employee_Extend model = repository.GetEmpInfoByEmpId(emoloyeeId);
                if (model != null)
                {
                    #region 获取基础信息
                    var gmodel = rb_Group_Repository.GetEntity(model.RB_Group_id);
                    string GroupName = gmodel != null ? gmodel.GroupName : "";
                    int versions = gmodel != null ? Convert.ToInt32(gmodel.Versions_Id) : 2;
                    string versionsName = "";
                    var bmodel = rb_branchRepository.GetEntity(model.RB_Branch_id);
                    string BranchName = bmodel != null ? bmodel.BName : "";
                    var dmodel = rb_departmentRepository.GetEntity(model.RB_Department_Id);
                    string DepartName = dmodel != null ? dmodel.DepartmentName : "";
                    var pmodel = rb_postRepository.GetEntity(model.RB_Post_Id);
                    string PostName = pmodel != null ? pmodel.PostName : "";
                    string domain = gmodel != null ? gmodel.Domain : "";
                    string b2bDomain = gmodel != null ? gmodel.B2BDomain : "";
                    #endregion
                    string token = "";
                    userInfo = new UserInfo
                    {
                        GroupName = GroupName,
                        VersionsName = versionsName,
                        BranchName = BranchName,
                        DepartName = DepartName,
                        PostName = PostName,
                        emName = model.EmName,
                        EmployeeId = model.EmployeeId,
                        RB_Group_id = model.RB_Group_id,
                        Versions_Id = versions,
                        RB_Branch_id = model.RB_Branch_id,
                        RB_Post_Id = model.RB_Post_Id,
                        RB_Department_Id = model.RB_Department_Id,
                        token = token,
                        SecretKey = model.SecretKey,
                        Icon = Config.GetFileUrl(model.EmPhoto),
                        IsParentCompany = bmodel != null ? bmodel.IsParentCompany.Value : 0,
                        ImAccount = model.EmLoginMobile,
                        B2BDomain = b2bDomain,
                        Domain = domain
                    };
                    try
                    {
                        TimeSpan ts = GetExpirTime(Config.JwtExpirTime);
                        redis.StringSet<UserInfo>(cacheKey, userInfo, ts);
                    }
                    catch
                    {
                    }
                }
            }
            return userInfo;
        }

        /// <summary>
        /// 根据员工编号获取员工信息
        /// </summary>
        /// <param name="EmployeeId">员工编号</param>
        /// <returns></returns>
        public static CacheUser GetEmployee(object EmployeeId)
        {
            if (EmployeeId != null)
            {
                int.TryParse(EmployeeId.ToString(), out int uid);
                string key = string.Concat(CacheKey.UserCacheKeyConfig.USER_EMPLOYEE, uid.ToString());
                if (uid > 0)
                {
                    if (redis.KeyExists(key))
                    {
                        return redis.StringGet<CacheUser>(key);
                    }
                    else
                    {
                        Rb_employeeRepository repository = new Repository.Rb_employeeRepository();
                        RB_Employee_Extend model = repository.GetEmpInfoByEmpId(uid);
                        if (model != null)
                        {
                            var user = new CacheUser()
                            {
                                EmployeeId = model.EmployeeId,
                                EmName = model.EmName,
                                RB_Group_id = model.RB_Group_id,
                                RB_Branch_id = model.RB_Branch_id,
                                IsLeave = model.IsLeave,
                                GroupName = model.GroupName,
                                BranchName = model.CompanyName,
                                DepartName = model.DepartmentName,
                                PostName = model.PostName,
                                EmPhoto = model.EmPhoto,
                                EmMobile = model.EmMobile,
                                LoginMobile = model.EmLoginMobile,
                                RB_Department_Id = model.RB_Department_Id,
                                RB_Post_Id = model.RB_Post_Id
                            };
                            TimeSpan ts = GetExpirTime((int)REBORN.Common.Enum.CacheTimeEnum.OneDay);
                            redis.StringSet<CacheUser>(key, user, ts);
                            return user;
                        }
                        else
                        {
                            return null;
                        }
                    }
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 判断key是否存在
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <returns></returns>
        public static bool Exists(string cacheKey) {
            return redis.KeyExists(cacheKey);
        }

        /// <summary>
        /// 设置缓存
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <param name="Data"></param>
        /// <param name="JwtExpirTime"></param>
        public static void Set(string cacheKey, object Data, int JwtExpirTime) {
            try
            {
                TimeSpan ts = GetExpirTime(JwtExpirTime);
                redis.StringSet(cacheKey, Data, ts);
            }
            catch (Exception)
            {
            }
        }
    }
}
