﻿using REBORN.Common;
using REBORN.Common.Enum;
using REBORN.Common.Plugin;
using Property.Model.Entity;
using Property.Model.Extend;
using Property.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Property.Module.UserModule
{
    /// <summary>
    /// 登录模块
    /// </summary>
    public class DepartmentModule
    {
        /// <summary>
        /// 公司
        /// </summary>
        private Rb_departmentRepository respository = new Rb_departmentRepository();

        /// <summary>
        /// 根据id获取部门信息
        /// </summary>
        /// <param name="DepartmentID">部门编号</param>
        /// <returns></returns>
        public RB_Department DepartmentGet(int DepartmentID)
        {
            return respository.GetEntity(DepartmentID);
        }


        /// <summary>
        /// 收款查询  部门树形级联  权限判断
        /// </summary>
        /// <param name="Type"></param>
        /// <param name="GBDid"></param>
        /// <param name="BranchId"></param>
        /// <returns></returns>
        public List<DepartmentChooseSell> GetDepartmentTreeForReceiveQuery(int Type, int GBDid, int BranchId = -1)
        {
            List<RB_Department_Extend> dList = new List<RB_Department_Extend>();
            List<RB_Department_Extend> FirstList = new List<RB_Department_Extend>();
            if (Type == 1)
            {
                dList = respository.GetList(new RB_Department_Extend() { RB_Group_Id = GBDid, RB_Branch_Id = BranchId });
                FirstList = dList.Where(x => x.Tier == dList.Min(y => y.Tier)).ToList();//找出1级部门
            }
            else if (Type == 2)
            {
                dList = respository.GetList(new RB_Department_Extend() { RB_Branch_Id = GBDid });
                FirstList = dList.Where(x => x.Tier == dList.Min(y => y.Tier)).ToList();//找出1级部门
            }
            else if (Type == 3)
            {
                var depStr = getDepartmentChildList(GBDid);
                dList = respository.GetDepartmentByDepartmentIds(depStr);//获取所有的部门
                FirstList = dList.Where(x => x.DepartmentID == GBDid).ToList();//找出1级部门
            }
            return GetNowDepartmentAndSubordinateChild(FirstList, dList);
        }

        /// <summary>
        /// 根据当前人的部门id获取下级部门id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public string getDepartmentChildList(int id)
        {
            return respository.getDepartmentChildList(id);
        }

        /// <summary>
        /// 获取部门子集菜单
        /// </summary>
        /// <param name="List"></param>
        /// <param name="dList"></param>
        /// <returns></returns>
        public List<DepartmentChooseSell> GetNowDepartmentAndSubordinateChild(List<RB_Department_Extend> List, List<RB_Department_Extend> dList)
        {
            List<DepartmentChooseSell> dclist = new List<DepartmentChooseSell>();
            foreach (var item in List)
            {
                DepartmentChooseSell dcmodel = new DepartmentChooseSell();
                List<RB_Department_Extend> ChildDList = dList.Where(x => x.ParentId == item.DepartmentID).ToList();
                List<DepartmentChooseSell> dcChildList = new List<DepartmentChooseSell>();


                if (ChildDList.Count > 0)
                    dcChildList.AddRange(GetNowDepartmentAndSubordinateChild(ChildDList, dList));

                dcmodel.ChildList = dcChildList ?? new List<DepartmentChooseSell>();
                dcmodel.DepartmentName = item.DepartmentName;
                dcmodel.DepartmentId = item.DepartmentID;
                dcmodel.ParentId = item.ParentId ?? 0;
                dcmodel.Tier = item.Tier ?? 1;
                dcmodel.Phone = "";
                dcmodel.EmPhoto = "";
                dclist.Add(dcmodel);
            }
            return dclist;
        }

        /// <summary>
        /// 部门树形结构
        /// </summary>
        public class DepartmentChooseSell
        {
            /// <summary>
            /// 部门名称
            /// </summary>
            public string DepartmentName { get; set; }
            /// <summary>
            /// 类型  1部门  2非部门
            /// </summary>
            public int Type { get; set; }

            /// <summary>
            /// 子集列表
            /// </summary>
            public List<DepartmentChooseSell> ChildList { get; set; }
            /// <summary>
            /// 部门id
            /// </summary>
            public int DepartmentId { get; set; }
            /// <summary>
            /// 父级部门
            /// </summary>
            public int ParentId { get; set; }
            /// <summary>
            /// 层级
            /// </summary>
            public int Tier { get; set; }
            /// <summary>
            /// 是否禁用
            /// </summary>
            public bool Disabled { get; set; }

            /// <summary>
            /// 头像
            /// </summary>
            public string EmPhoto { get; set; }

            /// <summary>
            /// 手机号码
            /// </summary>
            public string Phone { get; set; }

        }
    }
}
