﻿using REBORN.Common.AOP;
using REBORN.Common.Plugin;
using System;

namespace Property.Model.Entity
{
    /// <summary>
    /// rb_employee Entity
    /// </summary> 
    [Serializable]
    [DB(ConnectionName = "DefaultConnection")]
    public class RB_Employee
    {
        /// <summary>
        /// 主键:员工编号
        /// </summary>        
        public int EmployeeId
        {
            get;
            set;
        }
        /// <summary>
        /// 员工姓名
        /// </summary>    
        [RequiredValidate(showMessage = "请输入员工姓名")]
        [StringRangeValidate(1, 20, showMessage = "员工姓名不能超过20个字符")]
        public string EmName
        {
            get;
            set;
        }
        /// <summary>
        /// 性别
        /// </summary> 
        public int EmSex
        {
            get;
            set;
        }
        /// <summary>
        /// 身份证号码
        /// </summary>  
        [StringRangeValidate(1, 18, showMessage = "身份证不能超过18个字符")]
        public string EmCard
        {
            get;
            set;
        }
        /// <summary>
        /// 生日
        /// </summary>             
        public DateTime? EmBirthday
        {
            get;
            set;
        }
        /// <summary>
        /// 学历
        /// </summary>   
        [StringRangeValidate(1, 20, showMessage = "学历不能超过20个字符")]
        public string EmEducation
        {
            get;
            set;
        }
        /// <summary>
        /// 入职时间
        /// </summary>       
        [RequiredValidate(showMessage = "入职时间不能为空")]
        public DateTime? EmEntry
        {
            get;
            set;
        }
        /// <summary>
        /// 离职时间
        /// </summary>        
        public DateTime? EmLeave
        {
            get;
            set;
        }
        /// <summary>
        /// 地址
        /// </summary>   
        [StringRangeValidate(1, 255, showMessage = "地址不能超过255个字符")]
        public string EmAddress
        {
            get;
            set;
        }
        /// <summary>
        /// 手机号码
        /// </summary>     
        [StringRangeValidate(1, 50, showMessage = "手机号码不能超过50个字符")]
        public string EmMobile
        {
            get;
            set;
        }
        /// <summary>
        /// 联系电话
        /// </summary>
        [StringRangeValidate(1, 50, showMessage = "联系电话不能超过50个字符")]
        public string EmTel
        {
            get;
            set;
        }
        /// <summary>
        /// QQ号码
        /// </summary>     
        [StringRangeValidate(1, 20, showMessage = "QQ不能超过20个字符")]
        public string EmQQ
        {
            get;
            set;
        }
        /// <summary>
        /// 头像
        /// </summary>        
        public string EmPhoto
        {
            get;
            set;
        }
        /// <summary>
        /// 账号
        /// </summary>        
        [RequiredValidate(showMessage = "请输入帐号")]
        //[RegularValidate("^1[34578]\\d{9}$", showMessage = "帐号必须为手机号")]
        public string EmAccount
        {
            get;
            set;
        }
        /// <summary>
        /// 密码
        /// </summary>        
        public string EmPassword
        {
            get;
            set;
        }
        /// <summary>
        /// 集团编号
        /// </summary>        
        public int RB_Group_id
        {
            get;
            set;
        }
        /// <summary>
        /// 分公司编号
        /// </summary>        
        public int RB_Branch_id
        {
            get;
            set;
        }
        /// <summary>
        /// 部门编号
        /// </summary>        
        public int RB_Department_Id
        {
            get;
            set;
        }
        /// <summary>
        /// 职务编号
        /// </summary>        
        public int RB_Post_Id
        {
            get;
            set;
        }
        /// <summary>
        /// 创建时间
        /// </summary>        
        public DateTime? CreateTime
        {
            get;
            set;
        }
        /// <summary>
        /// 创建人
        /// </summary>        
        public int? CreateBy
        {
            get;
            set;
        }
        /// <summary>
        /// 更新时间
        /// </summary>        
        public DateTime? UpdateTime
        {
            get;
            set;
        }
        /// <summary>
        /// 更新人
        /// </summary>        
        public int? UpdateBy
        {
            get;
            set;
        }
        /// <summary>
        /// 密钥
        /// </summary>    
        public string SecretKey
        {
            get;
            set;
        }
        /// <summary>
        /// 是否离职，0:否，1是
        /// </summary>        
        public int? IsLeave
        {
            get;
            set;
        }

        /// <summary>
        /// 用户登陆手机号
        /// </summary>        
        public string EmLoginMobile
        {
            get;
            set;
        }
        /// <summary>
        /// 年假剩余天数
        /// </summary>
        public double? AnnualLeaveDay { get; set; }

        /// <summary>
        /// 是否管理员  1是  2否
        /// </summary>
        public int? IsAdmin { get; set; }

        /// <summary>
        /// 员工类型 0:正式员工,1:临时员工（外聘导游）
        /// </summary>
        public int? EmployeeType { get; set; }

        /// <summary>
        /// 财务催审次数
        /// </summary>
        public int? FinancePushOnNum { get; set; }

        /// <summary>
        /// 是否发送工作日志  1是  0否
        /// </summary>
        public int SendWorkLog { get; set; }
    }
}
