using System;
using System.Collections.Generic;
using Property.Model.Entity;
using REBORN.Common.AOP;

namespace Property.Model.Extend
{
    /// <summary>
    /// Rb_department Entity Model
    /// </summary>    
    [Serializable]
    [DB(ConnectionName = "DefaultConnection")]
    public class RB_Department_Extend : RB_Department
    {

        /// <summary>
        /// 负责人Name
        /// </summary>
        public string DepartmentManagerName { get; set; }

        /// <summary>
        /// 集团名称
        /// </summary>
        public string RB_Group_Name { get; set; }

        /// <summary>
        /// 分公司名称
        /// </summary>
        public string RB_Branch_Name { get; set; }

        /// <summary>
        /// 状态(0开启/1冻结)
        /// </summary>
        public string StatusStr
        {
            get
            {
                return this.Status == 0 ? "正常" : "失效";
            }
        }

        /// <summary>
        /// 员工总数
        /// </summary>
        public int EmployeeCount { get; set; }
        /// <summary>
        /// 岗位名称
        /// </summary>
        public string PostName { get; set; }
        /// <summary>
        /// 部门下的岗位列表
        /// </summary>
        public List<PostForDepartment> PostList { get; set; }
        /// <summary>
        /// 岗位id 数组
        /// </summary>
        public string[] PostIdArr { get; set; }
        /// <summary>
        /// 岗位id string
        /// </summary>
        public int[] PostIdStr { get; set; }
        /// <summary>
        /// 树形结构  子集
        /// </summary>
        public List<RB_Department_Extend> ChildList { get; set; }

        /// <summary>
        /// 公司编号【查询使用】
        /// </summary>
        public string QBranchIds { get; set; }

        /// <summary>
        /// 多个部门编号
        /// </summary>
        public string DepartmentIds { get; set; }

        /// <summary>
        /// 类型  1部门  2员工
        /// </summary>
        public int DType { get; set; } = 1;
        /// <summary>
        /// 员工头像
        /// </summary>
        public string Images { get; set; }

        /// <summary>
        /// 部门负责人id列表
        /// </summary>
        public List<int> ManagerIdArr { get; set; }

    }

    /// <summary>
    /// 部门弹窗选择
    /// </summary>
    public class DepartmentChoose
    {
        /// <summary>
        /// 部门名称
        /// </summary>
        public string DepartmentName { get; set; }
        /// <summary>
        /// 类型  1部门  2非部门
        /// </summary>
        public int Type { get; set; }

        /// <summary>
        /// 子集列表
        /// </summary>
        public List<DepartmentChoose> ChildList { get; set; }
        /// <summary>
        /// 部门id
        /// </summary>
        public int DepartmentId { get; set; }
        /// <summary>
        /// 父级部门
        /// </summary>
        public int ParentId { get; set; }
        /// <summary>
        /// 层级
        /// </summary>
        public int Tier { get; set; }
        /// <summary>
        /// 是否禁用
        /// </summary>
        public bool Disabled { get; set; }

        /// <summary>
        /// 头像
        /// </summary>
        public string EmPhoto { get; set; }

        /// <summary>
        /// 手机号码
        /// </summary>
        public string Phone { get; set; }
    }
    /// <summary>
    /// 部门下员工
    /// </summary>
    public class EmployeeForDepartment
    {
        /// <summary>
        /// 员工id
        /// </summary>
        public int EmployeeId { get; set; }
        /// <summary>
        /// 员工名称
        /// </summary>
        public string EmName { get; set; }
    }
    /// <summary>
    /// 部门下所有岗位
    /// </summary>
    public class PostForDepartment
    {
        /// <summary>
        /// 岗位id
        /// </summary>
        public int PostId { get; set; }
        /// <summary>
        /// 岗位名称
        /// </summary>
        public string PostName { get; set; }
    }
    /// <summary>
    /// 聊天树形列表
    /// </summary>
    public class CompanyDepartEmpForChar
    {
        /// <summary>
        /// 名称
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// 类型  1非员工  2员工
        /// </summary>
        public int Type { get; set; }
        /// <summary>
        /// 子集列表
        /// </summary>
        public List<CompanyDepartEmpForChar> ChildList { get; set; }
        /// <summary>
        /// id
        /// </summary>
        public int EmpId { get; set; }
        /// <summary>
        /// 是否禁用
        /// </summary>
        public bool Disabled { get; set; }
        /// <summary>
        /// IM账户
        /// </summary>
        public string IMAccount { get; set; }
        /// <summary>
        /// 头像
        /// </summary>
        public string Icon { get; set; }
        /// <summary>
        /// 数量
        /// </summary>
        public int Count { get; set; }
    }
}
