﻿using REBORN.Common;
using REBORN.Common.Enum;
using REBORN.Common.Plugin;
using Property.Model.Entity;
using Property.Model.Extend;
using Property.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;
using REBORN.Commom.Plugin;
using System.IO;
using Property.Repository.Supplies;
using Property.Model.Extend.Supplies;
using Newtonsoft.Json.Linq;

namespace Property.Module.FixedAssets
{
    /// <summary>
    /// 耗材模块
    /// </summary>
    public class SuppliesModule
    {
        /// <summary>
        /// 物料档案
        /// </summary>
        private readonly RB_Supplies_MaterialRepository supplies_MaterialRepository = new RB_Supplies_MaterialRepository();
        /// <summary>
        /// 仓库
        /// </summary>
        private readonly RB_Supplies_WareHouseRepository supplies_WareHouseRepository = new RB_Supplies_WareHouseRepository();
        /// <summary>
        /// 入库单
        /// </summary>
        private readonly RB_Supplies_StockInRepository supplies_StockInRepository = new RB_Supplies_StockInRepository();
        /// <summary>
        /// 入库明细
        /// </summary>
        private readonly RB_Supplies_StockInDetailRepository supplies_StockInDetailRepository = new RB_Supplies_StockInDetailRepository();
        /// <summary>
        /// 库存表
        /// </summary>
        private readonly RB_Supplies_InventoryRepository supplies_InventoryRepository = new RB_Supplies_InventoryRepository();
        /// <summary>
        /// 库存明细表
        /// </summary>
        private readonly RB_Supplies_InventoryDetailRepository supplies_InventoryDetailRepository = new RB_Supplies_InventoryDetailRepository();
        /// <summary>
        /// 出库单
        /// </summary>
        private readonly RB_Supplies_StockOutRepository supplies_StockOutRepository = new RB_Supplies_StockOutRepository();
        /// <summary>
        /// 出库单明细
        /// </summary>
        private readonly RB_Supplies_StockOutDetailRepository supplies_StockOutDetailRepository = new RB_Supplies_StockOutDetailRepository();
        private readonly Rb_branchRepository branchRepository = new Rb_branchRepository();
        private readonly Rb_departmentRepository departmentRepository = new Rb_departmentRepository();
        //耗材申请
        private readonly RB_Supplies_ApplyforRepository supplies_ApplyforRepository = new RB_Supplies_ApplyforRepository();
        private readonly RB_Supplies_ApplyforWeekdayRepository supplies_ApplyforWeekdayRepository = new RB_Supplies_ApplyforWeekdayRepository();
        private readonly RB_Supplies_ApplyforSpecialRepository supplies_ApplyforSpecialRepository = new RB_Supplies_ApplyforSpecialRepository();
        /// <summary>
        /// 分类
        /// </summary>
        private readonly RB_Property_CategoryRepository property_CategoryRepository = new RB_Property_CategoryRepository();
        /// <summary>
        /// 采购单
        /// </summary>
        private readonly RB_ProcurementRepository procurementRepository = new RB_ProcurementRepository();
        /// <summary>
        /// 采购单明细
        /// </summary>
        private readonly RB_Procurement_DetailRepository procurement_DetailRepository = new RB_Procurement_DetailRepository();
        /// <summary>
        /// 盘点单
        /// </summary>
        private readonly RB_Supplies_CheckRepository supplies_CheckRepository = new RB_Supplies_CheckRepository();
        /// <summary>
        /// 盘点明细
        /// </summary>
        private readonly RB_Supplies_CheckDetailRepository supplies_CheckDetailRepository = new RB_Supplies_CheckDetailRepository();

        /// <summary>
        ///盘点报损记录
        /// </summary>
        private readonly RB_Supplies_ReportLossRecordRepository suppliesReportLossRecordRepository = new RB_Supplies_ReportLossRecordRepository();
        /// <summary>
        /// 盘点报损记录详情
        /// </summary>
        private readonly RB_Supplies_ReportLossRecordDetailRepository suppliesReportLossRecordDetailRepository = new RB_Supplies_ReportLossRecordDetailRepository();


        #region 基础配置

        /// <summary>
        /// 获取物料详情
        /// </summary>
        /// <param name="materialId"></param>
        /// <returns></returns>
        public RB_Supplies_Material_Extend GetMaterialInfo(int materialId)
        {
            var model = supplies_MaterialRepository.GetEntity(materialId).RefMapperTo<RB_Supplies_Material_Extend>();
            if (model == null)
            {
                return new RB_Supplies_Material_Extend();
            }

            if (model.Images == "" || model.Images == null)
            {
                model.Images = "[]";
            }
            var imglist = JsonConvert.DeserializeObject<List<string>>(model.Images);
            model.ImageList = new List<string>();
            foreach (var img in imglist)
            {
                model.ImageList.Add(Config.GetFileUrl(img));
            }
            if (model.CategoryId > 0)
            {
                model.CategoryName = property_CategoryRepository.GetEntity(model.CategoryId)?.Name ?? "";
            }
            return model;
        }

        /// <summary>
        /// 获取物料档案分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_Material_Extend> GetMaterialPageList(int pageIndex, int pageSize, RB_Supplies_Material_Extend demodel, out long count)
        {
            var list = supplies_MaterialRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            List<RB_Supplies_Inventory_Extend> iList = new List<RB_Supplies_Inventory_Extend>();
            if (demodel.WareHouseId > 0 && list.Any())
            {
                //查询出该仓库  耗材下 所有的库存
                string Sstr = string.Join(",", list.Select(x => x.Id).Distinct());
                iList = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseIdStr = demodel.WareHouseId.Value.ToString(), SuppliesIdStr = Sstr });
            }
            foreach (var item in list)
            {
                if (item.Images == "" || item.Images == null)
                {
                    item.Images = "[]";
                }
                var imglist = JsonConvert.DeserializeObject<List<string>>(item.Images);
                item.ImageList = new List<string>();
                foreach (var img in imglist)
                {
                    item.ImageList.Add(Config.GetFileUrl(img));
                }
                item.InventoryNum = iList.Where(x => x.SuppliesId == item.Id).FirstOrDefault()?.Number ?? 0;
                item.GoodsSpecificationList = new List<string>();
                item.GoodsCategoryList = new List<string>();
                if (!string.IsNullOrEmpty(item.SpecificationName))
                {
                    item.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(item.SpecificationName);
                }
                if (!string.IsNullOrEmpty(item.GoodsCategoryName))
                {
                    item.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(item.GoodsCategoryName);
                }
            }
            return list;
        }

        /// <summary>
        /// 设置物料档案信息
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetMaterialInfo(RB_Supplies_Material_Extend demodel)
        {
            if (demodel.Id > 0)
            {
                //图片处理
                List<string> imgList = new List<string>();
                ImageOperation(demodel.ImageList, out imgList);//转移图片  未做删除处理
                demodel.Images = JsonConvert.SerializeObject(imgList);

                var dmodel = supplies_MaterialRepository.GetEntity(demodel.Id);
                dmodel.BrandName = demodel.BrandName;
                dmodel.CategoryId = demodel.CategoryId;
                dmodel.Images = demodel.Images;
                dmodel.Money = demodel.Money;
                dmodel.Name = demodel.Name;
                dmodel.Remark = demodel.Remark;
                dmodel.Units = demodel.Units;
                dmodel.UpdateBy = demodel.UpdateBy;
                dmodel.UpdateDate = demodel.UpdateDate;
                dmodel.Barcode = demodel.Barcode;
                dmodel.ReservedStock = demodel.ReservedStock;
                dmodel.SafetyStock = demodel.SafetyStock;
                dmodel.SupplierName = demodel.SupplierName;
                dmodel.SuppliesModel = demodel.SuppliesModel;
                bool flag = supplies_MaterialRepository.Update(dmodel);
                return flag;
            }
            else
            {
                //图片处理
                List<string> imgList = new List<string>();
                ImageOperation(demodel.ImageList, out imgList);//转移图片  未做删除处理
                demodel.Images = JsonConvert.SerializeObject(imgList);

                int PropertyId = supplies_MaterialRepository.Insert(demodel);
                bool flag = PropertyId > 0;
                if (flag)
                {
                    //编码自动生成
                    string BMStr = PropertyId.ToString();
                    if (BMStr.Length < 5)
                    {
                        switch (BMStr.Length)
                        {
                            case 1:
                                BMStr = "0000" + BMStr; break;
                            case 2:
                                BMStr = "000" + BMStr; break;
                            case 3:
                                BMStr = "00" + BMStr; break;
                            case 4:
                                BMStr = "0" + BMStr; break;
                        }
                    }
                    Dictionary<string, object> files = new Dictionary<string, object>() {
                        { nameof(RB_Supplies_Material_Extend.SuppliesNum),BMStr}
                    };
                    List<WhereHelper> wheres = new List<WhereHelper>() {
                        new WhereHelper(){
                             FiledName=nameof(RB_Supplies_Material_Extend.Id),
                              FiledValue=PropertyId,
                               OperatorEnum=OperatorEnum.Equal
                        }
                    };
                    supplies_MaterialRepository.Update(files, wheres);
                }
                return flag;
            }
        }

        /// <summary>
        /// 删除物料档案信息
        /// </summary>
        /// <param name="supplierId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool DelMaterialInfo(int materialId, UserInfo userInfo)
        {
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_Material_Extend.Status),(int)DateStateEnum.Delete},
                { nameof(RB_Supplies_Material_Extend.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_Material_Extend.UpdateDate),DateTime.Now},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                 FiledName=nameof(RB_Supplies_Material_Extend.Id),
                  FiledValue=materialId,
                   OperatorEnum=OperatorEnum.Equal
                }
            };
            return supplies_MaterialRepository.Update(files, wheres);
        }

        /// <summary>
        /// 获取仓库列表
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_WareHouse_Extend> GetWareHouseList(RB_Supplies_WareHouse_Extend demodel)
        {
            return supplies_WareHouseRepository.GetList(demodel);
        }

        /// <summary>
        /// 获取仓库分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_WareHouse_Extend> GetWareHousePageList(int pageIndex, int pageSize, RB_Supplies_WareHouse_Extend demodel, out long count)
        {
            return supplies_WareHouseRepository.GetPageList(pageIndex, pageSize, demodel, out count);
        }

        /// <summary>
        /// 处理图片
        /// </summary>
        /// <param name="images"></param>
        /// <param name="imgList"></param>
        private static void ImageOperation(List<string> images, out List<string> imgList)
        {
            imgList = new List<string>();
            if (images != null && images.Count() > 0)
            {
                for (int i = 0; i < images.Count(); i++)
                {
                    string userFile = "/New/Upload/Property/";
                    string NewFileUrl;
                    if (Config.GetFilePath(images[i]).Contains("/New/"))
                    {
                        NewFileUrl = Config.GetFilePath(images[i]).Replace("Temporary", "Property");
                    }
                    else
                    {
                        NewFileUrl = "/New" + Config.GetFilePath(images[i]).Replace("Temporary", "Property");
                    }
                    FileCommonHelper.OssFileMove(Config.GetFilePath(images[i]), userFile);
                    imgList.Add(NewFileUrl);
                }
            }
        }

        /// <summary>
        /// 设置仓库信息
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetWareHouseInfo(RB_Supplies_WareHouse_Extend demodel)
        {
            if (demodel.Id > 0)
            {
                Dictionary<string, object> files = new Dictionary<string, object>() {
                    { nameof(RB_Supplies_WareHouse.Name),demodel.Name},
                    { nameof(RB_Supplies_WareHouse.UpdateBy),demodel.UpdateBy},
                    { nameof(RB_Supplies_WareHouse.UpdateDate),demodel.UpdateDate}
                };
                List<WhereHelper> wheres = new List<WhereHelper>() {
                    new WhereHelper(){
                         FiledName=nameof(RB_Supplies_WareHouse.Id),
                          FiledValue=demodel.Id,
                           OperatorEnum=OperatorEnum.Equal
                    }
                };
                return supplies_WareHouseRepository.Update(files, wheres);
            }
            else
            {
                return supplies_WareHouseRepository.Insert(demodel) > 0;
            }
        }

        /// <summary>
        /// 获取入库单列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockIn_Extend> GetStockInPageList(int pageIndex, int pageSize, RB_Supplies_StockIn_Extend demodel, out long count)
        {
            var list = supplies_StockInRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            List<RB_Supplies_StockInDetail_Extend> sdList = new List<RB_Supplies_StockInDetail_Extend>();
            if (list.Any())
            {
                //查询出所有的耗材详细
                string IdStr = string.Join(",", list.Select(x => x.Id).Distinct());
                sdList = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockInIdStr = IdStr });
            }
            foreach (var item in list)
            {
                item.DetailList = sdList.Where(x => x.StockInId == item.Id).ToList();
                foreach (var qitem in item.DetailList)
                {
                    qitem.GoodsSpecificationList = new List<string>();
                    qitem.GoodsCategoryList = new List<string>();
                    if (!string.IsNullOrEmpty(qitem.SpecificationName))
                    {
                        qitem.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(qitem.SpecificationName);
                    }
                    if (!string.IsNullOrEmpty(qitem.GoodsCategoryName))
                    {
                        qitem.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(qitem.GoodsCategoryName);
                    }
                }
            }

            return list;
        }

        /// <summary>
        /// 删除仓库
        /// </summary>
        /// <param name="wareHouseId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool DelWareHouseInfo(int wareHouseId, UserInfo userInfo)
        {
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_WareHouse.Status),(int)DateStateEnum.Delete},
                { nameof(RB_Supplies_WareHouse.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_WareHouse.UpdateDate),DateTime.Now}
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_WareHouse.Id),
                        FiledValue=wareHouseId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            return supplies_WareHouseRepository.Update(files, wheres);
        }

        /// <summary>
        /// 验证库存
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool ValidateStockInInfo(RB_Supplies_StockIn_Extend demodel)
        {
            //更新明细
            var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockInId = demodel.Id });
            //修改
            var UpdateList = list.Where(x => demodel.DetailList.Select(y => y.SuppliesId).Contains(x.SuppliesId)).ToList();
            //删除
            var DelList = list.Where(x => !demodel.DetailList.Select(y => y.SuppliesId).Contains(x.SuppliesId)).ToList();
            foreach (var item in UpdateList)
            {
                var dmodel = demodel.DetailList.Where(x => x.SuppliesId == item.SuppliesId).FirstOrDefault();
                if ((dmodel.Number ?? 0) - (item.Number ?? 0) < 0)
                {
                    //新增库存表数据
                    var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                    if (ilist.Any())
                    {
                        var iModel = ilist.FirstOrDefault();
                        if (iModel.Number < (item.Number ?? 0) - (dmodel.Number ?? 0))
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }
            }
            foreach (var item in DelList)
            {
                //新增库存表数据
                var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                if (ilist.Any())
                {
                    var iModel = ilist.FirstOrDefault();
                    if (iModel.Number < (item.Number ?? 0))
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// 验证库存
        /// </summary>
        /// <param name="stockInId"></param>
        /// <returns></returns>
        public bool ValidateStockInDel(int stockInId)
        {
            var Model = supplies_StockInRepository.GetEntity(stockInId);
            //更新明细
            var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = Model.RB_Group_Id, StockInId = stockInId });
            foreach (var item in list)
            {
                //新增库存表数据
                var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = Model.RB_Group_Id, WarehouseId = Model.WarehouseId, SuppliesId = item.SuppliesId });
                if (ilist.Any())
                {
                    var iModel = ilist.FirstOrDefault();
                    if (iModel.Number < (item.Number ?? 0))
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// 设置入库单
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetStockInInfo(RB_Supplies_StockIn_Extend demodel)
        {
            var trans = supplies_StockInRepository.DbTransaction;
            try
            {
                if (demodel.Id > 0)
                {
                    var model = supplies_StockInRepository.GetEntity(demodel.Id);
                    model.Money = demodel.Money;
                    model.Remark = demodel.Remark;
                    model.StockInDate = demodel.StockInDate;
                    model.SupplierName = demodel.SupplierName;
                    model.WarehouseId = demodel.WarehouseId;
                    model.UpdateBy = demodel.UpdateBy;
                    model.UpdateDate = demodel.UpdateDate;
                    model.StockInState = StockInStatusEnum.StockIn;
                    bool flag = supplies_StockInRepository.Update(model, trans);
                    if (flag)
                    {
                        //更新明细
                        var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockInId = demodel.Id });

                        //新增
                        var InsertList = demodel.DetailList.Where(x => !list.Select(y => y.SuppliesId).Contains(x.SuppliesId)).ToList();
                        //修改
                        var UpdateList = list.Where(x => demodel.DetailList.Select(y => y.SuppliesId).Contains(x.SuppliesId)).ToList();
                        //删除
                        var DelList = list.Where(x => !demodel.DetailList.Select(y => y.SuppliesId).Contains(x.SuppliesId)).ToList();
                        foreach (var item in InsertList)
                        {
                            supplies_StockInDetailRepository.Insert(new RB_Supplies_StockInDetail()
                            {
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                Money = item.Money,
                                Number = item.Number,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockInId = demodel.Id,
                                SuppliesId = item.SuppliesId,
                                UnitPrice = item.UnitPrice,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);
                            //新增库存表数据
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number += item.Number;//增加库存
                                iModel.Money += item.Money;//增加库存金额
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "新增库存-修改入库单"
                                    }, trans);
                                }
                            }
                            else
                            {
                                //新增库存表
                                int iId = supplies_InventoryRepository.Insert(new RB_Supplies_Inventory()
                                {
                                    CreateBy = demodel.CreateBy,
                                    CreateDate = DateTime.Now,
                                    Id = 0,
                                    Number = item.Number,
                                    Money = item.Money,
                                    RB_Branch_Id = demodel.RB_Branch_Id,
                                    RB_Group_Id = demodel.RB_Group_Id,
                                    Status = 0,
                                    SuppliesId = item.SuppliesId,
                                    UpdateBy = demodel.UpdateBy,
                                    UpdateDate = demodel.UpdateDate,
                                    WarehouseId = demodel.WarehouseId
                                }, trans);
                                if (iId > 0)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iId,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "新增库存-修改入库单"
                                    }, trans);
                                }
                            }
                        }
                        foreach (var item in UpdateList)
                        {
                            var dmodel = demodel.DetailList.Where(x => x.SuppliesId == item.SuppliesId).FirstOrDefault();
                            Dictionary<string, object> files = new Dictionary<string, object>() {
                                { nameof(RB_Supplies_StockInDetail.Money),dmodel.Money},
                                { nameof(RB_Supplies_StockInDetail.Number),dmodel.Number},
                                { nameof(RB_Supplies_StockInDetail.UnitPrice),dmodel.UnitPrice},
                                { nameof(RB_Supplies_StockInDetail.Remark),dmodel.Remark}
                            };
                            List<WhereHelper> wheres = new List<WhereHelper>() {
                                new WhereHelper(){
                                     FiledName=nameof(RB_Supplies_StockInDetail.Id),
                                      FiledValue=item.Id,
                                       OperatorEnum=OperatorEnum.Equal
                                }
                            };
                            bool dflag = supplies_StockInDetailRepository.Update(files, wheres, trans);
                            if (dflag)
                            {

                                //新增库存表数据
                                var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                                if (ilist.Any())
                                {
                                    var iModel = ilist.FirstOrDefault();
                                    iModel.Number += dmodel.Number;//增加库存
                                    iModel.Money += dmodel.Money;
                                    var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                    if (iflag)
                                    {
                                        //增加库存明细
                                        supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                        {
                                            CreateBy = demodel.CreateBy,
                                            CreateDate = DateTime.Now,
                                            InventoryId = iModel.Id,
                                            Number = dmodel.Number,
                                            Money = dmodel.Money,
                                            RB_Branch_Id = demodel.RB_Branch_Id,
                                            RB_Group_Id = demodel.RB_Group_Id,
                                            Status = 0,
                                            Type = 1,
                                            Id = 0,
                                            Description = "增加库存-修改入库单"
                                        }, trans);
                                    }
                                }
                                else
                                {
                                    //新增库存表
                                    int iId = supplies_InventoryRepository.Insert(new RB_Supplies_Inventory()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        Id = 0,
                                        Number = dmodel.Number,
                                        Money = dmodel.Money,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        SuppliesId = item.SuppliesId,
                                        UpdateBy = demodel.UpdateBy,
                                        UpdateDate = demodel.UpdateDate,
                                        WarehouseId = demodel.WarehouseId
                                    }, trans);
                                    if (iId > 0)
                                    {
                                        //增加库存明细
                                        supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                        {
                                            CreateBy = demodel.CreateBy,
                                            CreateDate = DateTime.Now,
                                            InventoryId = iId,
                                            Number = dmodel.Number,
                                            Money = dmodel.Money,
                                            RB_Branch_Id = demodel.RB_Branch_Id,
                                            RB_Group_Id = demodel.RB_Group_Id,
                                            Status = 0,
                                            Type = 1,
                                            Id = 0,
                                            Description = "增加库存-修改入库单"
                                        }, trans);
                                    }
                                }
                            }
                        }
                        foreach (var item in DelList)
                        {
                            Dictionary<string, object> files = new Dictionary<string, object>() {
                                { nameof(RB_Supplies_StockInDetail.Status),(int)DateStateEnum.Delete}
                            };
                            List<WhereHelper> wheres = new List<WhereHelper>() {
                                new WhereHelper(){
                                     FiledName=nameof(RB_Supplies_StockInDetail.Id),
                                      FiledValue=item.Id,
                                       OperatorEnum=OperatorEnum.Equal
                                }
                            };
                            supplies_StockInDetailRepository.Update(files, wheres, trans);
                        }
                    }
                    supplies_StockInRepository.DBSession.Commit();
                    return flag;
                }
                else
                {
                    //生产编号
                    demodel.StockInState = StockInStatusEnum.StockIn;
                    int Id = supplies_StockInRepository.Insert(demodel, trans);
                    if (Id > 0)
                    {
                        //编码自动生成
                        string BMStr = Id.ToString();
                        if (BMStr.Length < 5)
                        {
                            switch (BMStr.Length)
                            {
                                case 1:
                                    BMStr = "0000" + BMStr; break;
                                case 2:
                                    BMStr = "000" + BMStr; break;
                                case 3:
                                    BMStr = "00" + BMStr; break;
                                case 4:
                                    BMStr = "0" + BMStr; break;
                            }
                        }
                        BMStr = "RK" + DateTime.Now.ToString("yyyyMMdd") + BMStr;
                        Dictionary<string, object> files = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_StockIn.StockInNum),BMStr}
                        };
                        List<WhereHelper> wheres = new List<WhereHelper>() {
                            new WhereHelper(){
                                 FiledName=nameof(RB_Supplies_StockIn.Id),
                                  FiledValue=Id,
                                   OperatorEnum=OperatorEnum.Equal
                            }
                        };
                        supplies_StockInRepository.Update(files, wheres, trans);

                        //新增明细
                        foreach (var item in demodel.DetailList)
                        {
                            supplies_StockInDetailRepository.Insert(new RB_Supplies_StockInDetail()
                            {
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                Money = item.Money,
                                Number = item.Number,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockInId = Id,
                                SuppliesId = item.SuppliesId,
                                UnitPrice = item.UnitPrice,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);

                            //新增库存表数据
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number += item.Number;//增加库存
                                iModel.Money += item.Money;
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "新增库存"
                                    }, trans);
                                }
                            }
                            else
                            {
                                //新增库存表
                                int iId = supplies_InventoryRepository.Insert(new RB_Supplies_Inventory()
                                {
                                    CreateBy = demodel.CreateBy,
                                    CreateDate = DateTime.Now,
                                    Id = 0,
                                    Number = item.Number,
                                    Money = item.Money,
                                    RB_Branch_Id = demodel.RB_Branch_Id,
                                    RB_Group_Id = demodel.RB_Group_Id,
                                    Status = 0,
                                    SuppliesId = item.SuppliesId,
                                    UpdateBy = demodel.UpdateBy,
                                    UpdateDate = demodel.UpdateDate,
                                    WarehouseId = demodel.WarehouseId
                                }, trans);
                                if (iId > 0)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iId,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "新增库存"
                                    }, trans);
                                }
                            }
                        }

                    }

                    supplies_StockInRepository.DBSession.Commit();
                    return Id > 0;
                }
            }
            catch (Exception ex)
            {
                supplies_StockInRepository.DBSession.Rollback("SetStockInInfo");
                LogHelper.Write(ex, "SetStockInInfo");
                return false;
            }
        }

        /// <summary>
        /// 领用统计
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public RB_Supplies_StockOutDetail_Extend GetRecipientsQueryTotal(RB_Supplies_StockOutDetail_Extend demodel)
        {
            var list = supplies_StockOutDetailRepository.GetRecipientsQueryTotal(demodel);
            return list.FirstOrDefault();
        }

        /// <summary>
        /// 获取领用列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockOutDetail_Extend> GetRecipientsQueryPageList(int pageIndex, int pageSize, RB_Supplies_StockOutDetail_Extend demodel, out long count)
        {
            var list = supplies_StockOutDetailRepository.GetRecipientsQueryPageList(pageIndex, pageSize, demodel, out count);
            List<RB_Branch_Extend> blist = new List<RB_Branch_Extend>();
            List<RB_Department_Extend> dlist = new List<RB_Department_Extend>();
            if (list.Any())
            {
                string bstr = string.Join(",", list.Select(x => x.BranchId ?? 0).Distinct());
                string dstr = string.Join(",", list.Select(x => x.DepartmentId ?? 0).Distinct());
                blist = branchRepository.GetBranchNameByIds(bstr);
                dlist = departmentRepository.GetDepartmentByDepartmentIds(dstr);
            }
            foreach (var item in list)
            {
                var bmodel = blist.Where(x => x.Id == item.BranchId).FirstOrDefault();
                var dmodel = dlist.Where(x => x.DepartmentID == item.DepartmentId).FirstOrDefault();
                item.BranchName = bmodel?.BName ?? "";
                item.DepartmentName = dmodel?.DepartmentName ?? "";
                item.EmName = CacheManager.User.UserReidsCache.GetEmployee(item.EmployeeId)?.EmName ?? "";
                item.GoodsSpecificationList = new List<string>();
                item.GoodsCategoryList = new List<string>();
                if (!string.IsNullOrEmpty(item.SpecificationName))
                {
                    item.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(item.SpecificationName);
                }
                if (!string.IsNullOrEmpty(item.GoodsCategoryName))
                {
                    item.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(item.GoodsCategoryName);
                }
            }
            return list;
        }

        /// <summary>
        /// 获取库存分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_Inventory_Extend> GetInventoryPageList(int pageIndex, int pageSize, RB_Supplies_Inventory_Extend demodel, out long count)
        {
            var list = supplies_InventoryRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            if (list.Any())
            {
                string wstr = string.Join(",", list.Select(x => x.WarehouseId ?? 0).Distinct());
                string sstr = string.Join(",", list.Select(x => x.SuppliesId ?? 0).Distinct());
                //查询仓库
                var wlist = supplies_WareHouseRepository.GetList(new RB_Supplies_WareHouse_Extend() { RB_Group_Id = demodel.RB_Group_Id, WareHouseIdStr = wstr });
                //查询耗材
                var mlist = supplies_MaterialRepository.GetList(new RB_Supplies_Material_Extend() { RB_Group_Id = demodel.RB_Group_Id, SuppliesIdStr = sstr }, true);

                foreach (var item in list)
                {
                    item.WModel = wlist.Where(x => x.Id == item.WarehouseId).FirstOrDefault();
                    item.SModel = mlist.Where(x => x.Id == item.SuppliesId).FirstOrDefault();
                    if (item.WModel == null)
                    {
                        item.WModel = new RB_Supplies_WareHouse_Extend();
                    }
                    if (item.SModel == null)
                    {
                        item.SModel = new RB_Supplies_Material_Extend();
                    }
                    item.SModel.GoodsSpecificationList = new List<string>();
                    item.SModel.GoodsCategoryList = new List<string>();
                    if (!string.IsNullOrEmpty(item.SModel.SpecificationName))
                    {
                        item.SModel.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(item.SModel.SpecificationName);
                    }
                    if (!string.IsNullOrEmpty(item.SModel.GoodsCategoryName))
                    {
                        item.SModel.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(item.SModel.GoodsCategoryName);
                    }
                }
            }
            return list;
        }

        /// <summary>
        /// 获取库存明细分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_InventoryDetail_Extend> GetInventoryDetailPageList(int pageIndex, int pageSize, RB_Supplies_Inventory_Extend demodel, out long count)
        {
            var list = supplies_InventoryRepository.GetInventoryDetailPageList(pageIndex, pageSize, demodel, out count);
            return list;
        }


        /// <summary>
        /// 取消出库单
        /// </summary>
        /// <param name="stockOutId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool CancelStockOutInfo(int stockOutId, UserInfo userInfo)
        {
            var demodel = supplies_StockOutRepository.GetEntity(stockOutId);
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_StockOut.StockOutState),(int)StockOutStatusEnum.StockOutCancel},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_StockOut.Id),
                        FiledValue=stockOutId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_StockOutRepository.Update(files, wheres);
            if (flag && demodel.StockOutState == StockOutStatusEnum.StockOut)
            {
                //删除明细 ---  查库存操作
                var list = supplies_StockOutDetailRepository.GetList(new RB_Supplies_StockOutDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockOutId = stockOutId });
                foreach (var item in list)
                {
                    //减少库存表数据
                    var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                    if (ilist.Any())
                    {
                        var iModel = ilist.FirstOrDefault();
                        iModel.Number += item.Number;//增加库存
                        iModel.Money += item.Money;
                        var iflag = supplies_InventoryRepository.Update(iModel);
                        if (iflag)
                        {
                            //增加库存明细
                            supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                            {
                                CreateBy = userInfo.EmployeeId,
                                CreateDate = DateTime.Now,
                                InventoryId = iModel.Id,
                                Number = item.Number,
                                Money = item.Money,
                                RB_Branch_Id = userInfo.RB_Branch_id,
                                RB_Group_Id = userInfo.RB_Group_id,
                                Status = 0,
                                Type = 1,
                                Id = 0,
                                Description = "增加库存-取消出库单"
                            });
                        }
                    }
                }
            }
            return flag;
        }

        /// <summary>
        /// 删除出库单
        /// </summary>
        /// <param name="stockOutId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool DelStockOutInfo(int stockOutId, UserInfo userInfo)
        {
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_StockOut.StockOutState),(int)StockOutStatusEnum.StockOutCancel},
                { nameof(RB_Supplies_StockOut.Status),(int)DateStateEnum.Delete},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_StockOut.Id),
                        FiledValue=stockOutId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_StockOutRepository.Update(files, wheres);
            if (flag)
            {
                //删除明细 ---  查库存操作
                var list = supplies_StockOutDetailRepository.GetList(new RB_Supplies_StockOutDetail_Extend() { RB_Group_Id = userInfo.RB_Group_id, StockOutId = stockOutId });
                foreach (var item in list)
                {
                    Dictionary<string, object> files1 = new Dictionary<string, object>() {
                        { nameof(RB_Supplies_StockOutDetail.Status),(int)DateStateEnum.Delete}
                    };
                    List<WhereHelper> wheres1 = new List<WhereHelper>() {
                        new WhereHelper(){
                                FiledName=nameof(RB_Supplies_StockOutDetail.Id),
                                FiledValue=item.Id,
                                OperatorEnum=OperatorEnum.Equal
                        }
                    };
                    supplies_StockOutDetailRepository.Update(files1, wheres1);
                }
            }
            return flag;
        }

        /// <summary>
        /// 验证出库单库存是否足够
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool ValidateStockOutInsert(RB_Supplies_StockOut_Extend demodel)
        {
            //更新明细
            var list = demodel.DetailList;
            foreach (var item in list)
            {
                //新增库存表数据
                var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                if (ilist.Any())
                {
                    var iModel = ilist.FirstOrDefault();
                    if (iModel.Number < (item.Number ?? 0))
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// 保存出库单
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetStockOutInfo(RB_Supplies_StockOut_Extend demodel)
        {
            var trans = supplies_StockInRepository.DbTransaction;
            try
            {
                if (demodel.IsSelfApplication == 1)
                {
                    demodel.StockOutState = StockOutStatusEnum.StockOutWait;
                }
                else
                {
                    demodel.StockOutState = StockOutStatusEnum.StockOut;
                }
                if (demodel.Id > 0)
                {
                    var model = supplies_StockOutRepository.GetEntity(demodel.Id);
                    if (demodel.IsSelfApplication == 1 && model.StockOutState != StockOutStatusEnum.StockOutWait)
                    {
                        supplies_StockInRepository.DBSession.Commit();
                        return false;
                    }
                    var delList = supplies_StockOutDetailRepository.GetList(new RB_Supplies_StockOutDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockOutId = demodel.Id });
                    model.EmployeeId = demodel.EmployeeId;
                    model.WarehouseId = demodel.WarehouseId;
                    model.StockOutDate = demodel.StockOutDate;
                    model.Remark = demodel.Remark;
                    model.UpdateBy = demodel.UpdateBy;
                    model.UpdateDate = demodel.UpdateDate;
                    model.StockOutState = StockOutStatusEnum.StockOut;
                    bool flag = supplies_StockOutRepository.Update(model, trans);
                    if (flag)
                    {
                        //删除明细
                        foreach (var item in delList)
                        {
                            supplies_StockOutDetailRepository.Delete(item);
                        }
                        //新增明细
                        foreach (var item in demodel.DetailList)
                        {
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                            decimal Price = 0;
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                Price = Math.Round((iModel.Money ?? 0) / (iModel.Number ?? 1), 2, MidpointRounding.AwayFromZero) * (item.Number ?? 0);
                            }
                            supplies_StockOutDetailRepository.Insert(new RB_Supplies_StockOutDetail()
                            {
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                Number = item.Number,
                                Money = Price,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockOutId = demodel.Id,
                                SuppliesId = item.SuppliesId,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);

                            //新增库存表数据
                            if (ilist.Any() && demodel.StockOutState == StockOutStatusEnum.StockOut)
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number -= item.Number;//减少库存
                                iModel.Money -= Price;
                                if (iModel.Money < 0)
                                {
                                    iModel.Money = 0;
                                }
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = Price,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 2,
                                        Id = 0,
                                        Description = "减少库存-出库"
                                    }, trans);
                                }
                            }
                        }
                    }
                    supplies_StockInRepository.DBSession.Commit();
                    return flag;
                }
                else
                {
                    int Id = supplies_StockOutRepository.Insert(demodel, trans);
                    if (Id > 0)
                    {
                        //编码自动生成
                        string BMStr = Id.ToString();
                        if (BMStr.Length < 5)
                        {
                            switch (BMStr.Length)
                            {
                                case 1:
                                    BMStr = "0000" + BMStr; break;
                                case 2:
                                    BMStr = "000" + BMStr; break;
                                case 3:
                                    BMStr = "00" + BMStr; break;
                                case 4:
                                    BMStr = "0" + BMStr; break;
                            }
                        }
                        BMStr = "CK" + DateTime.Now.ToString("yyyyMMdd") + BMStr;
                        Dictionary<string, object> files = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_StockOut.StockOutNum),BMStr}
                        };
                        List<WhereHelper> wheres = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_Supplies_StockOut.Id),
                                    FiledValue=Id,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                        supplies_StockOutRepository.Update(files, wheres, trans);

                        //新增明细
                        foreach (var item in demodel.DetailList)
                        {
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                            decimal Price = 0;
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                Price = Math.Round((iModel.Money ?? 0) / (iModel.Number ?? 1), 2, MidpointRounding.AwayFromZero) * (item.Number ?? 0);
                            }
                            supplies_StockOutDetailRepository.Insert(new RB_Supplies_StockOutDetail()
                            {
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                Number = item.Number,
                                Money = Price,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockOutId = Id,
                                SuppliesId = item.SuppliesId,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);

                            //新增库存表数据
                            if (ilist.Any() && demodel.StockOutState == StockOutStatusEnum.StockOut)
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number -= item.Number;//减少库存
                                iModel.Money -= Price;
                                if (iModel.Money < 0)
                                {
                                    iModel.Money = 0;
                                }
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = Price,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 2,
                                        Id = 0,
                                        Description = "减少库存-出库"
                                    }, trans);
                                }
                            }
                        }
                    }
                    supplies_StockInRepository.DBSession.Commit();
                    return Id > 0;
                }
            }
            catch (Exception ex)
            {
                supplies_StockInRepository.DBSession.Rollback("SetStockOutInfo");
                LogHelper.Write(ex, "SetStockOutInfo");
                return false;
            }
        }

        /// <summary>
        /// 获取出库分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockOut_Extend> GetStockOutPageList(int pageIndex, int pageSize, RB_Supplies_StockOut_Extend demodel, out long count)
        {
            var list = supplies_StockOutRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            List<RB_Supplies_StockOutDetail_Extend> sdList = new List<RB_Supplies_StockOutDetail_Extend>();
            List<RB_Supplies_Inventory_Extend> iList = new List<RB_Supplies_Inventory_Extend>();
            if (list.Any())
            {
                //查询出所有的耗材详细
                string IdStr = string.Join(",", list.Select(x => x.Id).Distinct());
                sdList = supplies_StockOutDetailRepository.GetList(new RB_Supplies_StockOutDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockOutIdStr = IdStr });
                //获取库存
                if (sdList.Any())
                {
                    string Wstr = string.Join(",", list.Select(x => x.WarehouseId).Distinct());
                    string Sstr = string.Join(",", sdList.Select(x => x.SuppliesId).Distinct());
                    iList = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseIdStr = Wstr, SuppliesIdStr = Sstr });

                    foreach (var qitem in sdList)
                    {
                        qitem.GoodsSpecificationList = new List<string>();
                        qitem.GoodsCategoryList = new List<string>();
                        if (!string.IsNullOrEmpty(qitem.SpecificationName))
                        {
                            qitem.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(qitem.SpecificationName);
                        }
                        if (!string.IsNullOrEmpty(qitem.GoodsCategoryName))
                        {
                            qitem.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(qitem.GoodsCategoryName);
                        }
                    }
                }
            }
            foreach (var item in list)
            {
                item.DetailList = sdList.Where(x => x.StockOutId == item.Id).ToList();
                foreach (var citem in item.DetailList)
                {
                    citem.InventoryNum = iList.Where(x => x.WarehouseId == item.WarehouseId && x.SuppliesId == citem.SuppliesId).FirstOrDefault()?.Number ?? 0;
                }
            }

            return list;
        }

        /// <summary>
        /// 获取入库单详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public RB_Supplies_StockIn GetStockInInfo(int Id)
        {
            return supplies_StockInRepository.GetEntity(Id);
        }

        /// <summary>
        /// 获取出库单详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public RB_Supplies_StockOut GetStockOutInfo(int Id)
        {
            return supplies_StockOutRepository.GetEntity(Id);
        }

        /// <summary>
        /// 恢复入库单
        /// </summary>
        /// <param name="stockInId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool RecoverStockInInfo(int stockInId, UserInfo userInfo)
        {
            var demodel = supplies_StockInRepository.GetEntity(stockInId);
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_StockIn.StockInState),(int)StockInStatusEnum.StockIn},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_StockIn.Id),
                        FiledValue=stockInId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_StockInRepository.Update(files, wheres);
            if (flag)
            {
                //明细 ---  查库存操作
                var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockInId = stockInId });
                foreach (var item in list)
                {
                    //增加库存表数据
                    var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                    if (ilist.Any())
                    {
                        var iModel = ilist.FirstOrDefault();
                        iModel.Number += item.Number;//减少库存
                        iModel.Money += item.Money;
                        var iflag = supplies_InventoryRepository.Update(iModel);
                        if (iflag)
                        {
                            //增加库存明细
                            supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                            {
                                CreateBy = userInfo.EmployeeId,
                                CreateDate = DateTime.Now,
                                InventoryId = iModel.Id,
                                Number = item.Number,
                                Money = item.Money,
                                RB_Branch_Id = userInfo.RB_Branch_id,
                                RB_Group_Id = userInfo.RB_Group_id,
                                Status = 0,
                                Type = 1,
                                Id = 0,
                                Description = "增加库存-恢复入库单"
                            });
                        }
                    }
                }
            }
            return flag;
        }

        /// <summary>
        /// 取消入库单  需要减少库存
        /// </summary>
        /// <param name="stockInId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool CancelStockInInfo(int stockInId, UserInfo userInfo)
        {
            var demodel = supplies_StockInRepository.GetEntity(stockInId);
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_StockIn.StockInState),(int)StockInStatusEnum.StockInWarting},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_StockIn.Id),
                        FiledValue=stockInId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_StockInRepository.Update(files, wheres);
            if (flag)
            {
                //删除明细 ---  查库存操作
                var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = demodel.RB_Group_Id, StockInId = stockInId });
                foreach (var item in list)
                {
                    //减少库存表数据
                    var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                    if (ilist.Any())
                    {
                        var iModel = ilist.FirstOrDefault();
                        iModel.Number -= item.Number;//减少库存
                        iModel.Money -= item.Money;
                        if (iModel.Money < 0)
                        {
                            iModel.Money = 0;
                        }
                        var iflag = supplies_InventoryRepository.Update(iModel);
                        if (iflag)
                        {
                            //增加库存明细
                            supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                            {
                                CreateBy = userInfo.EmployeeId,
                                CreateDate = DateTime.Now,
                                InventoryId = iModel.Id,
                                Number = item.Number,
                                Money = item.Money,
                                RB_Branch_Id = userInfo.RB_Branch_id,
                                RB_Group_Id = userInfo.RB_Group_id,
                                Status = 0,
                                Type = 2,
                                Id = 0,
                                Description = "减少库存-取消入库单"
                            });
                        }
                    }
                }
            }
            return flag;
        }

        /// <summary>
        /// 删除入库单
        /// </summary>
        /// <param name="stockInId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool DelStockInInfo(int stockInId, UserInfo userInfo)
        {
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_StockIn.StockInState),(int)StockInStatusEnum.StockInWarting},
                { nameof(RB_Supplies_StockIn.Status),(int)DateStateEnum.Delete},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                        FiledName=nameof(RB_Supplies_StockIn.Id),
                        FiledValue=stockInId,
                        OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_StockInRepository.Update(files, wheres);
            if (flag)
            {
                //删除明细 ---  查库存操作
                var list = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = userInfo.RB_Group_id, StockInId = stockInId });
                foreach (var item in list)
                {
                    Dictionary<string, object> files1 = new Dictionary<string, object>() {
                        { nameof(RB_Supplies_StockInDetail.Status),(int)DateStateEnum.Delete}
                    };
                    List<WhereHelper> wheres1 = new List<WhereHelper>() {
                        new WhereHelper(){
                                FiledName=nameof(RB_Supplies_StockInDetail.Id),
                                FiledValue=item.Id,
                                OperatorEnum=OperatorEnum.Equal
                        }
                    };
                    supplies_StockInDetailRepository.Update(files1, wheres1);
                }
            }
            return flag;
        }

        #endregion

        #region 耗材申请日期设置
        /// <summary>
        /// 耗材申请设置分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_Applyfor_Extend> GetApplyForSetPageList(int pageIndex, int pageSize, RB_Supplies_Applyfor_Extend demodel, out long count)
        {
            var list = supplies_ApplyforRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            var slist = new List<RB_Supplies_ApplyforWeekday_Extend>();
            var aslist = new List<RB_Supplies_ApplyforSpecial_Extend>();
            if (list.Any())
            {
                slist = supplies_ApplyforWeekdayRepository.GetList(new RB_Supplies_ApplyforWeekday_Extend() { RB_Group_Id = demodel.RB_Group_Id, ApplyforIds = string.Join(",", list.Select(x => x.Id).Distinct()) });
                aslist = supplies_ApplyforSpecialRepository.GetList(new RB_Supplies_ApplyforSpecial_Extend() { ApplyforIds = string.Join(",", list.Select(x => x.Id).Distinct()) });
            }
            foreach (var item in list)
            {
                //查询所有周期
                item.WeekdayList = slist.Where(x => x.ApplyforId == item.Id).ToList();
                item.SpecialList = aslist.Where(x => x.ApplyforId == item.Id).ToList();
            }
            return list;
        }

        /// <summary>
        /// 设置耗材申请时间
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetApplyForInfo(RB_Supplies_Applyfor_Extend demodel)
        {
            var trans = supplies_ApplyforRepository.DbTransaction;
            try
            {
                if (demodel.Id > 0)
                {
                    Dictionary<string, object> files = new Dictionary<string, object>() {
                        { nameof(RB_Supplies_Applyfor_Extend.BranchId),demodel.BranchId}
                    };
                    List<WhereHelper> wheres = new List<WhereHelper>() {
                     new WhereHelper(){
                      FiledName=nameof(RB_Supplies_Applyfor_Extend.Id),
                       FiledValue=demodel.Id,
                        OperatorEnum=OperatorEnum.Equal
                     }
                    };
                    bool flag = supplies_ApplyforRepository.Update(files, wheres, trans);
                    if (flag)
                    {
                        var slist = supplies_ApplyforWeekdayRepository.GetList(new RB_Supplies_ApplyforWeekday_Extend() { RB_Group_Id = demodel.RB_Group_Id, ApplyforId = demodel.Id });
                        var aslist = supplies_ApplyforSpecialRepository.GetList(new RB_Supplies_ApplyforSpecial_Extend() { ApplyforId = demodel.Id });
                        //删除所有工作日信息
                        foreach (var item in slist)
                        {
                            supplies_ApplyforWeekdayRepository.Delete(item, trans);
                        }
                        //再新增
                        foreach (var item in demodel.WeekdayList)
                        {
                            supplies_ApplyforWeekdayRepository.Insert(new RB_Supplies_ApplyforWeekday()
                            {
                                ApplyforId = demodel.Id,
                                Id = 0,
                                Weekday = item.Weekday,
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                EndTime = item.EndTime,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                RB_Group_Id = demodel.RB_Group_Id,
                                StartTime = item.StartTime,
                                Status = 0,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);
                        }
                        //特殊日期
                        foreach (var item in aslist)
                        {
                            supplies_ApplyforSpecialRepository.Delete(item, trans);
                        }
                        //写入特殊日期
                        if (demodel.SpecialList != null && demodel.SpecialList.Any())
                        {
                            foreach (var item in demodel.SpecialList)
                            {
                                supplies_ApplyforSpecialRepository.Insert(new RB_Supplies_ApplyforSpecial()
                                {
                                    ApplyforId = demodel.Id,
                                    Id = 0,
                                    CreateBy = demodel.CreateBy,
                                    CreateDate = demodel.CreateDate,
                                    Date = item.Date,
                                    EndTime = item.EndTime,
                                    StartTime = item.StartTime,
                                    Status = 0,
                                    UpdateBy = demodel.UpdateBy,
                                    UpdateDate = demodel.UpdateDate
                                }, trans);
                            }
                        }
                    }
                }
                else
                {
                    //新增
                    demodel.Status = 0;
                    demodel.Enabled = 1;//默认启用
                    int Id = supplies_ApplyforRepository.Insert(demodel, trans);
                    if (Id > 0)
                    {
                        //写入工作日
                        foreach (var item in demodel.WeekdayList)
                        {
                            supplies_ApplyforWeekdayRepository.Insert(new RB_Supplies_ApplyforWeekday()
                            {
                                ApplyforId = Id,
                                Id = 0,
                                Weekday = item.Weekday,
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                EndTime = item.EndTime,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                RB_Group_Id = demodel.RB_Group_Id,
                                StartTime = item.StartTime,
                                Status = 0,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);
                        }
                        //写入特殊日期
                        if (demodel.SpecialList != null && demodel.SpecialList.Any())
                        {
                            foreach (var item in demodel.SpecialList)
                            {
                                supplies_ApplyforSpecialRepository.Insert(new RB_Supplies_ApplyforSpecial()
                                {
                                    ApplyforId = Id,
                                    Id = 0,
                                    CreateBy = demodel.CreateBy,
                                    CreateDate = demodel.CreateDate,
                                    Date = item.Date,
                                    EndTime = item.EndTime,
                                    StartTime = item.StartTime,
                                    Status = 0,
                                    UpdateBy = demodel.UpdateBy,
                                    UpdateDate = demodel.UpdateDate
                                }, trans);
                            }
                        }
                    }
                }
                supplies_ApplyforRepository.DBSession.Commit();
                return true;
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "SetApplyForInfo");
                supplies_ApplyforRepository.DBSession.Rollback();
                return false;
            }
        }

        /// <summary>
        /// 验证公司别
        /// </summary>
        /// <param name="demodel"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool ValidateBranch(RB_Supplies_Applyfor_Extend demodel, UserInfo userInfo)
        {
            var list = supplies_ApplyforRepository.GetList(new RB_Supplies_Applyfor_Extend() { RB_Group_Id = userInfo.RB_Group_id, BranchId = demodel.BranchId });
            if (demodel.Id > 0)
            {
                if (list.Where(x => x.Id != demodel.Id).Any())
                {
                    return true;
                }
            }
            else
            {
                if (list.Any())
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// 设置状态
        /// </summary>
        /// <param name="applyforId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool SetApplyForStatus(int applyforId, UserInfo userInfo)
        {
            var model = supplies_ApplyforRepository.GetEntity(applyforId);
            if (model == null)
            {
                return false;
            }
            if (model.Status == 1)
            {
                return false;
            }
            Dictionary<string, object> files = new Dictionary<string, object>() {
                { nameof(RB_Supplies_Applyfor_Extend.Enabled),model.Enabled==1?2:1},
                { nameof(RB_Supplies_Applyfor_Extend.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_Applyfor_Extend.UpdateDate),DateTime.Now}
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                FiledName=nameof(RB_Supplies_Applyfor_Extend.Id),
                FiledValue=applyforId,
                OperatorEnum=OperatorEnum.Equal
                }
            };
            return supplies_ApplyforRepository.Update(files, wheres);
        }

        /// <summary>
        /// 获取详情
        /// </summary>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public RB_Supplies_Applyfor_Extend GetApplyForInfo(UserInfo userInfo)
        {
            var list = supplies_ApplyforRepository.GetList(new RB_Supplies_Applyfor_Extend() { BranchId = userInfo.RB_Branch_id, RB_Group_Id = userInfo.RB_Group_id });
            if (list.Any())
            {
                var demodel = list.FirstOrDefault();
                var slist = supplies_ApplyforWeekdayRepository.GetList(new RB_Supplies_ApplyforWeekday_Extend() { RB_Group_Id = demodel.RB_Group_Id, ApplyforId = demodel.Id });
                var aslist = supplies_ApplyforSpecialRepository.GetList(new RB_Supplies_ApplyforSpecial_Extend() { ApplyforId = demodel.Id });
                demodel.WeekdayList = slist;
                demodel.SpecialList = aslist;
                return demodel;
            }
            else
            {
                //查询通用化的
                list = supplies_ApplyforRepository.GetList(new RB_Supplies_Applyfor_Extend() { BranchId = -1, RB_Group_Id = userInfo.RB_Group_id });
                if (list.Any())
                {
                    var demodel = list.FirstOrDefault();
                    var slist = supplies_ApplyforWeekdayRepository.GetList(new RB_Supplies_ApplyforWeekday_Extend() { RB_Group_Id = demodel.RB_Group_Id, ApplyforId = demodel.Id });
                    var aslist = supplies_ApplyforSpecialRepository.GetList(new RB_Supplies_ApplyforSpecial_Extend() { ApplyforId = demodel.Id });
                    demodel.WeekdayList = slist;
                    demodel.SpecialList = aslist;
                    return demodel;
                }
                else
                {
                    return new RB_Supplies_Applyfor_Extend() { Id = -1 };
                }
            }
        }

        #endregion

        #region 采购单


        /// <summary>
        /// 获取采购单分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Procurement_Extend> GetProcurementPageList(int pageIndex, int pageSize, RB_Procurement_Extend demodel, out long count)
        {
            var list = procurementRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            if (list.Any())
            {
                string procurementIds = string.Join(",", list.Select(x => x.Id));
                //查询明细
                var pdList = procurement_DetailRepository.GetList(new RB_Procurement_Detail_Extend() { RB_Group_Id = demodel.RB_Group_Id, ProcurementIds = procurementIds });
                //查询物料
                if (pdList.Any())
                {
                    string MaterialIds = string.Join(",", pdList.Select(x => x.MaterialId).Distinct());
                    var mList = supplies_MaterialRepository.GetList(new RB_Supplies_Material_Extend() { RB_Group_Id = demodel.RB_Group_Id, SuppliesIdStr = MaterialIds });
                    foreach (var item in pdList)
                    {
                        item.MaterialModel = mList.Where(x => x.Id == item.MaterialId).FirstOrDefault();
                        if (item.MaterialModel != null)
                        {
                            item.MaterialModel.GoodsSpecificationList = new List<string>();
                            item.MaterialModel.GoodsCategoryList = new List<string>();
                            if (!string.IsNullOrEmpty(item.MaterialModel.SpecificationName))
                            {
                                item.MaterialModel.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.SpecificationName);
                            }
                            if (!string.IsNullOrEmpty(item.MaterialModel.GoodsCategoryName))
                            {
                                item.MaterialModel.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.GoodsCategoryName);
                            }
                            if (item.MaterialModel.Images == "" || item.MaterialModel.Images == null)
                            {
                                item.MaterialModel.Images = "[]";
                            }
                            item.MaterialModel.ImageList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.Images);
                        }
                        else
                        {
                            item.MaterialModel = new RB_Supplies_Material_Extend();
                        }
                    }
                }

                var wlist = supplies_WareHouseRepository.GetList(new RB_Supplies_WareHouse_Extend() { RB_Group_Id = demodel.RB_Group_Id, WareHouseIdStr = string.Join(",", list.Select(x => x.WareHouseId ?? 0).Distinct()) });
                foreach (var item in list)
                {
                    item.DetailList = pdList.Where(x => x.ProcurementId == item.Id).ToList();
                    item.WareHouseName = wlist.Where(x => x.Id == item.WareHouseId).FirstOrDefault()?.Name ?? "";
                }
            }
            return list;
        }

        /// <summary>
        /// 获取采购单详情
        /// </summary>
        /// <param name="procurementId"></param>
        /// <param name="TenantId"></param>
        /// <param name="MallBaseId"></param>
        /// <returns></returns>
        public RB_Procurement_Extend GetProcurementInfo(int procurementId, int ERPGroupId)
        {
            var pmodel = procurementRepository.GetEntity<RB_Procurement_Extend>(procurementId);
            //查询明细
            var pdList = procurement_DetailRepository.GetList(new RB_Procurement_Detail_Extend() { RB_Group_Id = ERPGroupId, ProcurementId = procurementId });
            //查询物料
            if (pdList.Any())
            {
                string MaterialIds = string.Join(",", pdList.Select(x => x.MaterialId).Distinct());
                var mList = supplies_MaterialRepository.GetList(new RB_Supplies_Material_Extend() { RB_Group_Id = ERPGroupId, SuppliesIdStr = MaterialIds });
                foreach (var item in pdList)
                {
                    item.MaterialModel = mList.Where(x => x.Id == item.MaterialId).FirstOrDefault();
                    if (item.MaterialModel != null)
                    {
                        item.MaterialModel.GoodsSpecificationList = new List<string>();
                        item.MaterialModel.GoodsCategoryList = new List<string>();
                        if (!string.IsNullOrEmpty(item.MaterialModel.SpecificationName))
                        {
                            item.MaterialModel.GoodsSpecificationList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.SpecificationName);
                        }
                        if (!string.IsNullOrEmpty(item.MaterialModel.GoodsCategoryName))
                        {
                            item.MaterialModel.GoodsCategoryList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.GoodsCategoryName);
                        }
                        if (item.MaterialModel.Images == "" || item.MaterialModel.Images == null)
                        {
                            item.MaterialModel.Images = "[]";
                        }
                        item.MaterialModel.ImageList = JsonConvert.DeserializeObject<List<string>>(item.MaterialModel.Images);
                    }
                    else
                    {
                        item.MaterialModel = new RB_Supplies_Material_Extend();
                    }
                }
            }
            var wlist = supplies_WareHouseRepository.GetList(new RB_Supplies_WareHouse_Extend() { RB_Group_Id = ERPGroupId, WareHouseIdStr = (pmodel.WareHouseId ?? 0).ToString() });

            pmodel.DetailList = pdList;
            pmodel.WareHouseName = wlist.FirstOrDefault()?.Name ?? "";

            return pmodel;
        }

        /// <summary>
        /// 获取扫码采购商品信息
        /// </summary>
        /// <param name="code"></param>
        /// <param name="wareHouseId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public object GetProcurementGoodsInfo(string code, int wareHouseId, UserInfo userInfo)
        {
            code = code.Replace("A", "-");
            string[] codeArr = code.Split('-');
            if (codeArr.Length != 5)
            {
                return new
                {
                    Status = 1,
                    Msg = "编码格式不正确"
                };
            }
            if (codeArr[0] != "D")
            {
                return new
                {
                    Status = 1,
                    Msg = "编码格式不正确"
                };
            }
            int WarehouseId = Convert.ToInt32(codeArr[1]);
            int ProcurementId = Convert.ToInt32(codeArr[2]);
            int SuppliesId = Convert.ToInt32(codeArr[3]);
            int StockInId = Convert.ToInt32(codeArr[4]);
            if (WarehouseId != wareHouseId)
            {
                return new
                {
                    Status = 1,
                    Msg = "商品仓库不正确"
                };
            }

            var materialModel = supplies_MaterialRepository.GetEntity(SuppliesId);
            if (materialModel == null)
            {
                return new
                {
                    Status = 1,
                    Msg = "物料档案不存在"
                };
            }
            List<string> SpecificationList = new List<string>();
            List<string> ImageList = new List<string>();
            if (!string.IsNullOrEmpty(materialModel.SpecificationName) && materialModel.SpecificationName != "[]")
            {
                SpecificationList = JsonConvert.DeserializeObject<List<string>>(materialModel.SpecificationName);
            }
            if (!string.IsNullOrEmpty(materialModel.Images) && materialModel.Images != "[]")
            {
                ImageList = JsonConvert.DeserializeObject<List<string>>(materialModel.Images);
            }

            #region 库存验证
            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = userInfo.RB_Group_id, WarehouseId = WarehouseId, SuppliesId = SuppliesId });
            if (ilist.Any())
            {
                var iModel = ilist.FirstOrDefault();
                if (iModel.Number <= 0)
                {
                    return new
                    {
                        Status = 1,
                        Msg = "该物料库存不足"
                    };
                }
            }
            #endregion

            var stockInList = supplies_StockInRepository.GetList(new RB_Supplies_StockIn_Extend() { RB_Group_Id = userInfo.RB_Group_id, Id = StockInId, ProcurementId = ProcurementId });
            if (!stockInList.Any())
            {
                return new
                {
                    Status = 1,
                    Msg = "该商品未入库"
                };
            }
            var stockInModel = stockInList.FirstOrDefault();
            var stockInDetailList = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = userInfo.RB_Group_id, SuppliesId = SuppliesId, StockInId = StockInId });
            if (!stockInDetailList.Any())
            {
                return new
                {
                    Status = 1,
                    Msg = "该商品未入库"
                };
            }
            var detModel = stockInDetailList.FirstOrDefault();


            return new
            {
                Status = 0,
                Msg = "查询成功",
                GoodsModel = new
                {
                    WarehouseId,
                    SuppliesId,
                    materialModel.GoodsId,
                    materialModel.Name,
                    materialModel.SuppliesNum,
                    materialModel.SpecificationKey,
                    SpecificationList,
                    ImageList,
                    materialModel.Units,
                    StockInId,
                    stockInModel.StockInNum,
                    StockInDate = stockInModel.StockInDate.HasValue ? stockInModel.StockInDate.Value.ToString("yyyy-MM-dd") : "",
                    detModel.UnitPrice
                }
            };
        }

        /// <summary>
        /// 获取入库单的标签
        /// </summary>
        /// <param name="procurementId"></param>
        /// <param name="StockInId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public object GetProcurementStockInLableList(int procurementId, int StockInId, UserInfo userInfo)
        {
            List<object> RObj = new List<object>();
            var pmodel = procurementRepository.GetEntity(procurementId);
            int eRPGroupId = userInfo.RB_Group_id;
            var siList = supplies_StockInRepository.GetList(new RB_Supplies_StockIn_Extend() { RB_Group_Id = eRPGroupId, ProcurementId = procurementId, Id = StockInId });
            if (siList.Any())
            {
                string siIds = string.Join(",", siList.Where(x => x.StockInState == StockInStatusEnum.StockIn).Select(x => x.Id));
                var sidList = supplies_StockInDetailRepository.GetList(new RB_Supplies_StockInDetail_Extend() { RB_Group_Id = eRPGroupId, StockInIdStr = siIds });
                List<RB_Supplies_Material_Extend> materialList = new List<RB_Supplies_Material_Extend>();
                if (sidList.Any())
                {
                    string matIds = string.Join(",", sidList.Select(x => x.SuppliesId).Distinct());
                    materialList = supplies_MaterialRepository.GetList(new RB_Supplies_Material_Extend() { RB_Group_Id = eRPGroupId, SuppliesIdStr = matIds });
                }
                foreach (var item in sidList)
                {

                    var siModel = siList.Where(x => x.Id == item.StockInId).FirstOrDefault();
                    var materialModel = materialList.Where(x => x.Id == item.SuppliesId).FirstOrDefault();
                    if (materialModel == null) { continue; }

                    List<string> spList = new List<string>();
                    if (!string.IsNullOrEmpty(materialModel.SpecificationName) && materialModel.SpecificationName != "[]")
                    {
                        spList = JsonConvert.DeserializeObject<List<string>>(materialModel.SpecificationName);
                    }

                    string QRCode = "D-" + (siModel.WarehouseId ?? 0) + "-" + procurementId + "-" + (item.SuppliesId ?? 0) + "-" + (item.StockInId ?? 0);
                    string TQRCode = "DA" + (siModel.WarehouseId ?? 0) + "A" + procurementId + "A" + (item.SuppliesId ?? 0) + "A" + (item.StockInId ?? 0);
                    #region 生成二维码 + 条形码
                    string QRName = "QR-" + DateTime.Now.ToString("yyyyMMdd") + "-" + procurementId + "-" + (item.SuppliesId ?? 0) + ".jpg";
                    string TQRName = "TQR-" + DateTime.Now.ToString("yyyyMMdd") + "-" + procurementId + "-" + (item.SuppliesId ?? 0) + ".jpg";
                    string basepath = AppContext.BaseDirectory;
                    string QRPath = basepath + "\\upfile\\qrcode\\" + QRName;
                    string QRImage = "/upfile/qrcode/" + QRName;
                    string TQRPath = basepath + "\\upfile\\qrcode\\" + TQRName;
                    string TQRImage = "/upfile/qrcode/" + TQRName;
                    //验证文件是否存在，  存在的话就不用生成了
                    if (!File.Exists(QRPath))
                    {
                        QRCodeHelper.CreateQRCode(QRCode, 5, "", QRPath, System.Drawing.Imaging.ImageFormat.Jpeg);
                    }
                    if (!File.Exists(TQRPath))
                    {
                        int width = 340;
                        if (TQRCode.Length > 30)
                        {
                            width = 370;
                        }
                        QRCodeHelper.CreateTQRCode(TQRCode, TQRPath, System.Drawing.Imaging.ImageFormat.Jpeg, width, 50);
                    }
                    #endregion

                    var empModel = CacheManager.User.UserReidsCache.GetEmployee(item.CreateBy);
                    for (int i = 0; i < (item.Number ?? 0); i++)
                    {
                        RObj.Add(new
                        {
                            materialModel.GoodsId,
                            GoodsName = materialModel.Name,
                            SpecificationList = spList,
                            CreateDate = item.CreateDate.HasValue ? item.CreateDate.Value.ToString("yyyy-MM-dd") : "",
                            EmName = empModel?.EmName ?? "",
                            siModel.WareHouseName,
                            CostMoney = item.UnitPrice,
                            QRImage,
                            TQRImage,
                            TQRCode
                        });
                    }
                }

            }
            return RObj;
        }

        /// <summary>
        /// 获取采购入库批次列表
        /// </summary>
        /// <param name="procurementId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public object GetProcurementSotckInBatchList(int procurementId, UserInfo userInfo)
        {
            List<object> RObj = new List<object>();
            var siList = supplies_StockInRepository.GetList(new RB_Supplies_StockIn_Extend() { RB_Group_Id = userInfo.RB_Group_id, ProcurementId = procurementId });
            siList = siList.Where(x => x.StockInState == StockInStatusEnum.StockIn).ToList();
            return siList.Select(x => new
            {
                StockInId = x.Id,
                x.StockInNum
            });
        }

        #endregion

        #region 盘点

        /// <summary>
        /// 获取盘点单分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_Check_Extend> GetSuppliesCheckPageList(int pageIndex, int pageSize, RB_Supplies_Check_Extend demodel, out long count)
        {
            var list = supplies_CheckRepository.GetPageList(pageIndex, pageSize, demodel, out count);
            if (list.Any())
            {
                //查询仓库
                var wlist = supplies_WareHouseRepository.GetList(new RB_Supplies_WareHouse_Extend() { RB_Group_Id = demodel.RB_Group_Id, WareHouseIdStr = string.Join(",", list.Select(x => x.WarehouseId ?? 0).Distinct()) });
                foreach (var item in list)
                {
                    var wmodel = wlist.Where(x => x.Id == item.WarehouseId).FirstOrDefault();
                    item.WarehouseName = wmodel?.Name ?? "";
                }
            }
            return list;
        }

        /// <summary>
        /// 根据条件获取盘点列表
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_Check_Extend> GetSuppliesCheckList(RB_Supplies_Check_Extend dmodel)
        {
            return supplies_CheckRepository.GetList(dmodel);
        }


        /// <summary>
        /// 新增修改盘点单
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetSuppliesCheck(RB_Supplies_Check_Extend demodel)
        {
            var trans = supplies_CheckRepository.DbTransaction;
            try
            {
                if (demodel.Id > 0)
                {
                    var model = supplies_CheckRepository.GetEntity(demodel.Id);

                    model.StartDate = demodel.StartDate;
                    model.EndDate = demodel.EndDate;
                    model.UpdateBy = demodel.UpdateBy;
                    model.UpdateDate = demodel.UpdateDate;
                    var flag = supplies_CheckRepository.Update(model, trans);
                    supplies_CheckRepository.DBSession.Commit();
                    return true;
                }
                else
                {
                    demodel.Periods = DateTime.Now.ToString("yyyyMMdd");
                    int Id = supplies_CheckRepository.Insert(demodel, trans);
                    if (Id > 0)
                    {
                        //根据仓库  查询出所有物料的库存
                        var list = supplies_MaterialRepository.GetSuppliesCheckMaterialList(demodel.WarehouseId, demodel.RB_Group_Id);
                        foreach (var item in list)
                        {
                            supplies_CheckDetailRepository.Insert(new RB_Supplies_CheckDetail()
                            {
                                CheckStatus = PropertyCheckStatusEnum.NoInventory,
                                CheckDate = null,
                                CheckEmpId = 0,
                                CheckId = Id,
                                Id = 0,
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                SuppliesId = item.Id,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = "",
                                Status = 0,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate,
                                DealStatus = 2,
                                FinanceId = 0,
                                Images = "",
                                InventoryNum = item.InventoryNum,
                                RealityNum = 0
                            }, trans);
                        }
                    }
                    supplies_CheckRepository.DBSession.Commit();
                    return true;
                }
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "SetSuppliesCheck");
                supplies_CheckRepository.DBSession.Rollback();
                return false;
            }
        }

        /// <summary>
        /// 获取盘点单
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public RB_Supplies_Check_Extend GetSuppliesCheck(int Id)
        {
            var model = supplies_CheckRepository.GetEntity<RB_Supplies_Check_Extend>(Id);
            if (model != null)
            {
                model.WarehouseName = supplies_WareHouseRepository.GetEntity(model.WarehouseId ?? 0)?.Name ?? "";
            }
            return model;
        }

        /// <summary>
        /// 取消盘点
        /// </summary>
        /// <param name="checkId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool CancelSuppliesCheck(int checkId, UserInfo userInfo)
        {
            Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                { nameof(RB_Supplies_Check_Extend.CheckState),SuppliesCheckStatusEnum.Cancel},
                { nameof(RB_Supplies_Check_Extend.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_Check_Extend.UpdateDate),DateTime.Now},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                     FiledName=nameof(RB_Supplies_Check_Extend.Id),
                      FiledValue=checkId,
                       OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_CheckRepository.Update(keyValues, wheres);
            return flag;
        }

        /// <summary>
        /// 盘点单复盘
        /// </summary>
        /// <param name="checkId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool RepeatSuppliesCheck(int checkId, string StartTime, string EndTime, UserInfo userInfo)
        {
            Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                { nameof(RB_Supplies_Check_Extend.CheckState),SuppliesCheckStatusEnum.InTheInventory},
                { nameof(RB_Supplies_Check_Extend.StartDate),Convert.ToDateTime(StartTime)},
                { nameof(RB_Supplies_Check_Extend.EndDate),Convert.ToDateTime(EndTime)},
                { nameof(RB_Supplies_Check_Extend.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_Check_Extend.UpdateDate),DateTime.Now},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                     FiledName=nameof(RB_Supplies_Check_Extend.Id),
                      FiledValue=checkId,
                       OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_CheckRepository.Update(keyValues, wheres);
            if (flag)
            {
                //是否需要重置盘点明细的盘点状态

            }
            return flag;
        }

        /// <summary>
        /// 获取盘点单明细分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_CheckDetail_Extend> GetSuppliesCheckDetailPageList(int pageIndex, int pageSize, RB_Supplies_CheckDetail_Extend demodel, out long count)
        {
            var list = supplies_CheckDetailRepository.GetPageList(pageIndex, pageSize, demodel, out count);

            if (list.Any())
            {
                //组装规格
                foreach (var item in list)
                {
                    item.SpecificationList = new List<string>();
                    if (!string.IsNullOrEmpty(item.SpecificationName) && item.SpecificationName != "[]")
                    {
                        item.SpecificationList = JsonConvert.DeserializeObject<List<string>>(item.SpecificationName);
                    }
                    item.ImageList = new List<string>();
                    if (!string.IsNullOrEmpty(item.Images) && item.Images != "[]")
                    {
                        item.ImageList = JsonConvert.DeserializeObject<List<string>>(item.Images);
                    }
                }
            }
            return list;
        }

        /// <summary>
        /// 获取盘点报损/报溢列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="demodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_CheckDetail_Extend> GetSuppliesCheckDetailBreakagePageList(int pageIndex, int pageSize, RB_Supplies_CheckDetail_Extend demodel, out long count)
        {
            var cmodel = supplies_CheckRepository.GetEntity(demodel.CheckId);
            if (cmodel == null)
            {
                count = 0;
                return new List<RB_Supplies_CheckDetail_Extend>();
            }
            var list = supplies_CheckDetailRepository.GetPageList(pageIndex, pageSize, demodel, out count);

            if (list.Any())
            {
                //组装规格
                foreach (var item in list)
                {
                    item.SpecificationList = new List<string>();
                    if (!string.IsNullOrEmpty(item.SpecificationName) && item.SpecificationName != "[]")
                    {
                        item.SpecificationList = JsonConvert.DeserializeObject<List<string>>(item.SpecificationName);
                    }
                    item.ImageList = new List<string>();
                    if (!string.IsNullOrEmpty(item.Images) && item.Images != "[]")
                    {
                        item.ImageList = JsonConvert.DeserializeObject<List<string>>(item.Images);
                    }
                    //查询最新入库价格
                    item.Unit_Money = supplies_CheckDetailRepository.GetSuppliesNewPrice(cmodel.WarehouseId ?? 0, item.SuppliesId ?? 0);
                }
            }
            return list;
        }

        /// <summary>
        /// 验证盘点单状态
        /// </summary>
        /// <param name="checkDetailId"></param>
        /// <returns></returns>
        public bool ValidataSuppliesCheckDetail(int checkDetailId)
        {
            var model = supplies_CheckDetailRepository.GetEntity(checkDetailId);
            var cmodel = supplies_CheckRepository.GetEntity(model?.CheckId ?? 0);
            if (cmodel != null && cmodel.CheckState == SuppliesCheckStatusEnum.InTheInventory)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 获取盘点明细
        /// </summary>
        /// <param name="checkDetailId"></param>
        /// <returns></returns>
        public RB_Supplies_CheckDetail_Extend GetSuppliesCheckDetailInfo(int checkDetailId)
        {
            return supplies_CheckDetailRepository.GetEntity<RB_Supplies_CheckDetail_Extend>(checkDetailId);
        }
        /// <summary>
        /// 根据条件获取盘点列表
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_CheckDetail_Extend> GetSuppliesCheckDetailList(RB_Supplies_CheckDetail_Extend dmodel)
        {
            return supplies_CheckDetailRepository.GetList(dmodel);
        }



        /// <summary>
        /// 设置盘点明细 耗材实盘数量
        /// </summary>
        /// <param name="checkDetailId"></param>
        /// <param name="type"></param>
        /// <param name="reality"></param>
        /// <param name="imagesList"></param>
        /// <param name="remark"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool SetSuppliesCheckDetailReality(int checkDetailId, int type, int reality, List<string> imagesList, string remark, UserInfo userInfo)
        {
            var dmodel = supplies_CheckDetailRepository.GetEntity(checkDetailId);
            if (dmodel == null) { return false; }
            if (type == 1)
            {
                if (dmodel.CheckStatus != PropertyCheckStatusEnum.NoInventory && dmodel.CheckStatus != PropertyCheckStatusEnum.HaveInventory && dmodel.DealStatus == 1)
                {
                    return false;
                }
                //更新实际数量
                PropertyCheckStatusEnum pcs = PropertyCheckStatusEnum.HaveInventory;
                if (reality == (dmodel.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.HaveInventory;
                }
                else if (reality > (dmodel.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.InventoryProfit;
                }
                else if (reality < (dmodel.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.InventoryLosses;
                }

                Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                    { nameof(RB_Supplies_CheckDetail.RealityNum),reality},
                    { nameof(RB_Supplies_CheckDetail.CheckStatus),pcs},
                    { nameof(RB_Supplies_CheckDetail.CheckEmpId),userInfo.EmployeeId},
                    { nameof(RB_Supplies_CheckDetail.CheckDate),DateTime.Now},
                };
                List<WhereHelper> wheres = new List<WhereHelper>() {
                    new WhereHelper(){
                         FiledName=nameof(RB_Supplies_CheckDetail.Id),
                          FiledValue=dmodel.Id,
                           OperatorEnum=OperatorEnum.Equal
                    }
                };
                return supplies_CheckDetailRepository.Update(keyValues, wheres);
            }
            else if (type == 2)
            {
                //上传图片
                Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                    { nameof(RB_Supplies_CheckDetail.Images),JsonConvert.SerializeObject(imagesList)},
                };
                List<WhereHelper> wheres = new List<WhereHelper>() {
                    new WhereHelper(){
                         FiledName=nameof(RB_Supplies_CheckDetail.Id),
                          FiledValue=dmodel.Id,
                           OperatorEnum=OperatorEnum.Equal
                    }
                };
                return supplies_CheckDetailRepository.Update(keyValues, wheres);
            }
            else if (type == 3)
            {
                //更新备注
                Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                    { nameof(RB_Supplies_CheckDetail.Remark),remark},
                };
                List<WhereHelper> wheres = new List<WhereHelper>() {
                    new WhereHelper(){
                         FiledName=nameof(RB_Supplies_CheckDetail.Id),
                          FiledValue=dmodel.Id,
                           OperatorEnum=OperatorEnum.Equal
                    }
                };
                return supplies_CheckDetailRepository.Update(keyValues, wheres);
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 批量设置盘点
        /// </summary>
        /// <param name="list"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool SetBatchSuppliesCheckDetailReality(List<RB_Supplies_CheckDetail_Extend> list, UserInfo userInfo, int Type = 1)
        {
            string checkdetailIds = string.Join(",", list.Select(x => x.Id));
            var dlist = supplies_CheckDetailRepository.GetList(new RB_Supplies_CheckDetail_Extend() { RB_Group_Id = userInfo.RB_Group_id, CheckDetailIds = checkdetailIds });
            if (dlist.Select(x => x.CheckId).Distinct().Count() != 1)
            {
                return false;
            }
            foreach (var item in dlist)
            {
                int reality = list.Where(x => x.Id == item.Id).FirstOrDefault()?.RealityNum ?? 0;
                if (item.CheckStatus != PropertyCheckStatusEnum.NoInventory && item.CheckStatus != PropertyCheckStatusEnum.HaveInventory && item.DealStatus == 1)
                {
                    continue;
                }
                //更新实际数量
                PropertyCheckStatusEnum pcs = PropertyCheckStatusEnum.HaveInventory;
                if (reality == (item.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.HaveInventory;
                }
                else if (reality > (item.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.InventoryProfit;
                }
                else if (reality < (item.InventoryNum ?? 0))
                {
                    pcs = PropertyCheckStatusEnum.InventoryLosses;
                }

                Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                    { nameof(RB_Supplies_CheckDetail.RealityNum),reality},
                    { nameof(RB_Supplies_CheckDetail.CheckStatus),pcs},
                    { nameof(RB_Supplies_CheckDetail.CheckEmpId),userInfo.EmployeeId},
                    { nameof(RB_Supplies_CheckDetail.CheckDate),DateTime.Now},
                };
                if (Type == 2)
                {
                    if (!string.IsNullOrEmpty(item.Remark))
                    {
                        keyValues.Add(nameof(RB_Supplies_CheckDetail.Remark), item.Remark);
                    }
                }
                List<WhereHelper> wheres = new List<WhereHelper>() {
                    new WhereHelper(){
                         FiledName=nameof(RB_Supplies_CheckDetail.Id),
                          FiledValue=item.Id,
                           OperatorEnum=OperatorEnum.Equal
                    }
                };
                supplies_CheckDetailRepository.Update(keyValues, wheres);
            }
            return true;
        }

        /// <summary>
        /// 设置盘点单完成
        /// </summary>
        /// <param name="checkId"></param>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public bool SetSuppliesCheckComplete(int checkId, UserInfo userInfo)
        {
            Dictionary<string, object> keyValues = new Dictionary<string, object>() {
                { nameof(RB_Supplies_Check_Extend.CheckState),SuppliesCheckStatusEnum.Completed},
                { nameof(RB_Supplies_Check_Extend.UpdateBy),userInfo.EmployeeId},
                { nameof(RB_Supplies_Check_Extend.UpdateDate),DateTime.Now},
            };
            List<WhereHelper> wheres = new List<WhereHelper>() {
                new WhereHelper(){
                     FiledName=nameof(RB_Supplies_Check_Extend.Id),
                      FiledValue=checkId,
                       OperatorEnum=OperatorEnum.Equal
                }
            };
            bool flag = supplies_CheckRepository.Update(keyValues, wheres);
            if (flag)
            {
                //未盘的  直接盘亏处理
                var list = supplies_CheckDetailRepository.GetList(new RB_Supplies_CheckDetail_Extend() { RB_Group_Id = userInfo.RB_Group_id, CheckId = checkId, CheckStatus = PropertyCheckStatusEnum.NoInventory });
                foreach (var item in list)
                {
                    Dictionary<string, object> flies1 = new Dictionary<string, object>();
                    if ((item.InventoryNum ?? 0) > (item.RealityNum ?? 0))
                    {
                        flies1.Add(nameof(RB_Supplies_CheckDetail_Extend.CheckStatus), PropertyCheckStatusEnum.InventoryLosses);
                    }
                    else if ((item.InventoryNum ?? 0) == (item.RealityNum ?? 0))
                    {
                        flies1.Add(nameof(RB_Supplies_CheckDetail_Extend.CheckStatus), PropertyCheckStatusEnum.HaveInventory);
                    }
                    else if ((item.InventoryNum ?? 0) < (item.RealityNum ?? 0))
                    {
                        flies1.Add(nameof(RB_Supplies_CheckDetail_Extend.CheckStatus), PropertyCheckStatusEnum.InventoryProfit);
                    }
                    List<WhereHelper> wheres1 = new List<WhereHelper>() {
                        new WhereHelper(){
                                FiledName=nameof(RB_Property_CheckDetail_Extend.Id),
                                FiledValue=item.Id,
                                OperatorEnum=OperatorEnum.Equal
                        }
                    };
                    supplies_CheckDetailRepository.Update(flies1, wheres1);
                }

            }
            return flag;
        }

        #endregion

        #region 盘点报损
        /// <summary>
        /// 盘点报损分页
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="rowCount"></param>
        /// <param name="query"></param>
        /// <returns></returns>
        public List<RB_Supplies_ReportLossRecord_Extend> GetSuppliesReportLossRecordPageList(int pageIndex, int pageSize, out long rowCount, RB_Supplies_ReportLossRecord_Extend query)
        {
            var list = suppliesReportLossRecordRepository.GetSuppliesReportLossRecordPageList(pageIndex, pageSize, out rowCount, query);

            return list;
        }


        /// <summary>
        /// 盘点报损列表
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_Supplies_ReportLossRecord_Extend> GetReportLossRecordList(RB_Supplies_ReportLossRecord_Extend query)
        {
            return suppliesReportLossRecordRepository.GetReportLossRecordList(query);
        }

        /// <summary>
        /// 盘点报损列表
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_Supplies_ReportLossRecordDetail_Extend> GetReportLossRecordDetail(RB_Supplies_ReportLossRecordDetail_Extend query)
        {
            return suppliesReportLossRecordDetailRepository.GetReportLossRecordDetail(query);
        }


            /// <summary>
            /// 保存商品盘点记录
            /// </summary>
            /// <param name="demodel"></param>
            /// <returns></returns>
            public bool SetReportLossRecord(RB_Supplies_ReportLossRecord_Extend applyModel, RB_Supplies_StockOut_Extend demodel, RB_Supplies_StockIn_Extend inDemodel, int RB_Department_Id)
        {
            var trans = suppliesReportLossRecordRepository.DbTransaction;
            try
            {
                int Id = suppliesReportLossRecordRepository.Insert(applyModel, trans);
                if (Id > 0)
                {

                    #region 生成财务单据
                    var detailList = applyModel.DetailList.Select(x => new
                    {
                        CostTypeId = applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryLosses ? 225 : 226,
                        Number = x.ReportLossNum,
                        OriginalMoney = (x.ReportLossPrice ?? 0),
                        UnitPrice = Math.Round((x.ReportLossPrice ?? 0) / (x.ReportLossNum ?? 1), 2, MidpointRounding.AwayFromZero),
                        Remark = x.GoodsName + x.SpecificationName
                    });
                    string sign = "";
                    if (applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryLosses)
                    {
                        var financeObj = new
                        {
                            CreateBy = demodel.CreateBy,
                            IsPublic = 0,//  supplierModel.ClientBankAccount.AccountType, //financeConfigurineModel.AccountType,
                            ClientType = 10,
                            ClientID = 3170,
                            CurrencyId = Config.OutCurrencyId,
                            WBMoney = applyModel.DetailList.Sum(x => (x.ReportLossPrice ?? 0)),
                            RB_Branch_Id = Config.ExpendBranchId,
                            PayDate = System.DateTime.Now.ToString("yyyy-MM-dd"),
                            TemplateId = 129,
                            OrderSource = 16,
                            OtherType = 17,
                            Remark = System.DateTime.Now.ToString("yyyy年MM月dd日") + "生成" + (applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryLosses ? "盘亏" : "盘盈") + "财务单据",
                            detailList,
                            RB_Depart_Id = RB_Department_Id,
                            ReFinanceId = Id
                        };
                        sign = EncryptionHelper.AesEncrypt(JsonHelper.Serialize(financeObj), Config.FinanceKey);
                    }
                    else if (applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryProfit)
                    {
                        var financeObj = new
                        {
                            CreateBy = applyModel.CreateBy,
                            IsPublic = 0,
                            BType = 4,
                            AccountId = 27,
                            WBMoney = applyModel.DetailList.Sum(x => (x.ReportLossPrice ?? 0)),
                            RB_Branch_Id = Config.ExpendBranchId,
                            RemitterName = "商品盘点-盘盈",
                            TradeDate = System.DateTime.Now.AddDays(-1).ToString("yyyy-MM-dd"),
                            AccountNumber = System.DateTime.Now.AddDays(-1).ToString("yyyyMMdd"),
                            TemplateId = 130,
                            OrderSource = 16,
                            OtherType = 17,
                            Remark = System.DateTime.Now.ToString("yyyy年MM月dd日") + "自动生成财务单据",
                            detailList,
                            RB_Depart_Id = RB_Department_Id,
                            ReFinanceId = Id
                        };
                        sign = EncryptionHelper.AesEncrypt(JsonHelper.Serialize(financeObj), Config.FinanceKey);
                    }




                    var resultInfo = new
                    {
                        msg = sign,
                    };
                    string apiResult = HttpHelper.HttpPost(applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryLosses ? Config.PaymentFinanceApi : Config.IncomeFinanceApi, JsonHelper.Serialize(resultInfo), "");
                    JObject parmsJob = JObject.Parse(apiResult);
                    string resultCode = parmsJob.GetStringValue("resultCode");
                    int frid = parmsJob.GetInt("data", 0);
                    #endregion

                    if (resultCode == "1" && frid > 0)//新增记录
                    {
                        applyModel.FinanceId = frid;
                        applyModel.DetailList.ForEach(x => x.ReportLossRecordId = Id);
                        applyModel.DetailList.ForEach(x => x.CheckId = applyModel.CheckId);
                        applyModel.DetailList.ForEach(x => x.CheckStatus = applyModel.CheckStatus);
                        applyModel.DetailList.ForEach(x => x.Status = 0);
                        applyModel.DetailList.ForEach(x => x.CreateBy = applyModel.CreateBy);
                        applyModel.DetailList.ForEach(x => x.CreateDate = applyModel.CreateDate);
                        applyModel.DetailList.ForEach(x => x.RB_Branch_Id = applyModel.RB_Branch_Id);
                        applyModel.DetailList.ForEach(x => x.RB_Group_Id = applyModel.RB_Group_Id);

                        //更新已出库的商品信息
                        foreach (var item in applyModel.DetailList)//更新盘点商品的盘点状态
                        {
                            Dictionary<string, object> filesApplyGoods = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_CheckDetail.DealStatus),1},
                            { nameof(RB_Supplies_CheckDetail.UpdateBy),applyModel.CreateBy},
                             { nameof(RB_Supplies_CheckDetail.UpdateDate),applyModel.CreateDate},
                               { nameof(RB_Supplies_CheckDetail.FinanceId),frid},
                                { nameof(RB_Supplies_CheckDetail.Remark),item.Remark+(string.IsNullOrWhiteSpace(item.Remark)?"":",")+ " 自动生成财务单据:"+frid+",制单单价："+Math.Round((item.ReportLossPrice ?? 0) / (item.ReportLossNum ?? 1), 2, MidpointRounding.AwayFromZero)},
                        };
                            List<WhereHelper> wheresApplyGoods = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_Supplies_CheckDetail.Id),
                                    FiledValue=item.CheckDetailId,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                            supplies_CheckDetailRepository.Update(filesApplyGoods, wheresApplyGoods, trans);
                        }
                        applyModel.DetailList.ForEach(x => x.Remark = applyModel.Remark);
                        //新增盘点商品的信息
                        suppliesReportLossRecordDetailRepository.InsertBatch(applyModel.DetailList, trans);
                    }

                    int outOrInId = 0;

                    if (applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryLosses)
                    {
                        outOrInId = supplies_StockOutRepository.Insert(demodel);
                        //编码自动生成
                        string BMStr = outOrInId.ToString();
                        if (BMStr.Length < 5)
                        {
                            switch (BMStr.Length)
                            {
                                case 1:
                                    BMStr = "0000" + BMStr; break;
                                case 2:
                                    BMStr = "000" + BMStr; break;
                                case 3:
                                    BMStr = "00" + BMStr; break;
                                case 4:
                                    BMStr = "0" + BMStr; break;
                            }
                        }
                        BMStr = "CK" + DateTime.Now.ToString("yyyyMMdd") + BMStr;
                        Dictionary<string, object> files = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_StockOut.StockOutNum),BMStr}
                        };
                        List<WhereHelper> wheres = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_Supplies_StockOut.Id),
                                    FiledValue=outOrInId,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                        supplies_StockOutRepository.Update(files, wheres, trans);

                        //新增明细
                        foreach (var item in demodel.DetailList)
                        {
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                            decimal Price = 0;
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                Price = Math.Round((iModel.Money ?? 0) / (iModel.Number ?? 1), 2, MidpointRounding.AwayFromZero) * (item.Number ?? 0);
                            }
                            supplies_StockOutDetailRepository.Insert(new RB_Supplies_StockOutDetail()
                            {
                                CreateBy = demodel.CreateBy,
                                CreateDate = demodel.CreateDate,
                                Number = item.Number,
                                Money = Price,
                                RB_Branch_Id = demodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = demodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockOutId = outOrInId,
                                SuppliesId = item.SuppliesId,
                                UpdateBy = demodel.UpdateBy,
                                UpdateDate = demodel.UpdateDate
                            }, trans);

                            //新增库存表数据
                            if (ilist.Any() && demodel.StockOutState == StockOutStatusEnum.StockOut)
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number -= item.Number;//减少库存
                                iModel.Money -= Price;
                                if (iModel.Money < 0)
                                {
                                    iModel.Money = 0;
                                }
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = demodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = Price,
                                        RB_Branch_Id = demodel.RB_Branch_Id,
                                        RB_Group_Id = demodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 2,
                                        Id = 0,
                                        Description = "商品盘亏-出库"
                                    }, trans);
                                }
                            }
                        }
                    }

                    else if (applyModel.CheckStatus == PropertyCheckStatusEnum.InventoryProfit)
                    {
                        inDemodel.StockInState = StockInStatusEnum.StockIn;
                        outOrInId = supplies_StockInRepository.Insert(inDemodel, trans);
                        //编码自动生成
                        string BMStr = outOrInId.ToString();
                        if (BMStr.Length < 5)
                        {
                            switch (BMStr.Length)
                            {
                                case 1:
                                    BMStr = "0000" + BMStr; break;
                                case 2:
                                    BMStr = "000" + BMStr; break;
                                case 3:
                                    BMStr = "00" + BMStr; break;
                                case 4:
                                    BMStr = "0" + BMStr; break;
                            }
                        }
                        BMStr = "RK" + DateTime.Now.ToString("yyyyMMdd") + BMStr;
                        Dictionary<string, object> files = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_StockIn.StockInNum),BMStr}
                        };
                        List<WhereHelper> wheres = new List<WhereHelper>() {
                            new WhereHelper(){
                                 FiledName=nameof(RB_Supplies_StockIn.Id),
                                  FiledValue=outOrInId,
                                   OperatorEnum=OperatorEnum.Equal
                            }
                        };
                        supplies_StockInRepository.Update(files, wheres, trans);

                        //新增明细
                        foreach (var item in inDemodel.DetailList)
                        {
                            supplies_StockInDetailRepository.Insert(new RB_Supplies_StockInDetail()
                            {
                                CreateBy = inDemodel.CreateBy,
                                CreateDate = inDemodel.CreateDate,
                                Money = item.Money,
                                Number = item.Number,
                                RB_Branch_Id = inDemodel.RB_Branch_Id,
                                Id = 0,
                                RB_Group_Id = inDemodel.RB_Group_Id,
                                Remark = item.Remark,
                                Status = 0,
                                StockInId = outOrInId,
                                SuppliesId = item.SuppliesId,
                                UnitPrice = item.UnitPrice,
                                UpdateBy = inDemodel.UpdateBy,
                                UpdateDate = inDemodel.UpdateDate
                            }, trans);

                            //新增库存表数据
                            var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = inDemodel.RB_Group_Id, WarehouseId = inDemodel.WarehouseId, SuppliesId = item.SuppliesId });
                            if (ilist.Any())
                            {
                                var iModel = ilist.FirstOrDefault();
                                iModel.Number += item.Number;//增加库存
                                iModel.Money += item.Money;
                                var iflag = supplies_InventoryRepository.Update(iModel, trans);
                                if (iflag)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = inDemodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iModel.Id,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = inDemodel.RB_Branch_Id,
                                        RB_Group_Id = inDemodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "商品盘赢-入库"
                                    }, trans);
                                }
                            }
                            else
                            {
                                //新增库存表
                                int iId = supplies_InventoryRepository.Insert(new RB_Supplies_Inventory()
                                {
                                    CreateBy = inDemodel.CreateBy,
                                    CreateDate = DateTime.Now,
                                    Id = 0,
                                    Number = item.Number,
                                    Money = item.Money,
                                    RB_Branch_Id = inDemodel.RB_Branch_Id,
                                    RB_Group_Id = inDemodel.RB_Group_Id,
                                    Status = 0,
                                    SuppliesId = item.SuppliesId,
                                    UpdateBy = inDemodel.UpdateBy,
                                    UpdateDate = inDemodel.UpdateDate,
                                    WarehouseId = inDemodel.WarehouseId
                                }, trans);
                                if (iId > 0)
                                {
                                    //增加库存明细
                                    supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                    {
                                        CreateBy = inDemodel.CreateBy,
                                        CreateDate = DateTime.Now,
                                        InventoryId = iId,
                                        Number = item.Number,
                                        Money = item.Money,
                                        RB_Branch_Id = inDemodel.RB_Branch_Id,
                                        RB_Group_Id = inDemodel.RB_Group_Id,
                                        Status = 0,
                                        Type = 1,
                                        Id = 0,
                                        Description = "商品盘赢-入库"
                                    }, trans);
                                }
                            }
                        }
                    }


                    //更新申请单的出库状态
                    Dictionary<string, object> filesApply = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_ReportLossRecord.OutOrInId),outOrInId},
                             { nameof(RB_Supplies_ReportLossRecord.FinanceId),frid}
                        };
                    List<WhereHelper> wheresApply = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_Supplies_ReportLossRecord.Id),
                                    FiledValue=Id,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                    suppliesReportLossRecordRepository.Update(filesApply, wheresApply, trans);


                }
                suppliesReportLossRecordRepository.DBSession.Commit();
                return Id > 0;

            }
            catch (Exception ex)
            {
                suppliesReportLossRecordRepository.DBSession.Rollback("SetReportLossRecord");
                LogHelper.Write(ex, "SetReportLossRecord");
                return false;
            }
        }

        #endregion
    }
}
