﻿using DapperExtensions.Lambda;
using Newtonsoft.Json.Linq;
using Property.Model.Entity;
using Property.Model.Entity.Mall;
using Property.Model.Extend;
using Property.Model.Extend.Mall;
using Property.Repository;
using Property.Repository.Mall;
using REBORN.Common;
using REBORN.Common.Enum;
using REBORN.Common.Plugin;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Module.Mall
{
    public class WarehouseOutModule
    {

        /// <summary>
        /// 商品出库申请仓储层
        /// </summary>
        private readonly RB_WarehouseOut_GoodsRepository warehouseOutRepository = new RB_WarehouseOut_GoodsRepository();

        /// <summary>
        /// 出库申请仓储层
        /// </summary>
        private readonly RB_WarehouseOut_ApplyRepository warehouseOutApplyRepository = new RB_WarehouseOut_ApplyRepository();
        private readonly RB_WarehouseOut_FinanceRepository warehouseOutFinanceRepository = new RB_WarehouseOut_FinanceRepository();
        private readonly RB_WarehouseOut_GoodsDetailRepository warehouseOutGoodsDetailRepository = new RB_WarehouseOut_GoodsDetailRepository();

        /// <summary>
        /// 小程序仓储
        /// </summary>
        private readonly RB_MiniProgramRepository miniProgramRepository = new RB_MiniProgramRepository();
        private readonly RB_Goods_OrderDetailRepository goodsOrderDetailRepository = new RB_Goods_OrderDetailRepository();



        /// <summary>
        /// 商户仓储
        /// </summary>
        private readonly RB_TenantRepository tenantRepository = new RB_TenantRepository();
        private readonly Rb_employeeRepository employeeRepository = new Rb_employeeRepository();


        /// <summary>
        /// 财务生成记录
        /// </summary>
        private static RB_Finance_RecordRepository financeRecordRepository = new RB_Finance_RecordRepository();
        /// <summary>
        /// 财务生成记录详情
        /// </summary>
        private static RB_Finance_RecordDetailRepository financeRecordDetailRepository = new RB_Finance_RecordDetailRepository();

        /// <summary>
        /// 库存表
        /// </summary>
        private readonly RB_Supplies_InventoryRepository supplies_InventoryRepository = new RB_Supplies_InventoryRepository();
        /// <summary>
        /// 库存明细表
        /// </summary>
        private readonly RB_Supplies_InventoryDetailRepository supplies_InventoryDetailRepository = new RB_Supplies_InventoryDetailRepository();
        /// <summary>
        /// 出库单
        /// </summary>
        private readonly RB_Supplies_StockOutRepository supplies_StockOutRepository = new RB_Supplies_StockOutRepository();
        /// <summary>
        /// 出库单明细
        /// </summary>
        private readonly RB_Supplies_StockOutDetailRepository supplies_StockOutDetailRepository = new RB_Supplies_StockOutDetailRepository();



        /// <summary>
        /// 出库申请列表
        /// </summary>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页显示条数</param>
        /// <param name="rowCount">总条数</param>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Goods_Extend> GetWarehouseOutPageList(int pageIndex, int pageSize, out long rowCount, RB_WarehouseOut_Goods_Extend query)
        {
            return warehouseOutRepository.GetWarehouseOutPageList(pageIndex, pageSize, out rowCount, query);
        }


        /// <summary>
        /// 出库申请列表
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Goods_Extend> GetWarehouseOutList(RB_WarehouseOut_Goods_Extend query)
        {
            return warehouseOutRepository.GetWarehouseOutList(query);
        }





        /// <summary>
        /// 获取已出库的商品列表
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Goods_Extend> GetWarehouseOutListByIds(RB_WarehouseOut_Goods_Extend query)
        {
            return warehouseOutRepository.GetWarehouseOutListByIds(query);
        }



        /// <summary>
        /// 根据出库申请id与仓库id获取出库单中的商品
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Goods_Extend> GetOuGoodsByWarehouseId(RB_WarehouseOut_Apply_Extend query)
        {
            return warehouseOutRepository.GetOuGoodsByWarehouseId(query);

        }

        /// <summary>
        /// 出库申请列表
        /// </summary>
        /// <param name="pageIndex">页码</param>
        /// <param name="pageSize">每页显示条数</param>
        /// <param name="rowCount">总条数</param>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Apply_Extend> GetApplyPageList(int pageIndex, int pageSize, out long rowCount, RB_WarehouseOut_Apply_Extend query)
        {
            var list = warehouseOutApplyRepository.GetApplyPageList(pageIndex, pageSize, out rowCount, query);
            if (list != null && list.Any())
            {
                string mallBaseIds = string.Join(",", list.Where(x => x.MallBaseId > 0).Select(x => x.MallBaseId));
                string tenantIds = string.Join(",", list.Where(x => x.TenantId > 0).Select(x => x.TenantId));
                string createBys = string.Join(",", list.Where(x => x.CreateBy > 0).Select(x => x.CreateBy));
                if (!string.IsNullOrWhiteSpace(mallBaseIds))
                {
                    var mallBaseList = miniProgramRepository.GetMiniProgramListIds(mallBaseIds);
                    if (mallBaseList != null && mallBaseList.Any())
                    {
                        list.ForEach(x => x.MallBaseName = mallBaseList.Where(u => u.MallBaseId == x.MallBaseId).FirstOrDefault()?.MallName);
                    }
                }
                if (!string.IsNullOrWhiteSpace(tenantIds))
                {
                    var tenantList = tenantRepository.GetTenantListByIds(tenantIds);
                    if (tenantList != null && tenantList.Any())
                    {
                        list.ForEach(x => x.TenantName = tenantList.Where(u => u.TenantId == x.TenantId).FirstOrDefault()?.Name);
                    }
                }

                if (!string.IsNullOrWhiteSpace(createBys))
                {
                    var tenantList = employeeRepository.GetList(new Model.Extend.RB_Employee_Extend { EmployeeIds = createBys });
                    if (tenantList != null && tenantList.Any())
                    {
                        list.ForEach(x => x.CreateByName = tenantList.Where(u => u.EmployeeId == x.CreateBy).FirstOrDefault()?.EmName);
                    }
                }

            }
            return list;
        }


        /// <summary>
        /// 出库申请列表
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_Apply_Extend> GetWarehouseOutList(RB_WarehouseOut_Apply_Extend query)
        {
            return warehouseOutApplyRepository.GetApplyList(query);
        }


        /// <summary>
        /// 获取出库申请实体
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public RB_WarehouseOut_Apply GetApplyEntity(int id)
        {
            return warehouseOutApplyRepository.GetEntity(id);
        }


        /// <summary>
        /// 审核权限
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool UpdateWarehouseOutApply(RB_WarehouseOut_Apply model)
        {
            if (model.ID > 0)
            {
                Dictionary<string, object> files = new Dictionary<string, object>() {
                        { nameof(RB_WarehouseOut_Apply.ApplyStatus),model.ApplyStatus},
                        { nameof(RB_WarehouseOut_Apply.UpdateBy),model.UpdateBy},
                       { nameof(RB_WarehouseOut_Apply.UpdateDate),model.UpdateDate},
                       { nameof(RB_WarehouseOut_Apply.ExamineReason),model.ExamineReason}
            };
                List<WhereHelper> wheres = new List<WhereHelper>() {
                        new WhereHelper(){
                             FiledName=nameof(RB_WarehouseOut_Apply.ID),
                              FiledValue=model.ID,
                               OperatorEnum=OperatorEnum.Equal
                        }
                    };
                return warehouseOutApplyRepository.Update(files, wheres);
            }
            return false;
        }




        /// <summary>
        /// 更新出库商品数量
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        public bool UpdateWarehouseOutGoods(List<RB_WarehouseOut_Goods_Extend> list)
        {
            var trans = warehouseOutRepository.DbTransaction;
            try
            {

                foreach (var item in list)
                {
                    if (item.ID > 0)
                    {
                        Dictionary<string, object> files = new Dictionary<string, object>()
                        {
                            {nameof(RB_WarehouseOut_Goods.Number),item.Number},
                            {nameof(RB_WarehouseOut_Goods.NewSpecification),item.NewSpecification},
                            {nameof(RB_WarehouseOut_Goods.NewSpecificationSort),item.NewSpecificationSort},
                        };
                        List<WhereHelper> wheres = new List<WhereHelper>() {
                        new WhereHelper(){
                             FiledName=nameof(RB_WarehouseOut_Goods.ID),
                              FiledValue=item.ID,
                               OperatorEnum=OperatorEnum.Equal
                        }
                        };
                        warehouseOutRepository.Update(files, wheres);
                    }
                }
                warehouseOutRepository.DBSession.Commit();
                return true;
            }
            catch (Exception ex)
            {
                warehouseOutRepository.DBSession.Rollback("WarehouseOut.UpdateWarehouseOutApply");
                LogHelper.Write(ex, "WarehouseOut.UpdateWarehouseOutApply");
                return false;
            }
        }


        /// <summary>
        /// 出库商品成本信息
        /// </summary>
        /// <param name="query">查询条件</param>
        /// <returns></returns>
        public List<RB_WarehouseOut_GoodsDetail> GetApplyByOrderGoodsIdList(RB_WarehouseOut_GoodsDetail query, string orderGoodsId)
        {
            return warehouseOutGoodsDetailRepository.GetApplyByOrderGoodsIdList(query, orderGoodsId);
        }


        /// <summary>
        /// 保存出库单
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public bool SetStockOutInfo(RB_WarehouseOut_Apply_Extend applyModel, RB_Supplies_StockOut_Extend demodel, List<RB_WarehouseOut_Goods_Extend> financeList, RB_Finance_Record_Extend record, int RB_Department_Id, List<RB_WarehouseOut_Goods_Extend> WarehouseOutGoodsList, List<RB_WarehouseOut_GoodsDetail> outGoodsDetailList)
        {
            var trans = supplies_StockOutRepository.DbTransaction;
            try
            {
                int Id = supplies_StockOutRepository.Insert(demodel, trans);
                if (Id > 0)
                {

                    #region 生成财务单据
                    #region 关联财务订单
                    List<object> ECList = new List<object>();
                    foreach (var item in applyModel.WarehouseOutGoodsList) {
                        ECList.Add(new
                        {
                            ECOrderId = item.OrderId,
                            ECOrderDetailId = item.OrderGoodsId
                        });
                    }
                    ECList = ECList.Distinct().ToList();
                    #endregion

                    var detailList = financeList.Select(x => new
                    {
                        CostTypeId = 213,
                        Number = x.Number,
                        OriginalMoney = (x.CostMoney ?? 0) * (x.Number ?? 0),
                        UnitPrice = (x.CostMoney ?? 0),
                        Remark = x.NewGoodsName + x.NewSpecification + x.OrderNo
                    });
                    var financeObj = new
                    {
                        CreateBy = demodel.CreateBy,
                        IsPublic = 0,//  supplierModel.ClientBankAccount.AccountType, //financeConfigurineModel.AccountType,
                        ClientType = 10,
                        ClientID = 3134,
                        CurrencyId = Config.OutCurrencyId,
                        WBMoney = financeList.Sum(x => (x.CostMoney ?? 0) * (x.Number ?? 0)),
                        RB_Branch_Id = Config.ExpendBranchId,
                        PayDate = System.DateTime.Now.ToString("yyyy-MM-dd"),
                        TemplateId = 128,
                        OrderSource = 16,
                        OtherType = 16,
                        Remark = System.DateTime.Now.ToString("yyyy年MM月dd日") + "生成财务单据",
                        detailList,
                        RB_Depart_Id = RB_Department_Id,
                        ReFinanceId = Id,
                        ECOrderList = ECList
                    };
                    string sign = EncryptionHelper.AesEncrypt(JsonHelper.Serialize(financeObj), Config.FinanceKey);
                    var resultInfo = new
                    {
                        msg = sign,
                    };
                    string apiResult = HttpHelper.HttpPost(Config.PaymentFinanceApi, JsonHelper.Serialize(resultInfo), "");
                    JObject parmsJob = JObject.Parse(apiResult);
                    string resultCode = parmsJob.GetStringValue("resultCode");
                    int frid = parmsJob.GetInt("data", 0);
                    #endregion

                    if (resultCode == "1" && frid > 0)//新增记录
                    {
                        applyModel.WarehouseOutFinanceModel.FinanceId = frid;
                        applyModel.WarehouseOutFinanceModel.StockInId = Id;
                        financeList.ForEach(x => x.WarehouseId = applyModel.WarehouseId);
                        financeList.ForEach(x => x.StockInId = Id);
                        applyModel.WarehouseOutGoodsDetailList.ForEach(x => x.StockOutId = Id);


                        //更新申请单的出库状态
                        Dictionary<string, object> filesApply = new Dictionary<string, object>() {
                            { nameof(RB_WarehouseOut_Apply.OutStatus),applyModel.OutStatus}
                        };
                        List<WhereHelper> wheresApply = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_WarehouseOut_Apply.ID),
                                    FiledValue=applyModel.ID,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                        warehouseOutApplyRepository.Update(filesApply, wheresApply, trans);

                        //更新已出库的商品信息
                        foreach (var item in applyModel.WarehouseOutGoodsList)
                        {
                            Dictionary<string, object> filesApplyGoods = new Dictionary<string, object>() {
                            { nameof(RB_WarehouseOut_Goods.IsOut),1},
                            { nameof(RB_WarehouseOut_Goods.CostMoney),item.CostMoney}
                        };
                            List<WhereHelper> wheresApplyGoods = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_WarehouseOut_Goods.ID),
                                    FiledValue=item.ID,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                            warehouseOutRepository.Update(filesApplyGoods, wheresApplyGoods, trans);

                            //更新订单商品的财务id与成本价
                            decimal costMoney = 0;

                            var nowGoods = applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.OrderGoodsId);
                            var nowGoodsFinan = WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.OrderGoodsId);
                            if (nowGoods != null && nowGoods.Any())
                            {
                                //查询已有的成本
                                decimal oldMoney = 0;
                                var oldOutGoods = outGoodsDetailList.Where(x => x.OrderGoodsId == item.OrderGoodsId);
                                if (oldOutGoods != null && oldOutGoods.Any())
                                {
                                    oldMoney = oldOutGoods.Sum(x => x.CostMoney ?? 0);
                                }
                                costMoney = ((nowGoods.Sum(x => x.CostMoney ?? 0) + oldMoney) / nowGoodsFinan.FirstOrDefault()?.Number ?? 0);
                                // costMoney = (nowGoods.Sum(x => x.CostMoney ?? 0) / nowGoodsFinan.Sum(x => x.Number ?? 0));
                                //更新申请单的出库状态
                                Dictionary<string, object> filesGoods = new Dictionary<string, object>() {
                                {nameof(RB_Goods_OrderDetail.CostFinanceId),frid},
                                { nameof(RB_Goods_OrderDetail.CostMoney),Math.Round((costMoney), 2, MidpointRounding.AwayFromZero)},
                                 { nameof(RB_Goods_OrderDetail.PaidCostMoney),Math.Round((costMoney), 2, MidpointRounding.AwayFromZero)}
                            };
                                List<WhereHelper> wheresGoods = new List<WhereHelper>() { new WhereHelper()
                                {
                                        FiledName=nameof(RB_Goods_OrderDetail.Id),
                                        FiledValue=item.OrderGoodsId,
                                        OperatorEnum=OperatorEnum.Equal
                                }
                            };
                                goodsOrderDetailRepository.Update(filesGoods, wheresGoods);
                            }
                        }
                        //新增申请单的财务单据
                        warehouseOutFinanceRepository.Insert(applyModel.WarehouseOutFinanceModel, trans);
                        warehouseOutGoodsDetailRepository.InsertBatch(applyModel.WarehouseOutGoodsDetailList, trans);
                        //写入电商财务记录

                        record.FinanceId = frid;

                        int recordId = 0;
                        if (record.ID == 0)
                        {
                            recordId = financeRecordRepository.Insert(record);
                            if (record.RecordDetailList != null && record.RecordDetailList.Any())
                            {
                                record.RecordDetailList.ForEach(x => x.RecordId = recordId);
                                record.RecordDetailList.ForEach(x => x.FinanceId = record.FinanceId);
                                financeRecordDetailRepository.InsertBatch(record.RecordDetailList);
                            }
                        }

                    }
                    //编码自动生成
                    string BMStr = Id.ToString();
                    if (BMStr.Length < 5)
                    {
                        switch (BMStr.Length)
                        {
                            case 1:
                                BMStr = "0000" + BMStr; break;
                            case 2:
                                BMStr = "000" + BMStr; break;
                            case 3:
                                BMStr = "00" + BMStr; break;
                            case 4:
                                BMStr = "0" + BMStr; break;
                        }
                    }
                    BMStr = "CK" + DateTime.Now.ToString("yyyyMMdd") + BMStr;
                    Dictionary<string, object> files = new Dictionary<string, object>() {
                            { nameof(RB_Supplies_StockOut.StockOutNum),BMStr}
                        };
                    List<WhereHelper> wheres = new List<WhereHelper>() {
                            new WhereHelper(){
                                    FiledName=nameof(RB_Supplies_StockOut.Id),
                                    FiledValue=Id,
                                    OperatorEnum=OperatorEnum.Equal
                            }
                        };
                    supplies_StockOutRepository.Update(files, wheres, trans);

                    //新增明细
                    foreach (var item in demodel.DetailList)
                    {
                        var ilist = supplies_InventoryRepository.GetList(new RB_Supplies_Inventory_Extend() { RB_Group_Id = demodel.RB_Group_Id, WarehouseId = demodel.WarehouseId, SuppliesId = item.SuppliesId });
                        decimal Price = 0;
                        if (ilist.Any())
                        {
                            var iModel = ilist.FirstOrDefault();
                            Price = Math.Round((iModel.Money ?? 0) / (iModel.Number ?? 1), 2, MidpointRounding.AwayFromZero) * (item.Number ?? 0);
                        }
                        supplies_StockOutDetailRepository.Insert(new RB_Supplies_StockOutDetail()
                        {
                            CreateBy = demodel.CreateBy,
                            CreateDate = demodel.CreateDate,
                            Number = item.Number,
                            Money = Price,
                            RB_Branch_Id = demodel.RB_Branch_Id,
                            Id = 0,
                            RB_Group_Id = demodel.RB_Group_Id,
                            Remark = item.Remark,
                            Status = 0,
                            StockOutId = Id,
                            SuppliesId = item.SuppliesId,
                            UpdateBy = demodel.UpdateBy,
                            UpdateDate = demodel.UpdateDate
                        }, trans);

                        //新增库存表数据
                        if (ilist.Any() && demodel.StockOutState == StockOutStatusEnum.StockOut)
                        {
                            var iModel = ilist.FirstOrDefault();
                            iModel.Number -= item.Number;//减少库存
                            iModel.Money -= Price;
                            if (iModel.Money < 0)
                            {
                                iModel.Money = 0;
                            }
                            var iflag = supplies_InventoryRepository.Update(iModel, trans);
                            if (iflag)
                            {
                                //增加库存明细
                                supplies_InventoryDetailRepository.Insert(new RB_Supplies_InventoryDetail()
                                {
                                    CreateBy = demodel.CreateBy,
                                    CreateDate = DateTime.Now,
                                    InventoryId = iModel.Id,
                                    Number = item.Number,
                                    Money = Price,
                                    RB_Branch_Id = demodel.RB_Branch_Id,
                                    RB_Group_Id = demodel.RB_Group_Id,
                                    Status = 0,
                                    Type = 2,
                                    Id = 0,
                                    Description = "减少库存-出库"
                                }, trans);
                            }
                        }
                    }
                }
                supplies_StockOutRepository.DBSession.Commit();
                return Id > 0;

            }
            catch (Exception ex)
            {
                supplies_StockOutRepository.DBSession.Rollback("WarehouseOut.SetStockOutInfo");
                LogHelper.Write(ex, "WarehouseOut.SetStockOutInfo");
                return false;
            }
        }

    }
}
