﻿using Property.Model.Entity;
using Property.Model.Extend;
using REBORN.Common.Enum;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 耗材出库单明细仓储类
    /// </summary>
    public partial class RB_Supplies_StockOutDetailRepository : RepositoryBase<RB_Supplies_StockOutDetail>
    {
        /// <summary>
        /// 获取出库明细数据
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockOutDetail_Extend> GetList(RB_Supplies_StockOutDetail_Extend dmodel)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ssd.{0}={1}", nameof(RB_Supplies_StockOutDetail_Extend.Status), 0);
            where += $@" and ssd.RB_Group_Id={dmodel.RB_Group_Id}";            

            if (dmodel.StockOutId > 0) {
                where += " and ssd." + nameof(RB_Supplies_StockOutDetail_Extend.StockOutId) + "=" + dmodel.StockOutId;
            }
            if (!string.IsNullOrEmpty(dmodel.StockOutIdStr)) {
                where += " and ssd." + nameof(RB_Supplies_StockOutDetail_Extend.StockOutId) + " in(" + dmodel.StockOutIdStr + ")";
            }
            
            string sql = $@" select ssd.*,sm.Name as SuppliesName,sm.SuppliesNum,sm.BrandName,sm.SpecificationName,sm.GoodsCategoryName,sm.SuppliesModel,sm.Units,pc.Name as CategoryName
from rb_supplies_stockoutdetail ssd 
left join rb_supplies_material sm on ssd.SuppliesId=sm.Id
left join rb_property_category pc on pc.Id=sm.CategoryId
{where} order by ssd.Id desc";
            return Get<RB_Supplies_StockOutDetail_Extend>(sql).ToList();
        }

        /// <summary>
        /// 获取领用情况
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockOutDetail_Extend> GetRecipientsQueryPageList(int pageIndex, int pageSize, RB_Supplies_StockOutDetail_Extend dmodel, out long count)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ssd.{0}={1}", nameof(RB_Supplies_StockOutDetail_Extend.Status), 0);
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Supplies_StockOut.Status), 0);
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Supplies_StockOut.StockOutState), (int)StockOutStatusEnum.StockOut);
            where += $@" and ssd.RB_Group_Id={dmodel.RB_Group_Id}";

            if (dmodel.EmployeeId > 0) {
                where += " and ss." + nameof(RB_Supplies_StockOut.EmployeeId) + "=" + dmodel.EmployeeId;
            }
            if (dmodel.CategoryId > 0) {
                where += " and sm." + nameof(RB_Supplies_Material.CategoryId) + "=" + dmodel.CategoryId;
            }
            if (!string.IsNullOrEmpty(dmodel.SuppliesName)) {
                where += " and sm." + nameof(RB_Supplies_Material.Name) + " like '%" + dmodel.SuppliesName + "%'";
            }
            if (dmodel.SuppliesId > 0) {
                where += " and sm." + nameof(RB_Supplies_Material.Id) + " =" + dmodel.SuppliesId;
            }
            if (!string.IsNullOrEmpty(dmodel.StartTime)) {
                where += " and ss." + nameof(RB_Supplies_StockOut.StockOutDate) + " >='"+dmodel.StartTime+"'";
            }
            if (!string.IsNullOrEmpty(dmodel.EndTime)) {
                where += " and ss." + nameof(RB_Supplies_StockOut.StockOutDate) + " <='" + dmodel.EndTime + " 23:59:59'";
            }
            if (dmodel.DepartmentId > 0) {
                where += " and sm." + nameof(RB_Supplies_StockOut.DepartmentId) + "=" + dmodel.DepartmentId;
            }
            if (dmodel.BranchId >= 0) {
                where += " and sm." + nameof(RB_Supplies_StockOut.BranchId) + "=" + dmodel.BranchId;
            }

            string sql = $@" select ssd.*,ss.StockOutDate,ss.EmployeeId,ss.BranchId,ss.DepartmentId,ss.StockOutNum,sm.Name as SuppliesName,sm.SpecificationName,sm.GoodsCategoryName,pc.Name as CategoryName
from rb_supplies_stockoutdetail ssd 
left join rb_supplies_stockout ss on ssd.StockOutId=ss.Id
left join rb_supplies_material sm on ssd.SuppliesId=sm.Id
left join rb_property_category pc on pc.Id=sm.CategoryId
{where} order by ss.StockOutDate desc";
            return GetPage<RB_Supplies_StockOutDetail_Extend>(pageIndex, pageSize, out count, sql).ToList();
        }

        /// <summary>
        /// 获取领用统计
        /// </summary>
        /// <param name="demodel"></param>
        /// <returns></returns>
        public List<RB_Supplies_StockOutDetail_Extend> GetRecipientsQueryTotal(RB_Supplies_StockOutDetail_Extend dmodel)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND ssd.{0}={1}", nameof(RB_Supplies_StockOutDetail_Extend.Status), 0);
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Supplies_StockOut.Status), 0);
            where += string.Format(" AND ss.{0}={1}", nameof(RB_Supplies_StockOut.StockOutState), (int)StockOutStatusEnum.StockOut);
            where += $@" and ssd.RB_Group_Id={dmodel.RB_Group_Id}";

            if (dmodel.EmployeeId > 0)
            {
                where += " and ss." + nameof(RB_Supplies_StockOut.EmployeeId) + "=" + dmodel.EmployeeId;
            }
            if (dmodel.CategoryId > 0)
            {
                where += " and sm." + nameof(RB_Supplies_Material.CategoryId) + "=" + dmodel.CategoryId;
            }
            if (!string.IsNullOrEmpty(dmodel.SuppliesName))
            {
                where += " and sm." + nameof(RB_Supplies_Material.Name) + " like '%" + dmodel.SuppliesName + "%'";
            }
            if (!string.IsNullOrEmpty(dmodel.StartTime))
            {
                where += " and ss." + nameof(RB_Supplies_StockOut.StockOutDate) + " >='" + dmodel.StartTime + "'";
            }
            if (!string.IsNullOrEmpty(dmodel.EndTime))
            {
                where += " and ss." + nameof(RB_Supplies_StockOut.StockOutDate) + " <='" + dmodel.EndTime + " 23:59:59'";
            }
            if (dmodel.DepartmentId > 0)
            {
                where += " and sm." + nameof(RB_Supplies_StockOut.DepartmentId) + "=" + dmodel.DepartmentId;
            }
            if (dmodel.BranchId >= 0)
            {
                where += " and sm." + nameof(RB_Supplies_StockOut.BranchId) + "=" + dmodel.BranchId;
            }

            string sql = $@" select sum(IFNULL(ssd.Number,0)) as Number,sum(IFNULL(ssd.Money,0)) as Money
from rb_supplies_stockoutdetail ssd 
left join rb_supplies_stockout ss on ssd.StockOutId=ss.Id
left join rb_supplies_material sm on ssd.SuppliesId=sm.Id
left join rb_property_category pc on pc.Id=sm.CategoryId
{where} order by ss.StockOutDate desc";
            return Get<RB_Supplies_StockOutDetail_Extend>(sql).ToList();
        }
    }
}
