﻿using Property.Model.Entity;
using Property.Model.Extend;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 分公司仓储类
    /// </summary>
    public partial class Rb_branchRepository : RepositoryBase<RB_Branch>
    {
        /// <summary>
        /// 获取分公司分页列表
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Branch_Extend> GetPageList(int pageIndex, int pageSize, RB_Branch_Extend dmodel, out long count)
        {
            string where = " where 1=1 ";
            if (!string.IsNullOrWhiteSpace(dmodel.BName))
            {
                where += " and b." + nameof(RB_Branch_Extend.BName) + " like '%" + dmodel.BName + "%'";
            }
            if (!string.IsNullOrWhiteSpace(dmodel.Domain))
            {
                where += " and b." + nameof(RB_Branch_Extend.Domain) + " like '%" + dmodel.Domain + "%'";
            }
            if (dmodel.Status.HasValue && dmodel.Status >= 0)
            {
                where += " and b." + nameof(RB_Branch_Extend.Status) + "=" + dmodel.Status;
            }
            if (dmodel.RB_Group_Id.HasValue && dmodel.RB_Group_Id > 0)
            {
                where += " and b." + nameof(RB_Branch_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            return GetPage<RB_Branch_Extend>(pageIndex, pageSize, out count, "select b.*,e.EmName as BranchManagerName,g.GroupName from Rb_branch b Left join Rb_employee e on b.BranchManager=e.EmployeeId left join rb_group g on g.Id=b.RB_Group_Id " + where + " order by b.Id desc").ToList();
        }

        /// <summary>
        /// 根据where获取分公司列表
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Branch_Extend> GetList(RB_Branch_Extend dmodel)
        {
            string where = " 1=1 ";
            if (dmodel.Status.HasValue)
            {
                where += " and " + nameof(RB_Branch_Extend.Status) + "=" + dmodel.Status;
            }
            else
            {
                where += " and " + nameof(RB_Branch_Extend.Status) + "=0 ";
            }
            if (dmodel.is_show.HasValue)
            {
                where += " and " + nameof(RB_Branch_Extend.is_show) + "=" + dmodel.is_show;
            }
            if (dmodel.RB_Group_Id.HasValue)
            {
                where += " and " + nameof(RB_Branch_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            if (dmodel.IsParentCompany.HasValue && dmodel.IsParentCompany > 0)
            {
                where += " and " + nameof(RB_Branch_Extend.IsParentCompany) + "=" + dmodel.IsParentCompany;
            }
            if (dmodel.B2BShow.HasValue)
            {
                where += " and " + nameof(RB_Branch_Extend.B2BShow) + "=" + dmodel.B2BShow;
            }
            return Get<RB_Branch_Extend>("select * from RB_Branch where" + where).ToList();
        }

        /// <summary>
        /// 根据公司IDs，批量获取分公司名称列表
        /// </summary>
        /// <param name="ids">员工Id，多个员工用,</param>
        /// <returns></returns>
        public List<RB_Branch_Extend> GetBranchNameByIds(string ids)
        {
            return Get<RB_Branch_Extend>($@"SELECT Id,BName from rb_branch  where Id in ({ids})").ToList();
        }

        /// <summary>
        /// 根据城市编号获取公司列表【HK新增】
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public List<RB_Branch_Extend> GetListByCityId(string Ids)
        {
            if (!string.IsNullOrEmpty(Ids))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendFormat(@" SELECT Id,BName,City FROM rb_branch WHERE 1=1  ");
                sb.AppendFormat(" AND {0}=0 ", nameof(RB_Branch_Extend.Status));
                sb.AppendFormat(" AND City in ({0}) ", Ids);
                return Get<RB_Branch_Extend>(sb.ToString()).ToList();
            }
            return new List<RB_Branch_Extend>();
        }

        /// <summary>
        /// 获取公司下面所有的员工
        /// </summary>
        /// <param name="branchId"></param>
        /// <returns></returns>
        public int GetBranchPeopleCount(int branchId)
        {
            string sql = $"select Count(0) from rb_employee where IsLeave=0 and RB_Branch_id={branchId}";
            object result = this.ExecuteScalar(sql);
            return !Convert.IsDBNull(result) && !object.Equals(result, null) ? Convert.ToInt32(result) : 0;
        }

        /// <summary>
        /// 获取公司的坐标
        /// </summary>
        /// <returns></returns>
        public List<RB_Branch_Extend> GetBranchLngLat(int BranchId,int GroupId=2)
        {
            var where = " WHERE 1=1 ";
            where += string.Format(" AND {0}=0 ", nameof(RB_Branch.Status));
            if (BranchId >= 0) {
                where += string.Format(" AND {0}={1} ", nameof(RB_Branch.Id), BranchId);
            }
            if (GroupId > 0) {
                where += string.Format(" AND {0}={1} ", nameof(RB_Branch.RB_Group_Id), GroupId);
            }
            return Get<RB_Branch_Extend>($@"SELECT * FROM RB_Branch {where}").ToList();
        }
    }
}
