﻿using Property.Model.Entity;
using Property.Model.Extend;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 部门信息仓储类
    /// </summary>
    public partial class Rb_departmentRepository : RepositoryBase<Model.Entity.RB_Department>
    {
        /// <summary>
        /// 获取部门分页数据
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetPageList(int pageIndex, int pageSize, RB_Department_Extend dmodel, out long count)
        {
            string where = " where 1=1 ";
            where += string.Format(" AND d.{0}={1}", nameof(RB_Department_Extend.Status), 0);
            where += $@" and d.RB_Group_Id={dmodel.RB_Group_Id}";
            if (dmodel.RB_Branch_Id > -1)
            {
                where += " and d." + nameof(RB_Department_Extend.RB_Branch_Id) + "=" + dmodel.RB_Branch_Id;
            }
            if (dmodel.ParentId.HasValue && dmodel.ParentId > 0)
            {
                where += " and d." + nameof(RB_Department_Extend.ParentId) + "=" + dmodel.ParentId;
            }
            if (dmodel.Tier.HasValue && dmodel.Tier > 0)
            {
                where += " and d." + nameof(RB_Department_Extend.Tier) + "=" + dmodel.Tier;
            }
            if (!string.IsNullOrWhiteSpace(dmodel.DepartmentName.Trim()))
            {
                where += " and d." + nameof(RB_Department_Extend.DepartmentName) + " like '%" + dmodel.DepartmentName.Trim() + "%'";
            }
            string sql = $"select d.*,e.{nameof(RB_Employee.EmName)} as DepartmentManagerName,b.{nameof(RB_Branch.BName)} as RB_Branch_Name,(select GROUP_CONCAT(p.PostName SEPARATOR '、') from rb_departmentpost_relevance dpr left join rb_post p on dpr.PostId=p.PostId where dpr.DepartmentId=d.DepartmentID) as PostName " +
                $" from  Rb_department d " +
                $" left join {nameof(RB_Employee)} e on d.{nameof(RB_Department.DepartmentManager)}=e.{nameof(RB_Employee.EmployeeId)}" +
                $" left join {nameof(RB_Branch)} b on d.{nameof(RB_Department.RB_Branch_Id)}=b.{nameof(RB_Branch.Id)} { where } order by d.Sort asc,d.DepartmentID desc";
            return GetPage<RB_Department_Extend>(pageIndex, pageSize, out count, sql).ToList();
        }

        /// <summary>
        /// App获取部门列表
        /// </summary>
        /// <param name="branchId">公司id</param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetCompanyDepartment(int branchId)
        {
            var departmentList = Get<RB_Department_Extend>($@" SELECT d.*,
(SELECT Count(*)  from rb_employee e where e.RB_Department_Id = d.DepartmentID and e.IsLeave = 0) as EmployeeCount
 from rb_department d where d.RB_Branch_Id ={branchId} and d.`Status` = 0 order by d.Sort asc").ToList();
            return departmentList;
        }

        /// <summary>
        /// 根据条件获取部门列表
        /// </summary>
        /// <param name="whereModel"></param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetList(RB_Department_Extend whereModel)
        {
            string where = " WHERE 1=1 ";
            where += string.Format(" AND {0}=0", nameof(RB_Department_Extend.Status));
            if (whereModel.RB_Branch_Id > -1)
            {
                where += " AND " + nameof(RB_Department_Extend.RB_Branch_Id) + "=" + whereModel.RB_Branch_Id;
            }
            if (whereModel.RB_Group_Id > 0)
            {
                where += " AND " + nameof(RB_Department_Extend.RB_Group_Id) + "=" + whereModel.RB_Group_Id;
            }
            if (whereModel.ParentId.HasValue && whereModel.ParentId > 0)
            {
                where += " AND " + nameof(RB_Department_Extend.ParentId) + "=" + whereModel.ParentId;
            }
            if (whereModel.Tier.HasValue && whereModel.Tier > 0)
            {
                where += " AND " + nameof(RB_Department_Extend.Tier) + "=" + whereModel.Tier;
            }
            if (whereModel.RootId.HasValue && whereModel.RootId > 0)
            {
                where += " AND " + nameof(RB_Department_Extend.RootId) + "=" + whereModel.RootId;
            }
            if (whereModel.DepartmentManager.HasValue && whereModel.DepartmentManager > 0)
            {
                where += " AND " + nameof(RB_Department_Extend.DepartmentManager) + "=" + whereModel.DepartmentManager;
            }
            return Get<RB_Department_Extend>($"select * from  Rb_department{ where } order by Sort asc,DepartmentID desc").ToList();
        }

        /// <summary>
        /// 根据员工id获取部门信息
        /// </summary>
        /// <param name="empId">员工id</param>
        /// <returns></returns>
        public RB_Department_Extend GetDepartmentByEmpId(int empId)
        {
            return Get<RB_Department_Extend>($@"SELECT * from  rb_department d where d.Status=0 and d.DepartmentID = (
                SELECT e.RB_Department_Id from rb_employee e where e.EmployeeId = {empId}
                )").FirstOrDefault();
        }
        /// <summary>
        /// 根据多个员工ID获取部门信息
        /// </summary>
        /// <param name="empIds"></param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetDepartmentByEmpIds(string empIds)
        {
            return Get<RB_Department_Extend>($@"SELECT * from  rb_department d where d.Status=0 and d.DepartmentID in (
                SELECT e.RB_Department_Id from rb_employee e where e.EmployeeId in ({empIds})
                )").ToList();
        }

        /// <summary>
        /// 根据员工id获取部门分组信息
        /// </summary>
        /// <param name="empId">员工id</param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetDepartmentGroupByEmpId(int empId)
        {
            return Get<RB_Department_Extend>($@"SELECT d.* from  rb_department d where d.RootId = (
            SELECT IFNULL(d.RootId,0) from  rb_department d where d.Status=0 and d.DepartmentID = (
                  SELECT e.RB_Department_Id from rb_employee e where e.EmployeeId = {empId}))
            ").ToList();
        }

        /// <summary>
        /// 根据根据部门id获取部门信息
        /// </summary>
        /// <param name="depIds">部门id(1,2,3)</param>
        /// <returns></returns>
        public List<RB_Department_Extend> GetDepartmentByDepartmentIds(string depIds)
        {
            string where = "";
            if (!string.IsNullOrWhiteSpace(depIds))
            {

                where = $@"  and d.DepartmentID in ({depIds}) ";
            }
            return Get<RB_Department_Extend>($"SELECT * from rb_department d where  d.`Status` = 0  {where}").ToList();
        }

        /// <summary>
        /// 根据部门负责人获取管理部门列表
        /// </summary>
        /// <param name="empId">员工id</param>
        /// <returns></returns>
        public List<Model.Entity.RB_Department> GetCompanyDepartmentByManager(int empId)
        {
            var departmentList = Get<Model.Entity.RB_Department>($@" SELECT * from rb_department where DepartmentManager = {empId} and `Status` = 0").ToList();
            return departmentList;
        }
        /// <summary>
        /// 获取部门主管
        /// </summary>
        /// <param name="empIdStr"></param>
        /// <returns></returns>
        public List<Model.Entity.RB_Department> GetCompanyDepartmentByManager(string empIdStr)
        {
            var departmentList = Get<Model.Entity.RB_Department>($@" SELECT * from rb_department where DepartmentManager in({empIdStr}) and `Status` = 0").ToList();
            return departmentList;
        }
        /// <summary>
        /// 根据公司id获取公司所有部门
        /// </summary>
        /// <param name="branchId">公司id</param>
        /// <returns></returns>
        public List<Model.Entity.RB_Department> GetBranchAllDepartment(int branchId)
        {
            var departmentList = Get<Model.Entity.RB_Department>($@"SELECT * from rb_department d where d.RB_Branch_Id = {branchId} and d.`Status` = 0").ToList();
            return departmentList;
        }
        /// <summary>
        /// 根据当前人的部门id获取下级部门id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public string getDepartmentChildList(int id)
        {
            string Rstr = Get<string>($@"SELECT getDepartmentChildList({id})").FirstOrDefault();
            if (!string.IsNullOrWhiteSpace(Rstr))
            {
                Rstr = Rstr.Replace("$,", "");
            }
            return Rstr;
        }

        /// <summary>
        /// 根据当前人的部门id获取父级部门id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public string getDepartmentParentLst(int id)
        {
            return Get<string>($@"SELECT getDepartmentParentLst({id})").FirstOrDefault();
        }

        /// <summary>
        /// 根据查询条件获取管理部门列表【可以自行扩展】
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <returns></returns>
        public List<Model.Entity.RB_Department> GetListByWhere(RB_Department_Extend where)
        {
            StringBuilder builder = new StringBuilder();
            builder.AppendFormat(" SELECT DepartmentID,DepartmentName FROM {0} WHERE 1=1 ", nameof(RB_Department));
            builder.AppendFormat(" AND {0}={1} ", nameof(RB_Department_Extend.Status), (int)REBORN.Common.Enum.DateStateEnum.Normal);
            if (where != null)
            {
                if (where.QBranchIds != null && !string.IsNullOrEmpty(where.QBranchIds))
                {
                    builder.AppendFormat(" AND {0} IN ({1}) ", nameof(RB_Department_Extend.RB_Branch_Id), where.QBranchIds.TrimStart(',').TrimEnd(','));
                }
            }
            return Get<Model.Entity.RB_Department>(builder.ToString()).ToList();
        }

        /// <summary>
        /// 返回指定部门ID下的所有员工（包含子部门员工）数量
        /// </summary>
        /// <param name="departmentId"></param>
        /// <returns></returns>
        public int GetDepartmentPeopleCount(int departmentId)
        {
            string sql = $"call PROC_Get_DepartAllUser('{departmentId}')";
            object result = this.ExecuteScalar(sql);
            return !Convert.IsDBNull(result) && !object.Equals(result, null) ? Convert.ToInt32(result) : 0;
        }
    }
}