﻿using Dapper;
using REBORN.Common.Enum;
using Property.Model.Entity;
using Property.Model.Extend;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace Property.Repository
{
    /// <summary>
    /// 员工扩展方法
    /// </summary>
    public partial class Rb_employeeRepository : RepositoryBase<RB_Employee>
    {
        /// <summary>
        /// 表名称
        /// </summary>
        public string TableName { get { return nameof(Model.Entity.RB_Employee); } }

        /// <summary>
        /// 根据where 获取列表数据
        /// </summary>
        /// <param name="where">查询条件</param>
        /// <returns></returns>
        public List<RB_Employee_Extend> GetList(RB_Employee_Extend where)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat($@" SELECT e.*,b.BName as CompanyName,p.PostName FROM {TableName} e 
left join RB_Post p on e.{nameof(RB_Employee.RB_Post_Id)}=p.PostId 
LEFT JOIN rb_branch b on e.RB_Branch_id = b.Id
WHERE 1=1 ");
            if (where != null)
            {
                if (where.RB_Group_id > 0)
                {
                    sb.AppendFormat(" AND e.{0}={1} ", nameof(Model.Entity.RB_Employee.RB_Group_id), where.RB_Group_id);
                }
                if (!string.IsNullOrWhiteSpace(where.EmployeeIds))
                {
                    sb.AppendFormat(" AND e.{0} in({1}) ", nameof(Model.Entity.RB_Employee.EmployeeId), where.EmployeeIds);
                }
                if (where.RB_Department_Id > 0)
                {
                    sb.AppendFormat(" AND e.{0}={1} ", nameof(Model.Entity.RB_Employee.RB_Department_Id), where.RB_Department_Id);
                }
                if (where.BranchId >= 0)
                {
                    sb.AppendFormat(" AND e.{0}={1} ", nameof(Model.Entity.RB_Employee.RB_Branch_id), where.BranchId);
                }
                if (where.RB_Post_Id > 0)
                {
                    sb.AppendFormat(" AND e.{0}={1} ", nameof(Model.Entity.RB_Employee.RB_Post_Id), where.RB_Post_Id);
                }
                if (!string.IsNullOrEmpty(where.EmName)) {
                    sb.AppendFormat(" AND e.{0} like '%{1}%' ", nameof(Model.Entity.RB_Employee.EmName), where.EmName);
                }
                sb.Append(" AND e.IsLeave=0 ");
            }
            return Get<RB_Employee_Extend>(sb.ToString()).ToList();
        }

        /// <summary>
        /// 获取登录员工信息
        /// </summary>
        /// <param name="EmployeeId"></param>
        /// <returns></returns>
        public RB_Employee_Extend GetEmpInfoByEmpId(int EmployeeId)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(@"
                                 SELECT e.*,g.GroupName,b.BName as CompanyName,d.DepartmentName,P.PostName  
                                 FROM rb_employee e
                                 LEFT JOIN rb_group g ON e.RB_Group_id=g.Id
                                 LEFT JOIN rb_branch b on e.RB_Branch_id = b.Id
                                 LEFT JOIN rb_department d on e.RB_Department_Id = d.DepartmentID
                                 LEFT JOIN rb_post p ON e.RB_Post_Id=p.PostId
                                 WHERE e.{0} ={1}", nameof(RB_Employee_Extend.EmployeeId), EmployeeId);
            sb.Append(" LIMIT 1;");
            return Get<RB_Employee_Extend>(sb.ToString()).ToList().FirstOrDefault();
        }


        /// <summary>
        /// 根据登陆手机号获取员工信息  最新（2018-12-18 Add）
        /// </summary>
        /// <param name="loginMobile">登陆手机号</param>
        /// <param name="RB_Group_id">集团id</param>
        /// <returns></returns>
        public RB_Employee GetByLoginMobile(string loginMobile, int RB_Group_id = 0)
        {
            string sql = $"SELECT * from rb_employee WHERE EmLoginMobile = '{loginMobile}' ";
            if (RB_Group_id > 0)
            {
                sql += $" and RB_Group_id={RB_Group_id}";
            }
            return Get<RB_Employee>(sql).FirstOrDefault();
        }

        /// <summary>
        /// 根据账号获取员工信息 最新（2018-12-18新增）
        /// </summary>
        /// <param name="account">账号</param>
        /// <param name="RB_Group_id">集团id</param>
        /// <returns></returns>
        public RB_Employee GetByAccount(string account, int RB_Group_id = 0)
        {
            string sql = $"SELECT * from rb_employee WHERE EmAccount = '{account}' and IsLeave = 0";
            if (RB_Group_id > 0)
            {
                sql += $" and RB_Group_id={RB_Group_id}";
            }
            return Get<RB_Employee>(sql).FirstOrDefault();
        }

        /// <summary>
        /// 根据员工IDs，批量获取员工信息
        /// </summary>
        /// <param name="ids">员工Id，多个员工用,</param>
        /// <returns></returns>
        public List<RB_Employee_Extend> GetEmpInfoByIds(string ids)
        {
            if (ids != null && !string.IsNullOrEmpty(ids.Trim()))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendFormat(@" SELECT EmployeeId,EmName,EmPhoto,EmMobile,CreateTime,EmLoginMobile,RB_Branch_id,RB_Group_id,IsLeave,RB_Post_Id FROM {0} ", TableName);
                sb.AppendFormat("  WHERE {0} in ({1})", nameof(Model.Entity.RB_Employee.EmployeeId), ids);
                return Get<RB_Employee_Extend>(sb.ToString()).ToList();
            }
            return new List<RB_Employee_Extend>();
        }
    }
}
