﻿using Property.Model.Entity;
using Property.Model.Extend;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Property.Repository
{
    /// <summary>
    /// 岗位仓储类
    /// </summary>
    public partial class Rb_postRepository : RepositoryBase<RB_Post>
    {
        /// <summary>
        /// 获取岗位分页数据
        /// </summary>
        /// <param name="pageIndex"></param>
        /// <param name="pageSize"></param>
        /// <param name="dmodel"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetPageList(int pageIndex, int pageSize, RB_Post_Extend dmodel, out long count)
        {
            string where = " WHERE 1=1 ";
            where += " and p." + nameof(RB_Post_Extend.Status) + "=0";

            if (dmodel.RB_Department_Id > 0)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Department_Id) + "=" + dmodel.RB_Department_Id;
            }
            if (dmodel.RB_Branch_Id > -1)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Branch_Id) + "=" + dmodel.RB_Branch_Id;
            }
            if (!string.IsNullOrWhiteSpace(dmodel.PostName))
            {
                where += " and p." + nameof(RB_Post_Extend.PostName) + " like '%" + dmodel.PostName + "%'";
            }
            if (dmodel.RB_Group_Id > 0)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            return GetPage<RB_Post_Extend>(pageIndex, pageSize, out count, $"select p.*,d.DepartmentName,b.BName from Rb_post p left join rb_department d on p.RB_Department_Id=d.DepartmentID left join rb_branch b on p.RB_Branch_Id=b.Id {where} order by p.PostId desc").ToList();
        }

        /// <summary>
        /// 根据条件获取岗位列表
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetList(RB_Post_Extend dmodel)
        {
            string where = " WHERE 1=1 ";
            where += " and p." + nameof(RB_Post_Extend.Status) + "=" + (int)REBORN.Common.Enum.DateStateEnum.Normal;
            if (dmodel.RB_Department_Id > 0)
            {
                where += " and dpr.DepartmentId=" + dmodel.RB_Department_Id;
            }
            if (!string.IsNullOrWhiteSpace(dmodel.PostName))
            {
                where += " and p." + nameof(RB_Post_Extend.PostName) + " like '%" + dmodel.PostName + "%'";
            }
            if (dmodel.RB_Group_Id > 0)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            return Get<RB_Post_Extend>($"select distinct p.* from Rb_post p left join rb_departmentpost_relevance dpr on p.PostId=dpr.PostId {where}").ToList();
        }

        /// <summary>
        /// 获取所有部门 及 下所有岗位
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetDepartmentPostList(RB_Post_Extend dmodel)
        {
            string where = " WHERE 1=1 ";
            where += " and p." + nameof(RB_Post_Extend.Status) + "=" + (int)REBORN.Common.Enum.DateStateEnum.Normal;
            if (dmodel.RB_Department_Id > 0)
            {
                where += " and dpr.DepartmentId=" + dmodel.RB_Department_Id;
            }
            if (!string.IsNullOrWhiteSpace(dmodel.PostName))
            {
                where += " and p." + nameof(RB_Post_Extend.PostName) + " like '%" + dmodel.PostName + "%'";
            }
            if (dmodel.RB_Group_Id > 0)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            return Get<RB_Post_Extend>($"select dpr.Id as PostId,p.PostName,dpr.DepartmentId as RB_Department_Id,p.RB_Group_Id,p.RB_Branch_Id,p.CreateTime,p.CreateBy,p.UpdateTime,p.UpdateBy,p.`Status` from Rb_post p left join rb_departmentpost_relevance dpr on p.PostId=dpr.PostId {where}").ToList();
        }

        /// <summary>
        /// 根据部门岗位关联表id获取部门岗位下所有员工
        /// </summary>
        /// <param name="depPostRelevanceIds">部门岗位关联表id(1,2,3)</param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetEmpByDepPostRelevanceIds(string depPostRelevanceIds)
        {
            string sql = $@"SELECT p.* from rb_departmentpost_relevance r 
                            INNER JOIN rb_post p on r.PostId = p.PostId
                            where r.Id in ({depPostRelevanceIds}) and p.`Status` = 0";
            return Get<RB_Post_Extend>(sql).ToList();
        }

        /// <summary>
        /// 获取部门下岗位  并获取岗位下员工数量
        /// </summary>
        /// <param name="dmodel"></param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetDepartmentPostForAppList(RB_Post_Extend dmodel)
        {
            string where = " WHERE 1=1 ";
            where += " and p." + nameof(RB_Post_Extend.Status) + "=" + (int)REBORN.Common.Enum.DateStateEnum.Normal;
            if (dmodel.RB_Department_Id > 0)
            {
                where += " and dpr.DepartmentId=" + dmodel.RB_Department_Id;
            }
            if (!string.IsNullOrWhiteSpace(dmodel.PostName))
            {
                where += " and p." + nameof(RB_Post_Extend.PostName) + " like '%" + dmodel.PostName + "%'";
            }
            if (dmodel.RB_Group_Id > 0)
            {
                where += " and p." + nameof(RB_Post_Extend.RB_Group_Id) + "=" + dmodel.RB_Group_Id;
            }
            return Get<RB_Post_Extend>($"select distinct p.*,(SELECT Count(*)  from rb_employee e where e.RB_Department_Id = dpr.DepartmentId and e.IsLeave = 0) as EmployeeCount from Rb_post p left join rb_departmentpost_relevance dpr on p.PostId=dpr.PostId {where}").ToList();
        }

        /// <summary>
        /// 根据查询条件获取岗位列表
        /// </summary>
        /// <param name="where"></param>
        /// <returns></returns>
        public List<RB_Post_Extend> GetListByWhere(RB_Post_Extend where)
        {
            StringBuilder builder = new StringBuilder();
            builder.AppendFormat(" SELECT PostId, PostName FROM {0} WHERE 1=1 ", nameof(RB_Post));
            builder.AppendFormat(" AND {0}={1} ", nameof(RB_Post_Extend.Status), (int)REBORN.Common.Enum.DateStateEnum.Normal);
            if (where != null)
            {
                if (where.QDepartmentIds != null && !string.IsNullOrEmpty(where.QDepartmentIds))
                {
                    builder.AppendFormat(" AND {0} IN ({1}) ", nameof(RB_Post_Extend.RB_Department_Id), where.QDepartmentIds.TrimEnd(',').TrimStart(','));
                }
                if (where.QPostIds != null && !string.IsNullOrEmpty(where.QPostIds))
                {
                    builder.AppendFormat(" AND {0} IN ({1}) ", nameof(RB_Post_Extend.PostId), where.QPostIds.TrimEnd(',').TrimStart(','));
                }
            }
            return Get<RB_Post_Extend>(builder.ToString()).ToList();
        }
    }
}
