﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Module.UserModule;
using Property.Model.Extend;
using Property.CacheKey;
using REBORN.Common;
using Property.WebApi.Filter;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using Property.Model.Entity;
using System;
using System.Collections.Generic;
using JWT;
using JWT.Algorithms;
using JWT.Serializers;
using Microsoft.AspNetCore.Cors;

namespace Property.WebApi.Controllers.Login
{
    [Route("api/[controller]/[action]")]    
    [ApiExceptionFilter]    
    [ApiController]
    [EnableCors("AllowCors")]
    public class LoginController : ControllerBase
    {
        //员工
        private EmployeeModule loginModule = new EmployeeModule();
        //公司
        private BranchModule branchModule = new BranchModule();
        //部门
        private DepartmentModule departmentModule = new DepartmentModule();
        //岗位
        private PostModule postModule = new PostModule();

        /// <summary>
        /// 获取用户登录
        /// </summary>
        /// <param name="requestMsg"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult UserLogin(object requestMsg)
        {
            var requestParm = JsonConvert.DeserializeObject<RequestParm>(requestMsg.ToString());

            //throw new Exception("here is what was happening");
            JObject parm = JObject.Parse(requestParm.msg.ToString());
            string EmAccount = parm.GetStringValue("EmAccount");
            string EmPassword = parm.GetStringValue("EmPassword");
            string Domain = parm.GetStringValue("Domain");
            if (string.IsNullOrEmpty(EmAccount) || string.IsNullOrEmpty(EmPassword))
            {
                return ApiResult.ParamIsNull();
            }
            if (Domain.Contains("zc"))
            {
                Domain = Domain.Replace("zc", "");//资产管理专用
            }
            var gmodel = loginModule.GetGroupByDomain(new RB_Group_Extend { Domain = Domain }).FirstOrDefault();
            if (gmodel == null)
            {
                return ApiResult.Failed("集团域名不存在，登录失败");
            }

            EmPassword = REBORN.Common.DES.Encrypt(EmPassword, REBORN.Common.Config.WebApiKey, REBORN.Common.Config.WebApiIV);
            RB_Employee model = new RB_Employee();
            ApiResult result = UsreLoginCheckWeb(EmAccount, EmPassword, gmodel.Id, out model);
            if (result.resultCode == (int)ResultCode.Success)
            {
                if (model.RB_Group_id != gmodel.Id)
                {
                    return ApiResult.Failed("你不属于该集团，登录失败");
                }
                if (model.IsLeave == 1)
                {
                    return ApiResult.Failed("你已离职，不能登录");
                }
                #region 获取基础信息
                if (gmodel.Status == REBORN.Common.Enum.DateStateEnum.Delete)
                {
                    return ApiResult.Failed("你所属集团已被禁用，登录失败");
                }
                string GroupName = gmodel != null ? gmodel.GroupName : "";
                string GroupPic = gmodel != null ? Config.GetFileUrl(gmodel.Logo) : "";
                Domain = gmodel != null ? gmodel.Domain : "";
                string b2bDomain = gmodel != null ? gmodel.B2BDomain : "";
                int versions = gmodel != null ? Convert.ToInt32(gmodel.Versions_Id) : 0;
                string versionsName = "";
                string BranchName = "";
                var bmodel = new RB_Branch();
                if (model.IsAdmin != 1)
                {
                    bmodel = branchModule.BranchGet(model.RB_Branch_id);
                    if (bmodel != null && bmodel.Status == 1)
                    {
                        return ApiResult.Failed("你所属公司已被禁用，登录失败");
                    }
                    if (bmodel == null)
                    {
                        bmodel = new RB_Branch();
                    }
                    BranchName = bmodel != null ? bmodel.BName : "";
                }
                string DepartName = "";
                if (model.IsAdmin != 1)
                {
                    var dmodel = departmentModule.DepartmentGet(model.RB_Department_Id);
                    DepartName = dmodel != null ? dmodel.DepartmentName : "";
                }
                var pmodel = postModule.PostGet(model.RB_Post_Id);
                string PostName = pmodel != null ? pmodel.PostName : "";
                List<RB_Sys_Menu_Extend> userMenuList = new List<RB_Sys_Menu_Extend>();
                List<RB_Sys_Menu_Extend> resultUserMenuList = loginModule.GetUserMenuAllList(model.RB_Post_Id, model.EmployeeId, out userMenuList);
                //string userSysMenuStr = "";
                //if (userMenuList != null && userMenuList.Count > 0)
                //{
                //    List<string> sysMenuListStr = new List<string>();
                //    foreach (var item in userMenuList)
                //    {
                //        sysMenuListStr.Add(item.MenuUrl);
                //    }
                //    userSysMenuStr = string.Join(",", sysMenuListStr);
                //}
                //获取用户所有的功能菜单编码   ld 2019-1-2
                //var ActionMenuList = sysMenuModule.GetActionCodeForUrlAndUser(model.RB_Post_Id, "");
                #endregion
                TokenUserInfo userInfo = new TokenUserInfo { uid = model.EmployeeId.ToString(), requestFrom = REBORN.Common.Enum.ApiRequestFromEnum.ERP };
                string token = "";
                #region JWT
                IDateTimeProvider provider = new UtcDateTimeProvider();
                var now = provider.GetNow().AddMinutes(-1);
                var unixEpoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc); // or use JwtValidator.UnixEpoch
                var secondsSinceEpoch = Math.Round((now - unixEpoch).TotalSeconds);
                var payload = new Dictionary<string, object>
                                {
                                        {"iat",secondsSinceEpoch },
                                        {"exp",secondsSinceEpoch+Config.JwtExpirTime},
                                        {"userInfo",userInfo }
                                };
                IJwtAlgorithm algorithm = new HMACSHA256Algorithm();
                IJsonSerializer serializer = new JsonNetSerializer();
                IBase64UrlEncoder urlEncoder = new JwtBase64UrlEncoder();
                IJwtEncoder encoder = new JwtEncoder(algorithm, serializer, urlEncoder);
                string secret = Config.JwtSecretKey;
                token = encoder.Encode(payload, secret);
                //IM注册或更新
                //AccountModel imAccount = employeeModule.GetIM(model.EmployeeId);
                //Task.Run(() =>
                //{
                //    AccountModel imAccountModel = employeeModule.UpdateIM(model.EmployeeId, imAccount.AccID, model.EmName, Config.GetFileUrl(model.EmPhoto));
                //});
                #endregion
                #region 处理权限节点
                string actionMenuName = string.Empty;
                //if (ActionMenuList != null && ActionMenuList.Count() > 0)
                //{
                //    actionMenuName = string.Join(",", ActionMenuList?.Select(x => x.ActionCode).ToList());
                //}
                #endregion

                UserInfo obj = new UserInfo
                {

                    GroupName = GroupName,
                    GroupPic = GroupPic,
                    Domain = "zc"+ Domain,
                    VersionsName = versionsName,
                    BranchName = BranchName,
                    DepartName = DepartName,
                    PostName = PostName,
                    emName = model.EmName,
                    EmployeeId = model.EmployeeId,
                    RB_Group_id = model.RB_Group_id,
                    Versions_Id = versions,
                    RB_Branch_id = model.RB_Branch_id,
                    RB_Post_Id = model.RB_Post_Id,
                    RB_Department_Id = model.RB_Department_Id,
                    token = token,
                    SecretKey = model.SecretKey,
                    FileDomain = Config.GetOssFileUrl,
                    Icon = Config.GetFileUrl(model.EmPhoto),
                    ImAccount = model.EmLoginMobile,
                    //ImAccount = imAccount?.AccID ?? model.EmLoginMobile, //model.EmLoginMobile,
                    //ImToken = IMService.DefaultPassword(),
                    IsParentCompany = bmodel.IsParentCompany ?? 2,
                    B2BDomain = b2bDomain,
                    ActionMenuCode = actionMenuName
                };
                CacheManager.User.UserReidsCache.UserInfoSet(UserModuleCacheKeyConfig.USER_Login_Info + model.EmployeeId, obj, Config.JwtExpirTime);
                obj.UserMenu = resultUserMenuList;
                //obj.ActionMenu = ActionMenuList.Select(x => new
                //{
                //    x.ActionCode,
                //    x.ActionName,
                //    x.ActionStatus,
                //    x.MenuUrl,
                //    x.Remarks,
                //    x.Sort
                //});
                return ApiResult.Success(data: obj);
            }
            else
            {
                return result;
            }
        }
        /// <summary>
        /// 用户登陆验证
        /// </summary>
        /// <param name="account">账号</param>
        /// <param name="pwd">密码</param>
        /// <param name="RB_Group_id">集团编号</param>
        /// <param name="emp">员工</param>
        /// <returns></returns>
        public ApiResult UsreLoginCheckWeb(string account, string pwd, int RB_Group_id, out RB_Employee emp)
        {
            //先查询是否存在是否是登陆手机号登陆
            emp = loginModule.GetByLoginMobileAndPwd(account, pwd, RB_Group_id);
            if (emp == null)
            {
                //再查询是否存在是否是账号登陆
                emp = loginModule.GetNewByAccountAndPwd(account, pwd, RB_Group_id);
                if (emp == null)
                {
                    return ApiResult.Failed("账号或密码错误，登录失败！");
                }
                else
                {
                    if (string.IsNullOrWhiteSpace(emp.EmLoginMobile))
                    {
                        return new ApiResult() { resultCode = (int)ResultCode.AccountNotBindMobile, message = "账号未绑定手机号" };
                    }
                    else
                    {
                        return ApiResult.Failed("请用手机号码登录");
                    }
                }
            }
            else
            {
                return ApiResult.Success();
            }
        }
    }
}
