﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Property.Model.Entity.Mall;
using Property.Model.Extend;
using Property.Model.Extend.Mall;
using Property.Module.FixedAssets;
using Property.Module.Mall;
using REBORN.Common;
using REBORN.Common.API;
using REBORN.Common.Enum;
using REBORN.Common.Plugin;

namespace Property.WebApi.Controllers.Mall
{
    public class WarehouseOutController : PApiController
    {
        private readonly WarehouseOutModule warehouseOutModule = new WarehouseOutModule();

        private readonly SuppliesModule suppliesModule = new SuppliesModule();

        #region 审核申请单
        /// <summary>
        /// 获取出库申请列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetApplyPageList()
        {
            var parms = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(parms.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(parms.msg.ToString());
            RB_WarehouseOut_Apply_Extend demodel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(parms.msg.ToString());
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = -1;
            var list = warehouseOutModule.GetApplyPageList(pagelist.pageIndex, pagelist.pageSize, out long count, demodel);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.ApplyStatus,
                x.CreateBy,
                x.ExamineReason,
                x.ID,
                x.MallBaseId,
                x.MallBaseName,
                x.TenantId,
                x.TenantName,
                x.OutNo,
                x.Delivered,
                x.NotOutStock,
                x.OutStatus,
                CreateByName = CacheManager.User.UserReidsCache.GetUserLoginInfo(x.CreateBy.ToString())?.emName,
                CreateDateStr = x.CreateDate.Value.ToString("yyyy-MM-dd HH:mm:ss")
            });
            return ApiResult.Success("", pagelist);
        }


        /// <summary>
        /// 获取申请单详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetWarehouseOutApply()
        {
            var requestParm = GetRequestParm();
            RB_WarehouseOut_Apply_Extend demodel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(requestParm.msg.ToString());
            if (demodel.ID <= 0)
            {
                return ApiResult.Failed("请选择您要操作的数据");
            }
            var oldModel = warehouseOutModule.GetApplyEntity(demodel.ID).RefMapperTo<RB_WarehouseOut_Apply_Extend>();
            if (oldModel != null)
            {
                oldModel.WarehouseOutGoodsList = new List<RB_WarehouseOut_Goods_Extend>();
                oldModel.WarehouseOutGoodsList = warehouseOutModule.GetWarehouseOutList(new RB_WarehouseOut_Goods_Extend { ApplyId = demodel.ID });
            }

            return ApiResult.Success("", oldModel);
        }



        /// <summary>
        /// 审核出库申请
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult UpdateWarehouseOutApply()
        {
            var requestParm = GetRequestParm();
            RB_WarehouseOut_Apply_Extend demodel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(requestParm.msg.ToString());
            demodel.UpdateDate = System.DateTime.Now;
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            demodel.UpdateBy = userInfo.EmployeeId;
            if (demodel.ID <= 0)
            {
                return ApiResult.Failed("请选择您要操作的数据");
            }
            var oldModel = warehouseOutModule.GetApplyEntity(demodel.ID);
            if (oldModel.ApplyStatus != 1)
            {
                return ApiResult.Failed("申请单审核，请勿重复审核");
            }
            if (demodel.ApplyStatus == 3 && String.IsNullOrWhiteSpace(demodel.ExamineReason))
            {
                return ApiResult.Failed("拒绝必须填写理由");
            }
            bool result = warehouseOutModule.UpdateWarehouseOutApply(demodel);

            if (result)
            {
                return ApiResult.Success("操作成功");
            }
            else
            {
                return ApiResult.Failed("操作失败");

            }
        }
        #endregion



        #region 申请单出库


        /// <summary>
        /// 修改出库商品信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult UpdateOutGoods()
        {
            var parms = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(parms.uid);
            RB_WarehouseOut_Apply_Extend demodel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(parms.msg.ToString());
            //判断出库申请的状态

            var oldModel = warehouseOutModule.GetApplyEntity(demodel.ID).RefMapperTo<RB_WarehouseOut_Apply_Extend>();
            if (oldModel != null)
            {
                oldModel.WarehouseOutGoodsList = new List<RB_WarehouseOut_Goods_Extend>();
                oldModel.WarehouseOutGoodsList = warehouseOutModule.GetWarehouseOutList(new RB_WarehouseOut_Goods_Extend { ApplyId = demodel.ID });
            }
            if (oldModel.ApplyStatus != 1)
            {
                return ApiResult.Failed("审核通过/拒绝的申请单不能修改出库商品数量");
            }

            foreach (var item in demodel.WarehouseOutGoodsList)//判断商品是否属于当前出库单
            {
                if (oldModel.WarehouseOutGoodsList.Any(x => x.ID == item.ID))
                {
                    if (item.Number <= 0)
                    {
                        return ApiResult.Failed("出库商品数量必须大于0");
                    }
                }
                else
                {
                    return ApiResult.Failed("商品名称：" + item.GoodsName + "订单号：" + item.OrderNo + "不属于当前申请单");
                }
            }

            //判断商品规格是否属于该商品


            var result = warehouseOutModule.UpdateWarehouseOutGoods(demodel.WarehouseOutGoodsList);
            if (result)
            {
                return ApiResult.Success("", "修改成功");

            }
            else
            {
                return ApiResult.Failed("修改失败");
            }

        }




        /// <summary>
        /// 根据仓库id和申请id获取申请单中的商品
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetOuGoodsByWarehouseId()
        {
            var parms = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(parms.uid);
            RB_WarehouseOut_Apply_Extend demodel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(parms.msg.ToString());
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = -1;
            var list = warehouseOutModule.GetOuGoodsByWarehouseId(demodel);
            return ApiResult.Success("", list);
        }




        /// <summary>
        /// 保存出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetStockOutInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            DateTime NowStr = System.DateTime.Now;
            RB_WarehouseOut_Apply_Extend applyModel = JsonConvert.DeserializeObject<RB_WarehouseOut_Apply_Extend>(requestParm.msg.ToString());
            if (applyModel.WarehouseId <= 0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }

            if (applyModel.WarehouseOutGoodsDetailList == null && !applyModel.WarehouseOutGoodsDetailList.Any())
            {
                return ApiResult.Failed("请选择您要出库的商品");
            }


            //判断当前仓库是否在盘点中2020-07-28 Add By:W
            var suppliesCheckList = suppliesModule.GetSuppliesCheckList(new RB_Supplies_Check_Extend { RB_Group_Id = userInfo.RB_Group_id, StartDate = System.DateTime.Now, EndDate = System.DateTime.Now, WarehouseId = applyModel.WarehouseId, CheckState = REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory });
            if (suppliesCheckList != null && suppliesCheckList.Any())
            {
                return ApiResult.ParamIsNull("仓库盘点中，不能出库");
            }
            //查询之前的申请信息
            var oldModel = warehouseOutModule.GetApplyEntity(applyModel.ID).RefMapperTo<RB_WarehouseOut_Apply_Extend>();
            if (oldModel != null)
            {
                oldModel.WarehouseOutGoodsList = new List<RB_WarehouseOut_Goods_Extend>();
                oldModel.WarehouseOutGoodsList = warehouseOutModule.GetWarehouseOutList(new RB_WarehouseOut_Goods_Extend { ApplyId = applyModel.ID });
            }
            if (oldModel.ApplyStatus != 2)
            {
                return ApiResult.Failed("出库申请单未通过，请勿操作出库");
            }
            if (oldModel.OutStatus == 2)
            {
                return ApiResult.Failed("出库单中的商品已全部出库");
            }

            #region 财务记录
            RB_Finance_Record_Extend record = new RB_Finance_Record_Extend();
            record.MallBaseId = applyModel.MallBaseId;
            record.TenantId = applyModel.TenantId;
            record.Type = 2;
            record.IsFinanceAll = 1;
            record.CreateDate = System.DateTime.Now;
            record.RecordDetailList = new List<RB_Finance_RecordDetail>();

            #endregion


            //判断此次商品是否已出库与数量是否相等
            applyModel.WarehouseOutGoodsList = new List<RB_WarehouseOut_Goods_Extend>();
            // var goodsList = applyModel.WarehouseOutGoodsDetailList.GroupBy(x => x.OrderGoodsId);
            var goodsList = applyModel.WarehouseOutGoodsDetailList.GroupBy(x => new { x.OrderGoodsId, x.GoodsId, x.SpecificationSort });
            oldModel.RB_Group_Id = userInfo.RB_Group_id;
            applyModel.RB_Branch_Id = oldModel.RB_Branch_Id;
            applyModel.RB_Group_Id = oldModel.RB_Group_Id;
            oldModel.RB_Branch_Id = -1;

            oldModel.WarehouseId = applyModel.WarehouseId;
            var wareHouseGoodsList = warehouseOutModule.GetOuGoodsByWarehouseId(oldModel);
            foreach (var item in goodsList)
            {
                var oldOrderGoods = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewGoodsId == item.Key.GoodsId && x.NewSpecificationSort == item.Key.SpecificationSort).FirstOrDefault();
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.OrderId = oldOrderGoods.OrderId);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.GoodsId = oldOrderGoods.NewGoodsId);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.GoodsName = oldOrderGoods.NewGoodsName);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.Specification = oldOrderGoods.NewSpecification);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.SpecificationSort = oldOrderGoods.NewSpecificationSort);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.ProductCode = oldOrderGoods.ProductCode);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.TenantId = oldOrderGoods.TenantId);
                applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).ToList().ForEach(x => x.MallBaseId = oldOrderGoods.MallBaseId);


                if (oldOrderGoods == null)
                {
                    var nowOrderGoods = applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort).FirstOrDefault();
                    return ApiResult.Failed("商品不属于出库单，请核实后再出库");
                }
                else if (oldOrderGoods.IsOut == 1)
                {
                    return ApiResult.Failed("订单号：" + oldOrderGoods.OrderNo + "商品名称：" + oldOrderGoods.GoodsName + "申请单中已出库");
                }
                var nowGoodsList = applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.GoodsId == item.Key.GoodsId && x.SpecificationSort == item.Key.SpecificationSort);
                int goodsCount = nowGoodsList.Count();
                if (oldOrderGoods.NewNumber != goodsCount)
                {
                    return ApiResult.Failed("订单号：" + oldOrderGoods.OrderNo + "商品名称：" + oldOrderGoods.GoodsName + "必须一起出库");
                }
                if (!wareHouseGoodsList.Any(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewGoodsId == item.Key.GoodsId && x.NewSpecificationSort == item.Key.SpecificationSort))
                {
                    return ApiResult.Failed("订单号：" + oldOrderGoods.OrderNo + "商品名称：" + oldOrderGoods.GoodsName + "没在当前仓库");
                }

                oldOrderGoods.CostMoney = nowGoodsList.Sum(x => x.CostMoney ?? 0);
                applyModel.WarehouseOutGoodsList.Add(oldOrderGoods);

                RB_Finance_RecordDetail financeRecordDetail = new RB_Finance_RecordDetail
                {
                    ID = 0,
                    RecordId = 0,
                    Type = record.Type,
                    TenantId = record.TenantId,
                    MallBaseId = record.MallBaseId,
                    FinanceId = 0,
                    CreateDate = record.CreateDate,
                    Name = oldOrderGoods.GoodsName,
                    OrderId = oldOrderGoods.OrderId,
                    OrderDetailId = oldOrderGoods.OrderGoodsId,
                    GoodsPrice = oldOrderGoods.CostMoney ?? 0,
                    FreightMoney = 0,
                    Unit_Price = Math.Round((oldOrderGoods.CostMoney ?? 0) / (oldOrderGoods.NewNumber), 2, MidpointRounding.AwayFromZero),
                    Number = oldOrderGoods.NewNumber
                };
                record.RecordDetailList.Add(financeRecordDetail);
            }

            //2020-07-29 Add By:W 判断同一个订单商品拆分后的商品是否是一起出库

            //var orderGoodsList = applyModel.WarehouseOutGoodsDetailList.GroupBy(x => x.OrderGoodsId);
            //foreach (var item in orderGoodsList)
            //{
            //    var oldOrderGoodsList = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key);
            //    var outGoodsList = applyModel.WarehouseOutGoodsDetailList.Where(x=>x.OrderGoodsId==item.Key);
            //    if (oldOrderGoodsList.Count()!= outGoodsList.Count())
            //    {
            //        return ApiResult.Failed("订单号：" + oldOrderGoodsList.FirstOrDefault()?.OrderNo + "中的商品必须一起出库");
            //    }
            //}

            if (oldModel.WarehouseOutGoodsList.Where(x => x.IsOut == 0).Count() == wareHouseGoodsList.Count())
            {
                applyModel.OutStatus = 2;
            }
            else if (oldModel.WarehouseOutGoodsList.Where(x => x.IsOut == 0).Count() > wareHouseGoodsList.Count())
            {
                applyModel.OutStatus = 1;
            }
            else
            {
                return ApiResult.Failed("已出库的商品类别数大于出库单的商品类别数");
            }

            //判断出库商品是否是同一个仓库


            #region 出库记录信息  主要是针对电商
            applyModel.WarehouseOutGoodsDetailList.ForEach(x => x.ApplyId = applyModel.ID);//将申请赋值给出库商品详情表
            applyModel.WarehouseOutGoodsDetailList.ForEach(x => x.WarehouseId = applyModel.WarehouseId);//将申请赋值给出库商品详情表
            applyModel.WarehouseOutFinanceModel = new Model.Entity.Mall.RB_WarehouseOut_Finance();//财务id ，后面更新出库id与财务id
            applyModel.WarehouseOutFinanceModel.ApplyId = applyModel.ID;
            applyModel.WarehouseOutFinanceModel.Status = 0;
            applyModel.WarehouseOutFinanceModel.TenantId = oldModel.TenantId;
            applyModel.WarehouseOutFinanceModel.MallBaseId = oldModel.MallBaseId;
            applyModel.WarehouseOutFinanceModel.CreateDate = NowStr;
            applyModel.WarehouseOutFinanceModel.CreateBy = userInfo.EmployeeId;
            applyModel.WarehouseOutFinanceModel.RB_Branch_Id = applyModel.RB_Branch_Id;
            applyModel.WarehouseOutFinanceModel.RB_Group_Id = applyModel.RB_Group_Id;
            #endregion


            #region 出库记录信息
            RB_Supplies_StockOut_Extend demodel = new RB_Supplies_StockOut_Extend();//出库申请表，等会儿组装

            demodel.WarehouseId = applyModel.WarehouseId;
            demodel.EmployeeId = oldModel.CreateBy;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.StockOutDate = NowStr;
            demodel.StockOutState = REBORN.Common.Enum.StockOutStatusEnum.StockOut;
            demodel.Remark = "电商订单商品出库";
            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateDate = NowStr;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = NowStr;

            UserInfo userInfoLQR = CacheManager.User.UserReidsCache.GetUserLoginInfo(oldModel.CreateBy.ToString());
            demodel.DepartmentId = userInfoLQR.RB_Department_Id;
            demodel.BranchId = userInfoLQR.RB_Branch_id;


            demodel.DetailList = new List<RB_Supplies_StockOutDetail_Extend>();

            foreach (var item in applyModel.WarehouseOutGoodsDetailList.GroupBy(x => x.MaterialId))
            {
                var oldOrderGoods = wareHouseGoodsList.Where(x => x.SuppliesId == item.Key).FirstOrDefault();
                if (!wareHouseGoodsList.Any(x => x.SuppliesId == item.Key))
                {

                    return ApiResult.Failed("出库商品的规格与出库规格不一致");
                }
                RB_Supplies_StockOutDetail_Extend stockOutDetailModel = new RB_Supplies_StockOutDetail_Extend
                {

                    Id = 0,
                    StockOutId = 0,
                    SuppliesId = item.Key,
                    Number = applyModel.WarehouseOutGoodsDetailList.Where(x => x.MaterialId == item.Key).Count(),
                    Remark = "电商订单商品出库",
                    Status = 0,
                    RB_Branch_Id = userInfo.RB_Branch_id,
                    RB_Group_Id = userInfo.RB_Group_id,
                    CreateBy = userInfo.EmployeeId,
                    CreateDate = NowStr,
                    UpdateBy = userInfo.EmployeeId,
                    UpdateDate = NowStr,
                    Money = applyModel.WarehouseOutGoodsDetailList.Where(x => x.MaterialId == item.Key).Sum(x => x.CostMoney ?? 0),

                };
                demodel.DetailList.Add(stockOutDetailModel);
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择物料");
            }
            if (!demodel.StockOutDate.HasValue)
            {
                return ApiResult.ParamIsNull("请输入出库日期");
            }
            if (demodel.EmployeeId <= 0)
            {
                return ApiResult.ParamIsNull("请选择领用人");
            }
            foreach (var item in demodel.DetailList)
            {
                if (item.SuppliesId <= 0)
                {
                    return ApiResult.ParamIsNull("请传递正确的物料id");
                }
                if (item.Number <= 0)
                {
                    return ApiResult.ParamIsNull("请输入出库数量");
                }
            }
            if (demodel.DetailList.Any())
            {
                var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                foreach (var item in slist)
                {
                    if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1)
                    {
                        return ApiResult.Failed("每种耗材只能选择一个");
                    }
                }
            }
            else
            {
                return ApiResult.Failed("请选择耗材");
            }
            //验证库存是否足够
            if (!suppliesModule.ValidateStockOutInsert(demodel))
            {
                return ApiResult.Failed("库存不足无法出库，请核实后再试");
            }

            #endregion


            #region 组装财务单据需要的数据
            List<RB_WarehouseOut_Goods_Extend> financeList = new List<RB_WarehouseOut_Goods_Extend>();
            foreach (var item in applyModel.WarehouseOutGoodsDetailList.GroupBy(x => new { x.OrderGoodsId, x.CostMoney,x.GoodsId,x.SpecificationSort }))
            {
                RB_WarehouseOut_Goods_Extend warehouseOutModel = new RB_WarehouseOut_Goods_Extend
                {

                    ApplyId = applyModel.ID,
                    ID = 0,
                    OrderId = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId).FirstOrDefault()?.OrderId ?? 0,
                    GoodsId = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId).FirstOrDefault()?.GoodsId ?? 0,
                    Status = 0,
                    IsOut = 1,
                    OrderGoodsId = item.Key.OrderGoodsId,
                    OrderNo = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.OrderNo ?? "",
                    GoodsName = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId&&x.NewSpecificationSort== item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.GoodsName,
                    Specification = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.Specification,
                    SpecificationSort = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.SpecificationSort,
                    NewGoodsName = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.NewGoodsName,
                    NewSpecification = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.NewSpecification,
                    NewSpecificationSort = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.NewSpecificationSort,
                    ProductCode = oldModel.WarehouseOutGoodsList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.NewSpecificationSort == item.Key.SpecificationSort && x.NewGoodsId == item.Key.GoodsId).FirstOrDefault()?.ProductCode,
                    Number = applyModel.WarehouseOutGoodsDetailList.Where(x => x.OrderGoodsId == item.Key.OrderGoodsId && x.SpecificationSort == item.Key.SpecificationSort && x.GoodsId == item.Key.GoodsId && x.CostMoney == item.Key.CostMoney).Count(),
                    TenantId = applyModel.TenantId,
                    MallBaseId = applyModel.MallBaseId,
                    CostMoney = item.Key.CostMoney ?? 0
                };
                financeList.Add(warehouseOutModel);
            }
            #endregion

            #region 获取出库申请下这批次已出库商品的信息
            string orderGoodsIds = string.Join(",", applyModel.WarehouseOutGoodsDetailList.GroupBy(x => x.OrderGoodsId).Select(x => x.Key));
            var outGoodsDetailList = warehouseOutModule.GetApplyByOrderGoodsIdList(new RB_WarehouseOut_GoodsDetail { TenantId = applyModel.TenantId, MallBaseId = applyModel.MallBaseId, ApplyId = applyModel.ID }, orderGoodsIds);
            #endregion 
            bool flag = warehouseOutModule.SetStockOutInfo(applyModel, demodel, financeList, record, userInfo.RB_Department_Id, oldModel.WarehouseOutGoodsList, outGoodsDetailList);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }



        #endregion



        #region 获取盘点中的仓库

        public ApiResult GetSuppliesCheckList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            //判断当前仓库是否在盘点中2020-07-28 Add By:W
            var suppliesCheckList = suppliesModule.GetSuppliesCheckList(new RB_Supplies_Check_Extend { RB_Group_Id = userInfo.RB_Group_id, StartDate = System.DateTime.Now, EndDate = System.DateTime.Now, CheckState = SuppliesCheckStatusEnum.InTheInventory });
            return ApiResult.Success("", suppliesCheckList);
        }

        #endregion
    }
}