﻿
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using REBORN.Common.Plugin;
using Property.Module.FixedAssets;
using System;
using System.IO;
using Property.WebApi.Filter;
using Microsoft.AspNetCore.Cors;
using Property.Model.Extend;
using System.Collections.Generic;
using System.Linq;
using REBORN.Common;
using Newtonsoft.Json;

namespace Property.WebApi.Controllers.User
{
    [Route("api/[controller]/[action]")]
    [ApiExceptionFilter]
    [ApiController]
    [EnableCors("AllowCors")]
    public class FileController : ControllerBase
    {
        SuppliesModule suppliesModule = new SuppliesModule();


        /// <summary>
        /// 上传文件解析文件内容
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult LocalFileUploadImport()
        {
            try
            {
                var files = Request.Form.Files;
                if (files.Count == 0)
                {
                    return new ApiResult { resultCode = (int)ResultCode.Fail, message = "未选择文件", data = "" };
                }
                
                string filename = files[0].FileName;
                string fileExtention = System.IO.Path.GetExtension(files[0].FileName);
                string path = Guid.NewGuid().ToString() + fileExtention; 
                string basepath = AppContext.BaseDirectory;
                string path_server = basepath + "\\upfile\\" + path;
                if (!Directory.Exists(basepath + "\\upfile"))
                {
                    Directory.CreateDirectory(basepath + "\\upfile");
                }
                using (FileStream fstream = new FileStream(path_server, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    files[0].CopyTo(fstream);
                }

                //处理文件内容读取
                List<RB_Property_Info_Extend> list = REBORN.DataHelper.PropertyBetchDataHelper.ImportClaimData(path_server);
                list = list.Where(x => !string.IsNullOrWhiteSpace(x.Name)).ToList();
                int Num = 1;
                list.ForEach(x =>
                {
                    x.Id = Num;
                    Num++;
                });
                System.IO.File.Delete(path_server);
                return ApiResult.Success("", list.Select(x => new
                {
                    x.Id,
                    x.Name,
                    x.CategoryName,
                    x.BrandName,
                    x.PropertyModel,
                    x.Units,
                    x.SerialNumber,
                    x.EmName,
                    x.StartTime,
                    x.EndTime,
                    x.Money,
                    x.UseMonths,
                    x.BranchName,
                    x.SupplierName
                }));
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "LocalFileUploadImport");
                return ApiResult.Failed();
            }
        }


        /// <summary>
        /// 上传盘点
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult LocalFileUploadImportForSuppliesCheck()
        {
            try
            {                
                var files = Request.Form.Files;
                if (files.Count == 0)
                {
                    return new ApiResult { resultCode = (int)ResultCode.Fail, message = "未选择文件", data = "" };
                }

                string filename = files[0].FileName;
                string fileExtention = System.IO.Path.GetExtension(files[0].FileName);
                string path = Guid.NewGuid().ToString() + fileExtention;
                string basepath = AppContext.BaseDirectory;
                string path_server = basepath + "\\upfile\\" + path;
                if (!Directory.Exists(basepath + "\\upfile"))
                {
                    Directory.CreateDirectory(basepath + "\\upfile");
                }
                using (FileStream fstream = new FileStream(path_server, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    files[0].CopyTo(fstream);
                }

                //处理文件内容读取
                List<RB_Supplies_CheckDetail_Extend> list = REBORN.DataHelper.SuppliesCheckBetchDataHelper.ImportClaimData(path_server);
                System.IO.File.Delete(path_server);
                var model = list.FirstOrDefault();
                var dmodel = suppliesModule.GetSuppliesCheckDetailInfo(model.Id);
                if (dmodel == null) { return ApiResult.Failed("盘点不存在"); }
                var pmodel = suppliesModule.GetSuppliesCheck(dmodel.CheckId ?? 0);
                if (pmodel == null) { return ApiResult.Failed("盘点不存在"); }
                if (pmodel.CheckState != REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory) {
                    return ApiResult.Failed("盘点已结束");
                }

                UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo((model.CheckEmpId ?? 0).ToString());

                //更新盘点数量
                bool flag = suppliesModule.SetBatchSuppliesCheckDetailReality(list, userInfo, 2);
                if (flag)
                {
                    return ApiResult.Success();
                }
                else {
                    return ApiResult.Failed();
                }
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, "LocalFileUploadImportForSuppliesCheck");
                return ApiResult.Failed();
            }
        }

    }
}
