﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Model.Extend;
using REBORN.Common;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using Property.Module.FixedAssets;
using System;
using System.Collections.Generic;
using Property.Model.Extend.Supplies;
using Google.Protobuf.WellKnownTypes;

namespace Property.WebApi.Controllers.User
{
    public class SuppliesController : PApiController
    {
        readonly SuppliesModule suppliesModule = new SuppliesModule();

        #region 基础配置

        /// <summary>
        /// 获取物料档案分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetMaterialPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Material_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Material_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetMaterialPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                x.Units,
                x.ImageList,
                x.Remark,
                x.SupplierName,
                x.Barcode,
                x.ReservedStock,
                x.SafetyStock,
                x.SuppliesModel,
                x.SuppliesNum,
                x.InventoryNum,
                x.GoodsSpecificationList,
                x.GoodsCategoryList,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取物料详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetMaterialInfo()
        {
            var requestParm = GetRequestParm();
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int MaterialId = parms.GetInt("MaterialId", 0);
            if (MaterialId <= 0)
            {
                return ApiResult.ParamIsNull();
            }

            var model = suppliesModule.GetMaterialInfo(MaterialId);
            return ApiResult.Success("", new
            {
                model.Id,
                model.Name,
                model.CategoryId,
                model.CategoryName,
                model.BrandName,
                model.Money,
                model.Units,
                model.ImageList,
                model.Remark,
                model.SupplierName,
                model.Barcode,
                model.ReservedStock,
                model.SafetyStock,
                model.SuppliesModel,
                model.SuppliesNum
            });
        }

        /// <summary>
        /// 设置物料档案
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetMaterialInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Material_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Material_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入物料名称");
            }
            if ((demodel.CategoryId ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择物料分类");
            }
            if (string.IsNullOrEmpty(demodel.BrandName))
            {
                return ApiResult.ParamIsNull("请输入品牌信息");
            }
            if (!demodel.ReservedStock.HasValue)
            {
                return ApiResult.ParamIsNull("请输入预留库存数量");
            }
            if (!demodel.SafetyStock.HasValue)
            {
                return ApiResult.ParamIsNull("请输入安全库存数量");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = suppliesModule.SetMaterialInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除物料档案
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelMaterialInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int MaterialId = parms.GetInt("MaterialId", 0);
            if (MaterialId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.DelMaterialInfo(MaterialId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取仓库分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetWareHousePageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());
            demodel.MallBaseId = requestParm.MallBaseId;
            demodel.TenantId = requestParm.TenantId;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetWareHousePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取所有仓库列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetWareHouseList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());
            demodel.MallBaseId = requestParm.MallBaseId;
            demodel.TenantId = requestParm.TenantId;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetWareHouseList(demodel);

            var robj = list.Select(x => new
            {
                x.Id,
                x.Name
            });
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 设置仓库信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetWareHouseInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_WareHouse_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_WareHouse_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入物料仓库名称");
            }

            demodel.MallBaseId = requestParm.MallBaseId;
            demodel.TenantId = requestParm.TenantId;
            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = suppliesModule.SetWareHouseInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除物料仓库
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelWareHouseInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int WareHouseId = parms.GetInt("WareHouseId", 0);
            if (WareHouseId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.DelWareHouseInfo(WareHouseId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取入库单列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetStockInPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetStockInPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.StockInState,
                StockInStateStr = x.StockInState.GetEnumName(),
                StockInDate = x.StockInDate.HasValue ? x.StockInDate.Value.ToString("yyyy-MM-dd") : "",
                x.StockInNum,
                x.WareHouseName,
                x.WarehouseId,
                x.Remark,
                x.Money,
                x.SupplierName,
                x.FinanceId,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.UnitPrice,
                    z.Money,
                    z.Number,
                    z.SuppliesName,
                    z.SuppliesNum,
                    z.BrandName,
                    z.SuppliesModel,
                    z.SuppliesId,
                    z.Units,
                    z.Remark,
                    z.GoodsSpecificationList,
                    z.GoodsCategoryList
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 入库单下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetStockInPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"入库状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"入库编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"入库时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"供应商名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单价"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetStockInPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.StockInState.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.StockInNum){ Rowspan=Count},
                                    new ExcelColumn(value:item.StockInDate.HasValue?item.StockInDate.Value.ToString("yyyy-MM-dd"):""){ Rowspan=Count},
                                    new ExcelColumn(value:item.WareHouseName){ Rowspan=Count},
                                    new ExcelColumn(value:item.SupplierName){ Rowspan=Count},
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.UnitPrice??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Money??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.UnitPrice??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Money??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion                
                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材入库单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材入库单.xls");
            }

        }

        /// <summary>
        /// 设置入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetStockInInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            if (demodel.WarehouseId <= 0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择物料");
            }
            if (!demodel.StockInDate.HasValue)
            {
                return ApiResult.ParamIsNull("请输入入库日期");
            }
            foreach (var item in demodel.DetailList)
            {
                if (item.SuppliesId <= 0)
                {
                    return ApiResult.ParamIsNull("请传递正确的物料id");
                }
                if (!item.Money.HasValue)
                {
                    return ApiResult.ParamIsNull("请输入入库金额");
                }
                if (item.Number <= 0)
                {
                    return ApiResult.ParamIsNull("请输入入库数量");
                }
            }


            //判断当前仓库是否在盘点中2020-07-28 Add By:W
            var suppliesCheckList = suppliesModule.GetSuppliesCheckList(new RB_Supplies_Check_Extend { RB_Group_Id = userInfo.RB_Group_id, StartDate = System.DateTime.Now, EndDate = System.DateTime.Now, WarehouseId = demodel.WarehouseId ?? 0, CheckState = REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory });
            if (suppliesCheckList != null && suppliesCheckList.Any())
            {
                return ApiResult.ParamIsNull("仓库盘点中，不能入库");
            }
            if (demodel.DetailList.Any())
            {
                var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                foreach (var item in slist)
                {
                    if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1)
                    {
                        return ApiResult.Failed("每种耗材只能选择一个");
                    }
                }
            }
            else
            {
                return ApiResult.Failed("请选择耗材");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;

            bool flag = suppliesModule.SetStockInInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelStockInInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model = suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("入库单已被删除");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockInWarting)
            {
                return ApiResult.Failed("入库单已完成，无法删除");
            }

            bool flag = suppliesModule.DelStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 取消入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelStockInInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            //判断状态  是否是待入库
            var model = suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("入库单已被删除");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockIn)
            {
                return ApiResult.Failed("入库单未入库，无法取消");
            }
            //验证库存是否足够减少
            if (!suppliesModule.ValidateStockInDel(StockInId))
            {
                return ApiResult.Failed("库存不足，请核实后再试");
            }

            bool flag = suppliesModule.CancelStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 恢复入库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult RecoverStockInInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockInId = parms.GetInt("StockInId", 0);
            if (StockInId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model = suppliesModule.GetStockInInfo(StockInId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("入库单已被删除，无法恢复");
            }
            if (model.StockInState != REBORN.Common.Enum.StockInStatusEnum.StockInWarting)
            {
                return ApiResult.Failed("入库单已完成，无法修改");
            }

            bool flag = suppliesModule.RecoverStockInInfo(StockInId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取出库单分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetStockOutPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Supplies_StockOut_Extend> list = suppliesModule.GetStockOutPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.StockOutState,
                StockOutStateStr = x.StockOutState.GetEnumName(),
                StockOutDate = x.StockOutDate.HasValue ? x.StockOutDate.Value.ToString("yyyy-MM-dd") : "",
                x.StockOutNum,
                x.WareHouseName,
                x.WarehouseId,
                x.Remark,
                x.EmployeeId,
                EmName = CacheManager.User.UserReidsCache.GetEmployee(x.EmployeeId)?.EmName ?? "",
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.Number,
                    z.SuppliesName,
                    z.SuppliesNum,
                    z.BrandName,
                    z.SuppliesModel,
                    z.SuppliesId,
                    z.Units,
                    z.Remark,
                    z.InventoryNum,
                    z.GoodsSpecificationList,
                    z.GoodsCategoryList
                }),
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 出库单下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetStockOutPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"出库状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"出库编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"出库时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"库存"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                List<RB_Supplies_StockOut_Extend> list = suppliesModule.GetStockOutPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.StockOutState.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.StockOutNum){ Rowspan=Count},
                                    new ExcelColumn(value:item.StockOutDate.HasValue?item.StockOutDate.Value.ToString("yyyy-MM-dd"):""){ Rowspan=Count},
                                    new ExcelColumn(value:item.WareHouseName){ Rowspan=Count},
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.InventoryNum??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.SuppliesNum){ },
                                    new ExcelColumn(value:citem.SuppliesName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(citem.InventoryNum??0).ToString()){ },
                                    new ExcelColumn(value:(citem.Number??0).ToString())
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion                
                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材出库单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材出库单.xls");
            }

        }

        /// <summary>
        /// 保存出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetStockOutInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOut_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOut_Extend>(requestParm.msg.ToString());

            if (demodel.WarehouseId <= 0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择物料");
            }
            if (!demodel.StockOutDate.HasValue)
            {
                return ApiResult.ParamIsNull("请输入入库日期");
            }
            if (demodel.EmployeeId <= 0)
            {
                return ApiResult.ParamIsNull("请选择领用人");
            }
            foreach (var item in demodel.DetailList)
            {
                if (item.SuppliesId <= 0)
                {
                    return ApiResult.ParamIsNull("请传递正确的物料id");
                }
                if (item.Number <= 0)
                {
                    return ApiResult.ParamIsNull("请输入入库数量");
                }
            }
            //判断当前仓库是否在盘点中2020-07-28 Add By:W
            var suppliesCheckList = suppliesModule.GetSuppliesCheckList(new RB_Supplies_Check_Extend { RB_Group_Id = userInfo.RB_Group_id, StartDate = System.DateTime.Now, EndDate = System.DateTime.Now, WarehouseId = demodel.WarehouseId ?? 0, CheckState = REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory });
            if (suppliesCheckList != null && suppliesCheckList.Any())
            {
                return ApiResult.ParamIsNull("仓库盘点中，不能出库");
            }
            if (demodel.DetailList.Any())
            {
                var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                foreach (var item in slist)
                {
                    if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1)
                    {
                        return ApiResult.Failed("每种耗材只能选择一个");
                    }
                }
            }
            else
            {
                return ApiResult.Failed("请选择耗材");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;//暂时这么赋值   看是否需要前端自行选择
            demodel.DepartmentId = emodel.RB_Department_Id;

            //验证库存是否足够
            if (!suppliesModule.ValidateStockOutInsert(demodel))
            {
                return ApiResult.Failed("库存不足无法出库，请核实后再试");
            }

            bool flag = suppliesModule.SetStockOutInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelStockOutInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockOutId = parms.GetInt("StockOutId", 0);
            if (StockOutId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //判断状态  是否是待入库
            var model = suppliesModule.GetStockOutInfo(StockOutId);
            if (model.Status == 1)
            {
                return ApiResult.Failed("出库单已被删除");
            }
            if (model.StockOutState != REBORN.Common.Enum.StockOutStatusEnum.StockOutCancel)
            {
                return ApiResult.Failed("出库单未取消，无法删除");
            }

            bool flag = suppliesModule.DelStockOutInfo(StockOutId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 取消出库单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelStockOutInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int StockOutId = parms.GetInt("StockOutId", 0);
            if (StockOutId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = suppliesModule.CancelStockOutInfo(StockOutId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取库存分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetInventoryPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetInventoryPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Number,
                x.Money,
                UnitPrice = x.Number == 0 ? 0 : Math.Round((x.Money ?? 0) / (x.Number ?? 1), 2, MidpointRounding.AwayFromZero),
                SuppliesName = x.SModel.Name,
                x.SuppliesId,
                x.WarehouseId,
                x.SModel.SuppliesNum,
                x.SModel.CategoryName,
                x.SModel.SuppliesModel,
                x.SModel.BrandName,
                x.SModel.Units,
                x.SModel.SafetyStock,
                x.SModel.ReservedStock,
                x.SModel.GoodsSpecificationList,
                x.SModel.GoodsCategoryList,
                WareHouseName = x.WModel.Name
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 物料库存统计下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetInventoryPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"仓库名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单位"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"安全库存"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预留库存"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单价"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"库存数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetInventoryPageList(1, 999999, demodel, out long count);

                #region 组装数据
                int TotalNum = 0;
                foreach (var item in list)
                {
                    TotalNum += (item.Number ?? 0);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.SModel.SuppliesNum){ },
                            new ExcelColumn(value:item.SModel.Name){ },
                            new ExcelColumn(value:item.SModel.CategoryName){ },
                            new ExcelColumn(value:item.SModel.SuppliesModel){ },
                            new ExcelColumn(value:item.WModel.Name){ },
                            new ExcelColumn(value:item.SModel.BrandName){ },
                            new ExcelColumn(value:item.SModel.Units){ },
                            new ExcelColumn(value:(item.SModel.SafetyStock??0).ToString()){ },
                            new ExcelColumn(value:(item.SModel.ReservedStock??0).ToString()){ },
                            new ExcelColumn(value:(item.Number == 0 ? 0 : Math.Round((item.Money ?? 0) / (item.Number ?? 1), 2, MidpointRounding.AwayFromZero)).ToString()){ },
                            new ExcelColumn(value:(item.Number??0).ToString()){ },
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计（共"+list.Count()+"条）"){ Colspan=10},
                            new ExcelColumn(value:TotalNum.ToString("#0")){},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "物料库存统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "物料库存统计.xls");
            }

        }

        /// <summary>
        /// 获取库存明细分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetInventoryDetailPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Inventory_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Inventory_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetInventoryDetailPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Type,
                TypeName = x.Type == 1 ? "增加" : "减少",
                x.Number,
                x.Money,
                x.Description,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.CreateBy)?.EmName ?? "",
                UpdateDate = x.CreateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 耗材领用查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetRecipientsQueryPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_StockOutDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOutDetail_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetRecipientsQueryPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);

            RB_Supplies_StockOutDetail_Extend Total = suppliesModule.GetRecipientsQueryTotal(demodel);//统计
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Number,
                x.Money,
                x.BranchName,
                x.DepartmentName,
                x.EmName,
                x.CategoryName,
                x.SuppliesName,
                x.StockOutNum,
                x.GoodsSpecificationList,
                x.GoodsCategoryList,
                StockOutDate = x.StockOutDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", new
            {
                pagelist,
                Total = new
                {
                    Total.Number,
                    Total.Money
                }
            });
        }

        /// <summary>
        /// 耗材领用统计下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetRecipientsQueryPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockOutDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockOutDetail_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"领取单号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"公司名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"部门名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领取日期"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领取数量"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetRecipientsQueryPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int TotalNum = 0;
                decimal TotalMoney = 0;
                foreach (var item in list)
                {
                    TotalNum += (item.Number ?? 0);
                    TotalMoney += (item.Money ?? 0);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.StockOutNum){ },
                            new ExcelColumn(value:item.BranchName){ },
                            new ExcelColumn(value:item.DepartmentName){ },
                            new ExcelColumn(value:item.EmName){ },
                            new ExcelColumn(value:item.StockOutDate.Value.ToString("yyyy-MM-dd")){ },
                            new ExcelColumn(value:item.SuppliesName){ },
                            new ExcelColumn(value:item.CategoryName){ },
                            new ExcelColumn(value:(item.Number??0).ToString()){ },
                            new ExcelColumn(value:(item.Money??0).ToString()){ }
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计（共"+list.Count()+"条）"){ Colspan=8},
                            new ExcelColumn(value:TotalNum.ToString("#0")){},
                            new ExcelColumn(value:TotalMoney.ToString("#0.00")){},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材领用统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材领用统计.xls");
            }

        }
        #endregion

        #region 耗材申请日期

        /// <summary>
        /// 获取耗材申请设置分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetApplyForSetPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Applyfor_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Applyfor_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Supplies_Applyfor_Extend> list = suppliesModule.GetApplyForSetPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.BranchId,
                BranchName = x.BranchId == -1 ? "通用" : CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                x.Enabled,
                WeekdayList = x.WeekdayList.OrderBy(x => x.Id).Select(x => new
                {
                    x.Id,
                    x.ApplyforId,
                    x.Weekday,
                    x.StartTime,
                    x.EndTime
                }),
                SpecialList = x.SpecialList.Select(x => new
                {
                    x.Id,
                    x.ApplyforId,
                    Date = x.Date.Value.ToString("yyyy-MM-dd"),
                    x.StartTime,
                    x.EndTime
                }),
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 新增耗材申请设置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetApplyForInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Applyfor_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Applyfor_Extend>(requestParm.msg.ToString());

            if ((demodel.BranchId ?? -1) < -1)
            {
                return ApiResult.ParamIsNull("请选择公司别");
            }
            if (demodel.WeekdayList == null || !demodel.WeekdayList.Any())
            {
                return ApiResult.ParamIsNull("请选择工作日");
            }
            foreach (var item in demodel.WeekdayList)
            {
                if (string.IsNullOrEmpty(item.Weekday))
                {
                    return ApiResult.ParamIsNull("工作日设置有误");
                }
                if (string.IsNullOrEmpty(item.StartTime))
                {
                    return ApiResult.ParamIsNull("工作日时间设置有误");
                }
                if (string.IsNullOrEmpty(item.EndTime))
                {
                    return ApiResult.ParamIsNull("工作日时间设置有误");
                }
            }
            if (demodel.SpecialList != null && demodel.SpecialList.Any())
            {
                foreach (var item in demodel.SpecialList)
                {
                    if (!item.Date.HasValue)
                    {
                        return ApiResult.ParamIsNull("请选择特殊日期");
                    }
                    if (string.IsNullOrEmpty(item.StartTime))
                    {
                        return ApiResult.ParamIsNull("特殊日期时间设置有误");
                    }
                    if (string.IsNullOrEmpty(item.EndTime))
                    {
                        return ApiResult.ParamIsNull("特殊日期时间设置有误");
                    }
                }
            }

            //验证公司别是否重复
            if (suppliesModule.ValidateBranch(demodel, userInfo))
            {
                return ApiResult.Failed("该公司已存在记录，无法新增");
            }

            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;

            bool flag = suppliesModule.SetApplyForInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 设置申请状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetApplyForStatus()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ApplyforId = parms.GetInt("ApplyforId", 0);
            if (ApplyforId == 0)
            {
                return ApiResult.Failed("请传递id");
            }

            bool flag = suppliesModule.SetApplyForStatus(ApplyforId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取耗材申请设置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetApplyForInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            var Model = suppliesModule.GetApplyForInfo(userInfo);
            if (Model.Id == -1)
            {
                return ApiResult.Success("", new { Status = 1, Model = "" });
            }
            else
            {
                return ApiResult.Success("", new
                {
                    Status = 2,
                    Model = new
                    {
                        Model.Id,
                        Model.BranchId,
                        BranchName = Model.BranchId == -1 ? "通用" : CacheManager.User.BranchReidsCache.GetBranch(Model.BranchId)?.BName ?? "",
                        WeekdayList = Model.WeekdayList.OrderBy(x => x.Id).Select(x => new
                        {
                            x.Weekday,
                            x.StartTime,
                            x.EndTime
                        }),
                        SpecialList = Model.SpecialList.OrderBy(x => x.Date).Select(x => new
                        {
                            Date = x.Date.Value.ToString("yyyy-MM-dd"),
                            x.StartTime,
                            x.EndTime
                        })
                    }
                });
            }
        }

        /// <summary>
        /// 是否可以申请
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetCanApplyFor()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            var Model = suppliesModule.GetApplyForInfo(userInfo);
            if (Model.Id == -1)
            {
                return ApiResult.Success();
            }
            else
            {
                if (Model.SpecialList != null && Model.SpecialList.Any())
                {
                    var smodel = Model.SpecialList.Where(x => x.Date == Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd"))).FirstOrDefault();
                    if (smodel != null)
                    {
                        smodel.StartTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + smodel.StartTime;
                        smodel.EndTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + smodel.EndTime;
                        if (Convert.ToDateTime(smodel.StartTime) <= DateTime.Now && DateTime.Now <= Convert.ToDateTime(smodel.EndTime))
                        {
                            return ApiResult.Success();
                        }
                        else
                        {
                            return ApiResult.Failed();
                        }
                    }
                }
                int Week = (int)DateTime.Now.DayOfWeek;
                string WeekDay = "";
                switch (Week)
                {
                    case 0:
                        WeekDay = "周日"; break;
                    case 1:
                        WeekDay = "周一"; break;
                    case 2:
                        WeekDay = "周二"; break;
                    case 3:
                        WeekDay = "周三"; break;
                    case 4:
                        WeekDay = "周四"; break;
                    case 5:
                        WeekDay = "周五"; break;
                    case 6:
                        WeekDay = "周六"; break;
                }
                var wmodel = Model.WeekdayList.Where(x => x.Weekday == WeekDay).FirstOrDefault();
                if (wmodel == null)
                {
                    return ApiResult.Failed();
                }
                wmodel.StartTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + wmodel.StartTime;
                wmodel.EndTime = DateTime.Now.ToString("yyyy-MM-dd") + " " + wmodel.EndTime;
                if (Convert.ToDateTime(wmodel.StartTime) <= DateTime.Now && DateTime.Now <= Convert.ToDateTime(wmodel.EndTime))
                {
                    return ApiResult.Success();
                }
                else
                {
                    return ApiResult.Failed();
                }

            }
        }

        #endregion

        #region 采购单

        /// <summary>
        /// 获取采购单分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetProcurementPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Procurement_Extend demodel = JsonConvert.DeserializeObject<RB_Procurement_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetProcurementPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.WareHouseId,
                x.WareHouseName,
                x.TotalCostMoney,
                x.TotalNumber,
                x.Remark,
                x.StockInStatus,
                DetailList = x.DetailList.Select(y => new
                {
                    y.Id,
                    y.MaterialId,
                    y.CostMoney,
                    y.Number,
                    y.StockInNum,
                    MaterialModel = new
                    {
                        y.MaterialModel.Name,
                        y.MaterialModel.SuppliesNum,
                        y.MaterialModel.Units,
                        y.MaterialModel.Money,
                        y.MaterialModel.GoodsSpecificationList,
                        y.MaterialModel.GoodsCategoryList,
                        y.MaterialModel.GoodsId,
                        y.MaterialModel.SpecificationKey,
                        y.MaterialModel.ImageList
                    }
                }),
                x.FinanceId,
                CreateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.CreateBy)?.EmName ?? ""),
                CreateDate = x.CreateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取采购单详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetProcurementInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ProcurementId = parms.GetInt("ProcurementId", 0);
            if (ProcurementId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数");
            }

            var model = suppliesModule.GetProcurementInfo(ProcurementId, userInfo.RB_Group_id);
            return ApiResult.Success("", new
            {
                model.Id,
                model.WareHouseId,
                model.WareHouseName,
                model.TotalCostMoney,
                model.TotalNumber,
                model.Remark,
                model.StockInStatus,
                DetailList = model.DetailList.Select(y => new
                {
                    y.Id,
                    y.MaterialId,
                    y.CostMoney,
                    y.Number,
                    y.StockInNum,
                    MaterialModel = new
                    {
                        y.MaterialModel.Name,
                        y.MaterialModel.SuppliesNum,
                        y.MaterialModel.Units,
                        y.MaterialModel.Money,
                        y.MaterialModel.GoodsSpecificationList,
                        y.MaterialModel.GoodsCategoryList,
                        y.MaterialModel.GoodsId,
                        y.MaterialModel.SpecificationKey,
                        y.MaterialModel.ImageList
                    }
                }),
                model.FinanceId
            });
        }

        /// <summary>
        /// 获取采购单 入库批次
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetProcurementSotckInBatchList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ProcurementId = parms.GetInt("ProcurementId", 0);
            if (ProcurementId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数");
            }

            var list = suppliesModule.GetProcurementSotckInBatchList(ProcurementId, userInfo);
            return ApiResult.Success("", list);
        }

        /// <summary>
        /// 获取采购入库 的标签
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetProcurementStockInLableList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ProcurementId = parms.GetInt("ProcurementId", 0);
            int StockInId = parms.GetInt("StockInId", 0);
            if (ProcurementId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数");
            }

            var list = suppliesModule.GetProcurementStockInLableList(ProcurementId, StockInId, userInfo);
            return ApiResult.Success("", list);

        }

        #endregion

        #region 根据二维码标识  获取商品信息
        /// <summary>
        /// 获取物料信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetProcurementGoodsInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            string Code = parms.GetStringValue("Code");
            int WareHouseId = parms.GetInt("WareHouseId", 0);
            if (string.IsNullOrEmpty(Code))
            {
                return ApiResult.ParamIsNull("请传递参数");
            }
            if (WareHouseId <= 0)
            {
                return ApiResult.ParamIsNull("请传递仓库id");
            }

            var obj = suppliesModule.GetProcurementGoodsInfo(Code, WareHouseId, userInfo);
            return ApiResult.Success("", obj);
        }

        #endregion

        #region 耗材盘点

        /// <summary>
        /// 获取盘点单分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSuppliesCheckPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_Check_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Check_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetSuppliesCheckPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Periods,
                x.WarehouseName,
                x.WarehouseId,
                StartDate = x.StartDate.HasValue ? x.StartDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                EndDate = x.EndDate.HasValue ? x.EndDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                x.CheckState,
                CheckStateName = x.CheckState.GetEnumName(),
                x.Remark,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取耗材盘点状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSuppliesCheckStatusEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.SuppliesCheckStatusEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        /// <summary>
        /// 获取盘点单详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSuppliesCheckModel() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            var cmodel = suppliesModule.GetSuppliesCheck(CheckId);

            return ApiResult.Success("",new {
                cmodel.Id,
                cmodel.Periods,
                cmodel.WarehouseName,
                cmodel.WarehouseId,
                StartDate = cmodel.StartDate.HasValue ? cmodel.StartDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                EndDate = cmodel.EndDate.HasValue ? cmodel.EndDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                cmodel.CheckState,
            });
        }

        /// <summary>
        /// 新增修改盘点单
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetSuppliesCheck()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_Check_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_Check_Extend>(requestParm.msg.ToString());

            if ((demodel.WarehouseId ?? 0) <= 0)
            {
                return ApiResult.Failed("请选择仓库");
            }
            if (!demodel.StartDate.HasValue)
            {
                return ApiResult.ParamIsNull("请选择开始时间");
            }
            if (!demodel.EndDate.HasValue)
            {
                return ApiResult.ParamIsNull("请选择结束时间");
            }

            demodel.Status = 0;
            demodel.CheckState = REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = suppliesModule.SetSuppliesCheck(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 取消盘点
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelSuppliesCheck()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            var cmodel = suppliesModule.GetSuppliesCheck(CheckId);
            if (cmodel == null)
            {
                return ApiResult.Failed("盘点单不存在");
            }
            if (cmodel.CheckState != REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory)
            {
                return ApiResult.Failed("盘点单无法取消");
            }

            bool flag = suppliesModule.CancelSuppliesCheck(CheckId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 盘点单复盘
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult RepeatSuppliesCheck()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            string StartTime = parms.GetStringValue("StartTime");
            string EndTime = parms.GetStringValue("EndTime");
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            var cmodel = suppliesModule.GetSuppliesCheck(CheckId);
            if (cmodel == null)
            {
                return ApiResult.Failed("盘点单不存在");
            }
            if (cmodel.CheckState != REBORN.Common.Enum.SuppliesCheckStatusEnum.Completed)
            {
                return ApiResult.Failed("盘点单无法复盘");
            }

            bool flag = suppliesModule.RepeatSuppliesCheck(CheckId, StartTime, EndTime, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 盘点完成
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetSuppliesCheckComplete()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            var cmodel = suppliesModule.GetSuppliesCheck(CheckId);
            if (cmodel == null)
            {
                return ApiResult.Failed("盘点单不存在");
            }
            if (cmodel.CheckState != REBORN.Common.Enum.SuppliesCheckStatusEnum.InTheInventory)
            {
                return ApiResult.Failed("盘点单状态不正确");
            }

            bool flag = suppliesModule.SetSuppliesCheckComplete(CheckId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取盘点明细列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSuppliesCheckDetailPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_CheckDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_CheckDetail_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = suppliesModule.GetSuppliesCheckDetailPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.CheckStatus,
                CheckStatusName = x.CheckStatus.GetEnumName(),
                x.Remark,
                x.SuppliesNo,
                x.SuppliesId,
                x.SuppliesName,
                x.SpecificationList,
                x.InventoryNum,
                x.RealityNum,
                x.ImageList,
                x.CheckEmpId,
                CheckEmpName = x.CheckEmpId.HasValue && x.CheckEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(x.CheckEmpId)?.EmName : "",
                CheckDate = x.CheckDate.HasValue ? x.CheckDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                DealStatus = x.DealStatus ?? 0,
                //UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                //UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 设置盘点明细 耗材实盘数量
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetSuppliesCheckDetailReality()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckDetailId = parms.GetInt("CheckDetailId", 0);
            int Type = parms.GetInt("Type", 0);// 1更新数量   2上传图片  3添加备注
            int Reality = parms.GetInt("Reality", 0);
            string ImageList = parms.GetStringValue("ImageList");
            string Remark = parms.GetStringValue("Remark");
            if (CheckDetailId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            if (Type == 1 && Reality < 0)
            {
                return ApiResult.ParamIsNull("请传递实际数量");
            }
            List<string> ImagesList = new List<string>();
            if (Type == 2)
            {
                if (string.IsNullOrEmpty(ImageList))
                {
                    return ApiResult.ParamIsNull("请上传图片");
                }
                ImagesList = JsonConvert.DeserializeObject<List<string>>(ImageList);
            }

            //验证盘点单是否已整单完成
            if (suppliesModule.ValidataSuppliesCheckDetail(CheckDetailId))
            {
                return ApiResult.Failed("该盘点单已整单完成/取消，无法操作");
            }

            var sdmodel = suppliesModule.GetSuppliesCheckDetailInfo(CheckDetailId);
            if (sdmodel == null) { return ApiResult.Failed("盘点单不存在"); }
            if (Type == 1 && (sdmodel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryProfit || sdmodel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryLosses) && sdmodel.DealStatus == 1)
            {
                return ApiResult.Failed("该耗材已报溢/报损处理，无法修改数量");
            }
            bool flag = suppliesModule.SetSuppliesCheckDetailReality(CheckDetailId, Type, Reality, ImagesList, Remark, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 批量更新盘点明细实际数量
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetBatchSuppliesCheckDetailReality()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            List<RB_Supplies_CheckDetail_Extend> list = JsonConvert.DeserializeObject<List<RB_Supplies_CheckDetail_Extend>>(requestParm.msg.ToString());

            if (!list.Any())
            {
                return ApiResult.ParamIsNull("请传递列表");
            }
            foreach (var item in list)
            {
                if (item.Id <= 0)
                {
                    return ApiResult.ParamIsNull("请传递盘点单明细id");
                }
                if ((item.RealityNum ?? 0) < 0)
                {
                    return ApiResult.ParamIsNull("请传递实际库存数量");
                }
            }
            bool flag = suppliesModule.SetBatchSuppliesCheckDetailReality(list, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 盘点明细导出Excel
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public FileContentResult GetSuppliesCheckDetailToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_CheckDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_CheckDetail_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"ID"){CellWidth=10,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"UserId"){CellWidth=10,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"耗材规格"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"当前库存"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点数量"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = suppliesModule.GetSuppliesCheckDetailPageList(1, 99999, demodel, out long count);

                #region 组装数据
                foreach (var item in list)
                {
                    string SpName = string.Join(",", item.SpecificationList);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.Id.ToString()){ },
                            new ExcelColumn(value:userInfo.EmployeeId.ToString()){ },
                            new ExcelColumn(value:item.CheckStatus.GetEnumName()){ },
                            new ExcelColumn(value:item.SuppliesNo??""){ },
                            new ExcelColumn(value:item.SuppliesName??""){ },
                            new ExcelColumn(value:SpName??""){ },
                            new ExcelColumn(value:(item.InventoryNum??0).ToString()){ },
                            new ExcelColumn(value:(item.RealityNum??0).ToString()){ },
                            new ExcelColumn(value:item.Remark??""){ }
                        },
                        ColumnHight = 30
                    };
                    slist.Add(datarow);
                }
                #endregion                
                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "耗材盘点.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetSuppliesCheckDetailToExcel: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "耗材盘点.xls");
            }
        }


        /// <summary>
        /// 获取盘点报损/报溢列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSuppliesCheckDetailBreakagePageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Supplies_CheckDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_CheckDetail_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            if (demodel.CheckStatus != REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryLosses && demodel.CheckStatus != REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryProfit)
            {
                demodel.IsSelectCheckLoss = 1;
            }
            if (demodel.CheckId <= 0)
            {
                return ApiResult.Failed("请传递盘点单id");
            }
            var list = suppliesModule.GetSuppliesCheckDetailBreakagePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.CheckStatus,
                CheckStatusName = x.CheckStatus.GetEnumName(),
                x.Remark,
                x.SuppliesNo,
                x.SuppliesId,
                x.SuppliesName,
                x.SpecificationList,
                x.InventoryNum,
                x.RealityNum,
                x.ImageList,
                x.Unit_Money,
                x.CheckEmpId,
                CheckEmpName = x.CheckEmpId.HasValue && x.CheckEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(x.CheckEmpId)?.EmName : "",
                CheckDate = x.CheckDate.HasValue ? x.CheckDate.Value.ToString("yyyy-MM-dd HH:mm:ss") : "",
                DealStatus = x.DealStatus ?? 0
            });
            return ApiResult.Success("", pagelist);
        }

        #endregion


        #region 盘点报损

        /// <summary>
        /// 保存盘点报损信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetSuppliesReportLoss()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            DateTime NowStr = System.DateTime.Now;
            RB_Supplies_ReportLossRecord_Extend applyModel = JsonConvert.DeserializeObject<RB_Supplies_ReportLossRecord_Extend>(requestParm.msg.ToString());
            if (applyModel.WarehouseId <= 0)
            {
                return ApiResult.ParamIsNull("请选择仓库");
            }
            if ((applyModel.CheckId ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择盘点期数");
            }

            if (applyModel.DetailList == null && !applyModel.DetailList.Any())
            {
                return ApiResult.Failed("请选择您要报损的商品");
            }

            //判断当前仓库是否在盘点中2020-07-28 Add By:W
            var suppliesCheckModel = suppliesModule.GetSuppliesCheckList(new RB_Supplies_Check_Extend { Id = applyModel.CheckId ?? 0, RB_Group_Id = userInfo.RB_Group_id, WarehouseId = applyModel.WarehouseId }).FirstOrDefault();
            if (suppliesCheckModel == null)
            {
                return ApiResult.ParamIsNull("盘点期数不存在");
            }
            applyModel.WarehouseId = suppliesCheckModel.WarehouseId;
            if (suppliesCheckModel.CheckState != REBORN.Common.Enum.SuppliesCheckStatusEnum.Completed)
            {
                return ApiResult.ParamIsNull("盘点未完成不能报损/报溢");
            }
            //查询盘点信息
            var checkDetailList = suppliesModule.GetSuppliesCheckDetailList(new RB_Supplies_CheckDetail_Extend { CheckId = applyModel.CheckId ?? 0, RB_Group_Id = userInfo.RB_Group_id });
            foreach (var item in applyModel.DetailList)//判断盘点商品是否已经报损或者是否是损益
            {
                var detailModel = checkDetailList.Where(x => x.SuppliesId == item.SuppliesId).FirstOrDefault();
                if (detailModel == null)
                {
                    return ApiResult.Failed("盘点部分损益商品不属于当前盘点期数");
                }
                if (detailModel.DealStatus == 1)
                {
                    return ApiResult.Failed("盘点部分损益商品已处理");
                }

                int ReportLossNum = ((detailModel.RealityNum ?? 0) - (detailModel.InventoryNum ?? 0));
                if (applyModel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryLosses)
                {
                    if (ReportLossNum > 0)
                    {
                        return ApiResult.Failed("部分盘点商品不是盘亏商品");
                    }
                    ReportLossNum = ReportLossNum * -1;
                }
                else if (applyModel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryProfit)
                {
                    if (ReportLossNum < 0)
                    {
                        return ApiResult.Failed("部分盘点商品不是盘赢商品");
                    }
                }
                if (item.ReportLossNum != ReportLossNum)
                {
                    return ApiResult.Failed("部分盘点商品亏盈数错误");
                }


            }


            #region 出库记录信息
            RB_Supplies_StockOut_Extend demodel = new RB_Supplies_StockOut_Extend();//出库申请表，等会儿组装
            RB_Supplies_StockIn_Extend demodelStockIn = new RB_Supplies_StockIn_Extend();
            if (applyModel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryLosses)//盘亏出库
            {
                demodel.WarehouseId = applyModel.WarehouseId;
                demodel.EmployeeId = userInfo.EmployeeId;
                demodel.CreateBy = userInfo.EmployeeId;
                demodel.StockOutDate = NowStr;
                demodel.StockOutState = REBORN.Common.Enum.StockOutStatusEnum.StockOut;
                demodel.Remark = "电商商品盘点-盘亏出库";
                demodel.Status = 0;
                demodel.RB_Branch_Id = userInfo.RB_Branch_id;
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.CreateDate = NowStr;
                demodel.UpdateBy = userInfo.EmployeeId;
                demodel.UpdateDate = NowStr;
                demodel.DepartmentId = userInfo.RB_Department_Id;
                demodel.BranchId = userInfo.RB_Branch_id;


                demodel.DetailList = new List<RB_Supplies_StockOutDetail_Extend>();

                foreach (var item in applyModel.DetailList)
                {
                    RB_Supplies_StockOutDetail_Extend stockOutDetailModel = new RB_Supplies_StockOutDetail_Extend
                    {

                        Id = 0,
                        StockOutId = 0,
                        SuppliesId = item.SuppliesId,
                        Number = item.ReportLossNum,
                        Remark = "电商商品盘点-盘亏出库",
                        Status = 0,
                        RB_Branch_Id = userInfo.RB_Branch_id,
                        RB_Group_Id = userInfo.RB_Group_id,
                        CreateBy = userInfo.EmployeeId,
                        CreateDate = NowStr,
                        UpdateBy = userInfo.EmployeeId,
                        UpdateDate = NowStr,
                        Money = item.ReportLossPrice,
                    };
                    demodel.DetailList.Add(stockOutDetailModel);
                }
                if (demodel.DetailList == null || !demodel.DetailList.Any())
                {
                    return ApiResult.ParamIsNull("请选择物料");
                }
                if (!demodel.StockOutDate.HasValue)
                {
                    return ApiResult.ParamIsNull("请输入出库日期");
                }
                if (demodel.EmployeeId <= 0)
                {
                    return ApiResult.ParamIsNull("请选择领用人");
                }
                foreach (var item in demodel.DetailList)
                {
                    if (item.SuppliesId <= 0)
                    {
                        return ApiResult.ParamIsNull("请传递正确的物料id");
                    }
                    if (item.Number <= 0)
                    {
                        return ApiResult.ParamIsNull("请输入出库数量");
                    }
                }
                if (demodel.DetailList.Any())
                {
                    var slist = demodel.DetailList.Select(x => x.SuppliesId).Distinct().ToList();
                    foreach (var item in slist)
                    {
                        if (demodel.DetailList.Where(x => x.SuppliesId == item).Count() > 1)
                        {
                            return ApiResult.Failed("每种耗材只能选择一个");
                        }
                    }
                }
                else
                {
                    return ApiResult.Failed("请选择耗材");
                }
                //验证库存是否足够
                if (!suppliesModule.ValidateStockOutInsert(demodel))
                {
                    return ApiResult.Failed("库存不足无法出库，请核实后再试");
                }

            }
            #endregion

            #region 入库记录

            else if (applyModel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryProfit)//盘赢入库
            {
                demodelStockIn.WarehouseId = applyModel.WarehouseId;
                demodelStockIn.CreateBy = userInfo.EmployeeId;
                demodelStockIn.StockInDate = NowStr;
                demodelStockIn.StockInState = REBORN.Common.Enum.StockInStatusEnum.StockIn;
                demodelStockIn.Remark = "电商商品盘点-盘赢入库";
                demodelStockIn.Status = 0;
                demodelStockIn.RB_Branch_Id = userInfo.RB_Branch_id;
                demodelStockIn.RB_Group_Id = userInfo.RB_Group_id;
                demodelStockIn.CreateDate = NowStr;
                demodelStockIn.UpdateBy = userInfo.EmployeeId;
                demodelStockIn.UpdateDate = NowStr;
                demodelStockIn.SupplierName = "电商商品盘点";
                demodelStockIn.DetailList = new List<RB_Supplies_StockInDetail_Extend>();
                foreach (var item in applyModel.DetailList)
                {
                    RB_Supplies_StockInDetail_Extend stockOutDetailModel = new RB_Supplies_StockInDetail_Extend
                    {

                        Id = 0,
                        StockInId = 0,
                        SuppliesId = item.SuppliesId,
                        Number = item.ReportLossNum,
                        Remark = "电商商品盘点-盘赢入库",
                        Status = 0,
                        RB_Branch_Id = userInfo.RB_Branch_id,
                        RB_Group_Id = userInfo.RB_Group_id,
                        CreateBy = userInfo.EmployeeId,
                        CreateDate = NowStr,
                        UpdateBy = userInfo.EmployeeId,
                        UpdateDate = NowStr,
                        Money = item.ReportLossPrice,
                        UnitPrice = Math.Round((item.ReportLossPrice ?? 0) / (item.ReportLossNum ?? 1), 2, MidpointRounding.AwayFromZero)
                    };
                    demodelStockIn.DetailList.Add(stockOutDetailModel);
                }
                demodelStockIn.ProcurementId = 0;
                demodelStockIn.Money = demodelStockIn.DetailList.Sum(x => x.Money);

                if (demodelStockIn.WarehouseId <= 0)
                {
                    return ApiResult.ParamIsNull("请选择仓库");
                }
                if (demodelStockIn.DetailList == null || !demodelStockIn.DetailList.Any())
                {
                    return ApiResult.ParamIsNull("请选择物料");
                }
                if (!demodelStockIn.StockInDate.HasValue)
                {
                    return ApiResult.ParamIsNull("请输入入库日期");
                }
                foreach (var item in demodelStockIn.DetailList)
                {
                    if (item.SuppliesId <= 0)
                    {
                        return ApiResult.ParamIsNull("请传递正确的物料id");
                    }
                    if (!item.Money.HasValue)
                    {
                        return ApiResult.ParamIsNull("请输入入库金额");
                    }
                    if (item.Number <= 0)
                    {
                        return ApiResult.ParamIsNull("请输入入库数量");
                    }
                }

            }

            #endregion

            #region 报损记录
            applyModel.CreateDate = System.DateTime.Now;
            applyModel.CreateBy = userInfo.EmployeeId;
            applyModel.Remark = applyModel.CheckStatus == REBORN.Common.Enum.PropertyCheckStatusEnum.InventoryLosses ? "电商商品盘点-盘亏" : "电商商品盘点-盘盈";
            applyModel.Status = 0;
            applyModel.RB_Branch_Id = userInfo.RB_Branch_id;
            applyModel.RB_Group_Id = userInfo.RB_Group_id;
            #endregion

            bool flag = suppliesModule.SetReportLossRecord(applyModel, demodel, demodelStockIn, userInfo.RB_Department_Id);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }
        #endregion
    }
}
