﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Model.Extend;
using REBORN.Common;
using Property.Module.UserModule;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using System.Collections.Generic;

namespace Property.WebApi.Controllers.User
{
    public class UserController : PApiController
    {
        //员工
        private readonly EmployeeModule employeeModule = new EmployeeModule();
        //公司
        private readonly BranchModule branchModule = new BranchModule();
        //部门
        private readonly DepartmentModule departmentModule = new DepartmentModule();

        /// <summary>
        /// 获取公司列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetBranchList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            var list = branchModule.GetBranchList(new RB_Branch_Extend()
            {
                Status = 0,
                RB_Group_Id = userInfo.RB_Group_id
            });
            return ApiResult.Success("", list.Select(x => new
            {
                x.Id,
                x.BName
            }));
        }

        /// <summary>
        /// 获取员工列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetEmployeeList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Employee_Extend demodel = JsonConvert.DeserializeObject<RB_Employee_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_id = userInfo.RB_Group_id;
            var list = employeeModule.GetList(demodel);
            var robj = list.Select(x => new
            {
                x.EmployeeId,
                x.EmName
            });
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取部门树形结构
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetDepartmentTreeForReceiveQuery()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int BranchId = parms.GetInt("RB_Branch_Id", -1);
            List<Module.UserModule.DepartmentModule.DepartmentChooseSell> list = departmentModule.GetDepartmentTreeForReceiveQuery(1, userInfo.RB_Group_id, BranchId);
            //if (!string.IsNullOrEmpty(userInfo.ActionMenuCode) && ("," + userInfo.ActionMenuCode + ",").Contains(",S_CheckAllOrder,"))
            //{
            //    list = departmentModule.GetDepartmentTreeForReceiveQuery(1, userInfo.RB_Group_id, BranchId);
            //}
            //else if (!string.IsNullOrEmpty(userInfo.ActionMenuCode) && ("," + userInfo.ActionMenuCode + ",").Contains(",S_CheckBranchOrder,"))
            //{
            //    list = departmentModule.GetDepartmentTreeForReceiveQuery(2, userInfo.RB_Branch_id);
            //}
            //else if (!string.IsNullOrEmpty(userInfo.ActionMenuCode) && ("," + userInfo.ActionMenuCode + ",").Contains(",S_CheckDepartmentOrder,"))
            //{
            //    if (userInfo.IsParentCompany == 1)
            //    {
            //        list = departmentModule.GetDepartmentTreeForReceiveQuery(2, userInfo.RB_Branch_id);
            //    }
            //    else
            //    {
            //        list = departmentModule.GetDepartmentTreeForReceiveQuery(3, userInfo.RB_Department_Id);
            //    }

            //}
            //else
            //{

            //}
            return ApiResult.Success("", list);
        }


        /// <summary>
        /// 获取用户登录
        /// </summary>
        /// <param name="requestMsg"></param>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetUserLoginMenu()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            List<RB_Sys_Menu_Extend> resultUserMenuList = employeeModule.GetUserMenuAllList(userInfo.RB_Post_Id, userInfo.EmployeeId, out List<RB_Sys_Menu_Extend> userMenuList);

            userInfo.UserMenu = resultUserMenuList;
            userInfo.token = requestParm.token;
            return ApiResult.Success(data: userInfo);
        }

    }
}
