using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.Encodings.Web;
using System.Text.Unicode;

namespace Property.WebApi
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.Configure<Microsoft.AspNetCore.Server.Kestrel.Core.KestrelServerOptions>(x => x.AllowSynchronousIO = true)
                .Configure<IISServerOptions>(x => x.AllowSynchronousIO = true);

            services.AddControllers();

            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>(); //עhttp

            services.AddMvc().AddJsonOptions(options =>
            {
                options.JsonSerializerOptions.Encoder = JavaScriptEncoder.Create(UnicodeRanges.All);
                options.JsonSerializerOptions.PropertyNamingPolicy = null;
            });

            List<string> corsArray = GetAllowCorsList();
            services.AddCors(options => options.AddPolicy("AllowCors", policy => policy.AllowAnyHeader().AllowAnyMethod().AllowCredentials().WithOrigins(corsArray.ToArray())));
        }

        /// <summary>
        /// ȡõ
        /// </summary>
        /// <returns></returns>
        private static List<string> GetAllowCorsList()
        {
            string accessSiteStr = new ConfigurationBuilder().Add(new Microsoft.Extensions.Configuration.Json.JsonConfigurationSource { Path = "appsettings.json" }).Build().GetSection("accessSiteStr").Value;
            List<string> corsArray = new List<string>()
            {
                "http://www.test.com:8080",
                "http://www.test.com:8081",
                "http://www.test.com:8082",
                "http://localhost:8080",
                "http://localhost:8081",
                "http://testzcyx.oytour.com:8080",
                "http://testzcyx.oytour.com:8081",
                "http://testzcyx.oytour.com",
                "http://testzcerp.oytour.com",
                "http://zcyx.oytour.com",
                "http://yx.oytour.com",
                "http://mall.oytour.com",
                "http://mallapi.oytour.com",
                "http://www.oytour.com"
            };
            if (!string.IsNullOrEmpty(accessSiteStr))
            {
                var accesslist = accessSiteStr.Split(',');
                foreach (var item in accesslist)
                {
                    if (!string.IsNullOrEmpty(item))
                    {
                        corsArray.Add(item);
                    }
                }
                corsArray = corsArray.Distinct().ToList();
            }

            return corsArray;
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();

            app.UseRouting();
            // пcorsConfigureServicesõĿ
            app.UseCors("AllowCors");

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });

            System.WebHttpContext.HttpContext.Configure(app.ApplicationServices.GetRequiredService<Microsoft.AspNetCore.Http.IHttpContextAccessor>());

            app.UseStaticFiles(new StaticFileOptions
            {
                FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(
                Path.Combine(Directory.GetCurrentDirectory(), "ExportFile")),
                RequestPath = "/ExportFile"
            });
            app.UseStaticFiles(new StaticFileOptions
            {
                FileProvider = new Microsoft.Extensions.FileProviders.PhysicalFileProvider(
                Path.Combine(Directory.GetCurrentDirectory(), "upfile")),
                RequestPath = "/upfile"
            });
        }
    }
}
