﻿using System.Linq;
using Microsoft.AspNetCore.Mvc;
using REBORN.Common.API;
using Newtonsoft.Json;
using Property.Model.Extend;
using REBORN.Common;
using Newtonsoft.Json.Linq;
using REBORN.Common.Plugin;
using Property.Module.FixedAssets;
using System;
using System.Collections.Generic;
using REBORN.Common.Enum;
using System.Net.Http;
using System.Net;
using System.IO;
using System.Net.Http.Headers;
using Microsoft.AspNetCore.Http;

namespace Property.WebApi.Controllers.User
{
    public class PropertyController : PApiController
    {
        PropertyModule propertyModule = new PropertyModule();

        #region 资产分类
        /// <summary>
        /// 获取资产分类 分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CategoryGetPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Category_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Category_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.CateporyGetPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.ParentId,
                x.ParentName,
                x.Tier,
                x.Sort,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取分类列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CateporyGetList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Category_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Category_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.CateporyGetList(demodel);
            return ApiResult.Success("", list.Select(x => new
            {
                x.Id,
                x.Name,
                ParentId = x.ParentId ?? 0,
                x.Tier
            }));
        }

        /// <summary>
        /// 获取分类树形结构列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CateporyGetTreeList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Category_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Category_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.CateporyGetList(demodel);
            return ApiResult.Success("", GetChildForTree(list, 0));
        }

        /// <summary>
        /// 分类树形结构
        /// </summary>
        /// <param name="list"></param>
        /// <param name="parentId"></param>
        /// <returns></returns>
        public List<RB_Property_Category_Extend> GetChildForTree(List<RB_Property_Category_Extend> list, int parentId)
        {
            //获取1级
            var FristList = list.Where(x => x.ParentId == parentId).ToList();
            if (FristList.Count == 0 && parentId == 0 && list.Count > 0)
            {
                //为总公司部门 下的公司
                int minTier = list.Select(x => x.Tier.Value).Min();
                FristList = list.Where(x => x.Tier == minTier).ToList();
            }
            List<RB_Property_Category_Extend> Rlist = new List<RB_Property_Category_Extend>();
            foreach (var item in FristList)
            {
                item.ChildList = new List<RB_Property_Category_Extend>();
                item.ChildList.AddRange(GetChildForTree(list, item.Id));
                Rlist.Add(item);
            }
            return Rlist;
        }

        /// <summary>
        /// 获取分类详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetCategoryInfo() {
            RequestParm requestParm = GetRequestParm();
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CateporyId = parms.GetInt("CategoryId", 0);
            if (CateporyId <= 0) {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            var model = propertyModule.GetCateporyInfo(CateporyId);
            return ApiResult.Success("", new {
                model.Id,
                model.Name,
                model.ParentId,
                model.Tier,
                model.Sort,
                model.Type
            });
        }

        /// <summary>
        /// 设置分类详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetCategoryInfo() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Category_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Category_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name)) {
                return ApiResult.ParamIsNull("请传递分类名称");
            }
            if (demodel.ParentId == null) {
                return ApiResult.ParamIsNull("请传递父级分类");
            }
            if (demodel.Tier <= 0) {
                return ApiResult.ParamIsNull("请传递层级");
            }
            if (demodel.Type != 1 && demodel.Type != 2) {
                return ApiResult.ParamIsNull("请传递类型");
            }

            //parentId如果更换  查询是否跟换到其下级目录    未做

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = propertyModule.SetCateporyInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除分类
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelCategoryInfo() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CateporyId = parms.GetInt("CategoryId", 0);
            if (CateporyId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = propertyModule.DelCateporyInfo(CateporyId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }
        #endregion

        #region 资产基本
        /// <summary>
        /// 获取资产分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult PropertyGetPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            if (demodel.IsSelectSelfApplication == 1) {
                demodel.CreateBy = userInfo.EmployeeId;
            }
            if (demodel.IsSelectMyProperty == 1) {
                demodel.CreateBy = userInfo.EmployeeId;
                demodel.EmployeeId = 0;
            }
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.PropertyGetPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                BuyDate = x.BuyDate.HasValue ? x.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                x.BuyType,
                BuyTypeName = x.BuyType.GetEnumName(),
                x.Units,
                x.UseStatus,
                UseStatusName = x.UseStatus.GetEnumName(),
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.PropertyModel,
                x.ImageList,
                x.SerialNumber,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                GetTime = x.GetTime.HasValue ? x.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.SupplierId,
                x.SupplierName,
                x.BranchId,
                x.UseMonths,
                x.QRCode,
                UseReceiveId = x.UseReceiveId ?? 0,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 资产清单下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult PropertyGetListStatisticsToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"购置日期"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单位"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"设备序列号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"公司别"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"供应商"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用日期"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }
                         },
            };
            slist.Add(header);
            try
            {
                if (demodel.IsSelectSelfApplication == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                }
                if (demodel.IsSelectMyProperty == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                    demodel.EmployeeId = 0;
                }
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.PropertyGetPageList(1, 99999, demodel, out long count);

                #region 组装数据
                foreach (var item in list)
                {
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.PropertyNum){ },
                            new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ },
                            new ExcelColumn(value:item.Name){ },
                            new ExcelColumn(value:item.CategoryName){ },
                            new ExcelColumn(value:item.BrandName){ },
                            new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                            new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                            new ExcelColumn(value:item.UseStatus.GetEnumName()){ },
                            new ExcelColumn(value:item.Units){ },
                            new ExcelColumn(value:item.PropertyModel){ },
                            new ExcelColumn(value:item.SerialNumber){ },
                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                            new ExcelColumn(value:item.SupplierName){ },
                            new ExcelColumn(value:item.EmName){ },
                            new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ }
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "资产清单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产清单.xls");
            }

        }

        /// <summary>
        /// 资产领用下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyUseListStatisticsToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                    new ExcelColumn(value:"公司"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                    new ExcelColumn(value:"部门"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                    new ExcelColumn(value:"人员"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用日期"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"购置日期"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单位"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"设备序列号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"公司别"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"供应商"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }                            
                         },
            };
            slist.Add(header);
            try
            {
                if (demodel.IsSelectSelfApplication == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                }
                if (demodel.IsSelectMyProperty == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                    demodel.EmployeeId = 0;
                }
                demodel.IsSelectUseProperty = 1;
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.PropertyGetPageList(1, 99999, demodel, out long count);

                #region 组装数据
                //根据公司  部门  人员 分类
                List<int> Blist = list.Select(x => x.EmBranchId ?? 0).Distinct().ToList();
                int bNum = 1, dNum = 1, eNum = 1;
                foreach (var bitem in Blist) {
                    bNum = 1;
                    var bplist = list.Where(x => x.EmBranchId == bitem).ToList();
                    var bcount = bplist.Count();
                    List<int> Dlist = bplist.Select(x => x.EmDepartmentId ?? 0).Distinct().ToList();
                    foreach (var ditem in Dlist) {
                        dNum = 1;
                        var dplist = bplist.Where(x => x.EmDepartmentId == ditem).ToList();
                        var Elist = dplist.Select(x => x.EmployeeId ?? 0).Distinct().ToList();
                        var dcount = dplist.Count();
                        foreach (var eitem in Elist)
                        {
                            eNum = 1;
                            var eplist = dplist.Where(x => x.EmployeeId == eitem).ToList();
                            var ecount = eplist.Count();
                            foreach (var item in eplist) {
                                if (bNum == 1 && dNum == 1 && eNum == 1)
                                {
                                    ExcelDataSource datarow = new ExcelDataSource()
                                    {
                                        ExcelRows = new List<ExcelColumn>(30) {
                                            new ExcelColumn(value:item.EmBranchName){ Rowspan=bcount },
                                            new ExcelColumn(value:item.EmDepartmentName){ Rowspan=dcount },
                                            new ExcelColumn(value:item.EmName){ Rowspan=ecount },
                                            new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.PropertyNum){ },
                                            new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Name){ },
                                            new ExcelColumn(value:item.CategoryName){ },
                                            new ExcelColumn(value:item.BrandName){ },
                                            new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                            new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.UseStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Units){ },
                                            new ExcelColumn(value:item.PropertyModel){ },
                                            new ExcelColumn(value:item.SerialNumber){ },
                                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                                            new ExcelColumn(value:item.SupplierName){ }
                                        }
                                    };
                                    slist.Add(datarow);
                                }
                                else if (bNum != 1 && dNum == 1 && eNum == 1)
                                {
                                    ExcelDataSource datarow = new ExcelDataSource()
                                    {
                                        ExcelRows = new List<ExcelColumn>(30) {
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:item.EmDepartmentName){ Rowspan=dcount },
                                            new ExcelColumn(value:item.EmName){ Rowspan=ecount },
                                            new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.PropertyNum){ },
                                            new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Name){ },
                                            new ExcelColumn(value:item.CategoryName){ },
                                            new ExcelColumn(value:item.BrandName){ },
                                            new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                            new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.UseStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Units){ },
                                            new ExcelColumn(value:item.PropertyModel){ },
                                            new ExcelColumn(value:item.SerialNumber){ },
                                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                                            new ExcelColumn(value:item.SupplierName){ }
                                        }
                                    };
                                    slist.Add(datarow);
                                }
                                else if (bNum != 1 && dNum != 1 && eNum == 1) {
                                    ExcelDataSource datarow = new ExcelDataSource()
                                    {
                                        ExcelRows = new List<ExcelColumn>(30) {
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:item.EmName){ Rowspan=ecount },
                                            new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.PropertyNum){ },
                                            new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Name){ },
                                            new ExcelColumn(value:item.CategoryName){ },
                                            new ExcelColumn(value:item.BrandName){ },
                                            new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                            new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.UseStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Units){ },
                                            new ExcelColumn(value:item.PropertyModel){ },
                                            new ExcelColumn(value:item.SerialNumber){ },
                                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                                            new ExcelColumn(value:item.SupplierName){ }
                                        }
                                    };
                                    slist.Add(datarow);
                                }
                                else
                                {
                                    ExcelDataSource datarow = new ExcelDataSource()
                                    {
                                        ExcelRows = new List<ExcelColumn>(30) {
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:""){ },
                                            new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.PropertyNum){ },
                                            new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Name){ },
                                            new ExcelColumn(value:item.CategoryName){ },
                                            new ExcelColumn(value:item.BrandName){ },
                                            new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                            new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                                            new ExcelColumn(value:item.UseStatus.GetEnumName()){ },
                                            new ExcelColumn(value:item.Units){ },
                                            new ExcelColumn(value:item.PropertyModel){ },
                                            new ExcelColumn(value:item.SerialNumber){ },
                                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                                            new ExcelColumn(value:item.SupplierName){ }
                                        }
                                    };
                                    slist.Add(datarow);
                                }

                                bNum++;
                                dNum++;
                                eNum++;
                            }
                        }
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "资产清单.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产清单.xls");
            }

        }

        /// <summary>
        /// 资产批量导入
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyInfoBatchImport()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            List<RB_Property_Info_Extend> deList = JsonConvert.DeserializeObject<List<RB_Property_Info_Extend>>(requestParm.msg.ToString());
            foreach (var demodel in deList)
            {
                if (string.IsNullOrEmpty(demodel.Name))
                {
                    return ApiResult.ParamIsNull("资产名称不能为空");
                }
                if (string.IsNullOrEmpty(demodel.CategoryName))
                {
                    return ApiResult.ParamIsNull("分类名称不能为空");
                }
                if (string.IsNullOrEmpty(demodel.BrandName))
                {
                    return ApiResult.ParamIsNull("品牌不能为空");
                }
                if (string.IsNullOrEmpty(demodel.PropertyModel))
                {
                    return ApiResult.ParamIsNull("型号不能为空");
                }
                if (string.IsNullOrEmpty(demodel.Units))
                {
                    return ApiResult.ParamIsNull("单位不能为空");
                }
                if (string.IsNullOrEmpty(demodel.SerialNumber))
                {
                    return ApiResult.ParamIsNull("序列号不能为空");
                }
                if (string.IsNullOrEmpty(demodel.EndTime))
                {
                    return ApiResult.ParamIsNull("购置日期不能为空");
                }
                if ((demodel.Money??0)<=0)
                {
                    return ApiResult.ParamIsNull("金额不能小于0");
                }
                if ((demodel.UseMonths??0)<=0)
                {
                    return ApiResult.ParamIsNull("使用月份期限不能为空");
                }
                if (string.IsNullOrEmpty(demodel.BranchName))
                {
                    return ApiResult.ParamIsNull("所属公司不能为空");
                }
                if (string.IsNullOrEmpty(demodel.SupplierName))
                {
                    return ApiResult.ParamIsNull("供应商名称不能为空");
                }

                demodel.Status = 0;
                demodel.RB_Branch_Id = userInfo.RB_Branch_id;
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.CreateBy = userInfo.EmployeeId;
                demodel.CreateDate = DateTime.Now;
                demodel.UpdateBy = userInfo.EmployeeId;
                demodel.UpdateDate = DateTime.Now;
            }
            return propertyModule.SetPropertyInfoBatchImport(deList, userInfo);
        }

        /// <summary>
        /// 获取资产信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());
            if (demodel.Id <= 0) {
                return ApiResult.ParamIsNull("请传递资产id");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            RB_Property_Info_Extend model = propertyModule.GetPropertyInfo(demodel);

            return ApiResult.Success("", new
            {
                model.Id,
                model.Name,
                model.PropertyNum,
                model.CategoryId,
                model.CategoryName,
                model.BrandName,
                model.Money,
                BuyDate = model.BuyDate.HasValue ? model.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                model.BuyType,
                BuyTypeName = model.BuyType.GetEnumName(),
                model.Units,
                model.UseStatus,
                UseStatusName = model.UseStatus.GetEnumName(),
                model.PropertyStatus,
                PropertyStatusName = model.PropertyStatus.GetEnumName(),
                model.PropertyModel,
                model.ImageList,
                model.SerialNumber,
                model.Remark,
                model.EmployeeId,
                model.EmName,
                GetTime = model.GetTime.HasValue ? model.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = model.BackTime.HasValue ? model.BackTime.Value.ToString("yyyy-MM-dd") : "",
                model.SupplierId,
                model.SupplierName,
                model.BranchId,
                model.UseMonths,
                model.QRCode,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(model.BranchId)?.BName ?? ""
            });
        }

        /// <summary>
        /// 设置资产信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyInfo() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入资产名称");
            }
            if ((demodel.CategoryId ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择资产分类");
            }
            if (string.IsNullOrEmpty(demodel.BrandName))
            {
                return ApiResult.ParamIsNull("请输入品牌信息");
            }
            if (!demodel.BuyDate.HasValue)
            {
                return ApiResult.ParamIsNull("请选择购置日期");
            }
            if (!demodel.BranchId.HasValue)
            {
                return ApiResult.ParamIsNull("请选择资产所属公司");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = propertyModule.SetPropertyInfo(demodel, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyInfo() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int PropertyId = parms.GetInt("PropertyId", 0);
            if (PropertyId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = propertyModule.DelPropertyInfo(PropertyId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取供应商分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSupplierPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Supplier_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Supplier_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Property_Supplier_Extend> list = propertyModule.GetSupplierPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.Contact,
                x.Mobile,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取供应商列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetSupplierList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Supplier_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Supplier_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Property_Supplier_Extend> list = propertyModule.GetSupplierList(demodel);
            return ApiResult.Success("", list.Select(x => new
            {
                x.Id,
                x.Name,
                x.Contact,
                x.Mobile
            }));
        }

        /// <summary>
        /// 设置供应商信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetSupplierInfo() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Supplier_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Supplier_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请传递供应商名称");
            }
            if (string.IsNullOrEmpty(demodel.Contact))
            {
                return ApiResult.ParamIsNull("请传递联系人名称");
            }
            if (string.IsNullOrEmpty(demodel.Mobile))
            {
                return ApiResult.ParamIsNull("请传递联系人手机号码");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = propertyModule.SetSupplierInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除供应商信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelSupplierInfo() {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int SupplierId = parms.GetInt("SupplierId", 0);
            if (SupplierId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = propertyModule.DelSupplierInfo(SupplierId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取资产日志
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyLogPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Log_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Log_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            List<RB_Property_Log_Extend> list = propertyModule.GetPropertyLogPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.TypeName,
                x.Content,
                IsSignature = x.IsSignature ?? 0,
                x.SignImage,
                CreateBy = CacheManager.User.UserReidsCache.GetEmployee(x.CreateBy)?.EmName ?? "",
                CreateDate = x.CreateDate.Value.ToString("yyyy-MM-dd HH:mm:ss")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取购置方式枚举
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyBuyTypeEnumList() {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyBuyTypeEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }
        /// <summary>
        /// 获取使用状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyUseStateEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyUseStateEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        /// <summary>
        /// 获取资产状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyStatusEnumList() {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyStatusEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        /// <summary>
        /// 获取资产来源
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertySourceEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertySourceEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        /// <summary>
        /// 获取资产领用状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyOrderStateEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.OrderStateEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        #endregion

        #region 流程设定
        /// <summary>
        /// 获取资产审批流程设定列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyFlowAuditSetList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            var auditList = propertyModule.GetPropertyFlowAuditSetList(userInfo.RB_Group_id);
            var signList = propertyModule.GetPropertySignSetList(userInfo.RB_Group_id);

            var auditEnumlist = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyFlowEnum));
            var signEnumlist = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertySignEnum));

            List<object> RobjAudit = new List<object>();
            List<object> RobjSign = new List<object>();
            foreach (var item in auditEnumlist) {
                var auditModel = auditList.Where(x => x.Type == (PropertyFlowEnum)Convert.ToInt32(item.Value)).FirstOrDefault();
                if (auditModel != null)
                {
                    RobjAudit.Add(new
                    {
                        auditModel.Type,
                        Name = auditModel.Type.GetEnumName(),
                        auditModel.Enabled
                    });
                }
                else {
                    RobjAudit.Add(new
                    {
                        Type = item.Value,
                        Name = item.Key,
                        Enabled = 2
                    });
                }
            }
            foreach (var item in signEnumlist)
            {
                var signModel = signList.Where(x => x.Type == (PropertySignEnum)Convert.ToInt32(item.Value)).FirstOrDefault();
                if (signModel != null)
                {
                    RobjSign.Add(new
                    {
                        signModel.Type,
                        Name = signModel.Type.GetEnumName(),
                        signModel.Enabled
                    });
                }
                else
                {
                    RobjSign.Add(new
                    {
                        Type = item.Value,
                        Name = item.Key,
                        Enabled = 2
                    });
                }
            }

            return ApiResult.Success("", new
            {
                AuditList = RobjAudit,
                SignList = RobjSign
            });
        }

        /// <summary>
        /// 保存审批设定
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyFlowAuditSetList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int Type = parms.GetInt("Type", 0);
            int Enabled = parms.GetInt("Enabled", 2);
            int AuditType = parms.GetInt("AuditType", 1);
            if (Type <= 0) {
                return ApiResult.Failed("请传递type参数");
            }

            bool flag = propertyModule.SetPropertyFlowAuditSetList(Type, Enabled, AuditType, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else {
                return ApiResult.Failed();
            }
        }
        #endregion

        #region 资产领用
        /// <summary>
        /// 获取资产领用分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyReceivePageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.Receive;
            if (demodel.IsSelfApplication == 1) {
                demodel.CreateBy = userInfo.EmployeeId;
            }
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                Time = x.Time.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                TempleteId = x.TempleteId ?? 0,
                x.IsApplyForSign,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                x.CreateBy,
                IsSelfApplication = x.IsSelfApplication ?? 0,
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "") + (x.IsSelfApplication == 1 ? "(自助申请)" : ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 领用下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyReceivePageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领用公司"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领用部门"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领用人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"领取时间"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.Receive;
                if (demodel.IsSelfApplication == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                }
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList) {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.BranchName){Rowspan=Count },
                                    new ExcelColumn(value:item.DepartmentName){Rowspan=Count },
                                    new ExcelColumn(value:item.EmName){Rowspan=Count },
                                    new ExcelColumn(value:item.Time.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }                                    
                                }
                            };
                            slist.Add(datarow);
                        }
                        else {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产领用.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产领用.xls");
            }

        }

        /// <summary>
        /// 资产派发
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyReceive()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if (demodel.EmployeeId <= 0)
            {
                return ApiResult.ParamIsNull("请选择领用人");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择领取时间");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Receive;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;
            demodel.DepartmentId = emodel.RB_Department_Id;
            demodel.Status = 0;
            demodel.IsSelfApplication = demodel.IsSelfApplication ?? 0;
            if (demodel.IsSelfApplication == 1 && demodel.EmployeeId != userInfo.EmployeeId){
                return ApiResult.Failed("自助申请传递参数有误");
            }

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "") {
                return ApiResult.Failed("资产编码" + vmsg + " 不是空闲状态，无法领取");
            }

            if (demodel.Id > 0) {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate) {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 变更领用人
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyChangePersonReceive()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if (demodel.EmployeeId <= 0)
            {
                return ApiResult.ParamIsNull("请选择领用人");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择领取时间");
            }
            if (demodel.PropertyId <= 0) {
                return ApiResult.ParamIsNull("请选择变更领用人的资产");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Receive;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;
            demodel.DepartmentId = emodel.RB_Department_Id;
            demodel.Status = 0;
            demodel.IsSelfApplication = demodel.IsSelfApplication ?? 0;
            if (demodel.IsSelfApplication == 1 && demodel.EmployeeId != userInfo.EmployeeId)
            {
                return ApiResult.Failed("自助申请传递参数有误");
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyChangePersonReceive(demodel, userInfo);
        }

        /// <summary>
        /// 终止资产派发
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyReceive() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产派发
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyReceive()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        #endregion

        #region 资产退库
        /// <summary>
        /// 获取资产退库分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyCancellingStocksPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.CancellingStocks;
            if (demodel.IsSelfApplication == 1) {
                demodel.CreateBy = userInfo.EmployeeId;
            }
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyCancellingStocksPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                Time = x.BackTime.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                x.OrderCode,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                TempleteId = x.TempleteId ?? 0,
                x.IsApplyForSign,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName,
                    z.CancelStockEmployeeName
                }),
                x.CreateBy,
                IsSelfApplication = x.IsSelfApplication ?? 0,
                UpdateBy = (CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "") + (x.IsSelfApplication == 1 ? "(自助申请)" : ""),
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 退库下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyCancellingStocksPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"退库时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"退库前使用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.CancellingStocks;
                if (demodel.IsSelfApplication == 1)
                {
                    demodel.CreateBy = userInfo.EmployeeId;
                }
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyCancellingStocksPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.BackTime.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:item.Remark){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.CancelStockEmployeeName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.CancelStockEmployeeName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产退库.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产退库.xls");
            }

        }

        /// <summary>
        /// 资产退库
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyCancellingStocks()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if (!demodel.BackTime.HasValue)
            {
                return ApiResult.ParamIsNull("请选择退库时间");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.CancellingStocks;
            demodel.Status = 0;
            demodel.BranchId = -1;
            demodel.DepartmentId = 0;
            demodel.IsSelfApplication = demodel.IsSelfApplication ?? 0;

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "")
            {
                return ApiResult.Failed("资产编码" + vmsg + " 不是领用状态，无法退库");
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该申请无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产退库
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyCancellingStocks()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产退库
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyCancellingStocks()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }
        #endregion

        #region 资产审批

        /// <summary>
        /// 获取资产我发起领用的分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyMyStartedPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyWaitMyAuditPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count, 2);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                x.Source,
                SourceName = x.Source.GetEnumName(),
                Time = x.Time.HasValue ? x.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                x.AuditDescription,
                x.DisposeType,
                DisposeTypeName = x.DisposeType.GetEnumName(),
                InBranchName = (x.InBranchId.HasValue && x.InBranchId >= 0) ? (CacheManager.User.BranchReidsCache.GetBranch(x.InBranchId)?.BName ?? "") : "",
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取资产带我审批的分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyWaitMyAuditPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.AuditEmId = userInfo.EmployeeId;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyWaitMyAuditPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count, 1, userInfo);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                x.Source,
                SourceName = x.Source.GetEnumName(),
                Time = x.Time.HasValue ? x.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                x.AuditDescription,
                Is_AuditSelf = x.Is_AuditSelf ?? 0,
                TempleteId = x.TempleteId ?? 0,
                x.DisposeType,
                DisposeTypeName = x.DisposeType.GetEnumName(),
                InBranchName = (x.InBranchId.HasValue && x.InBranchId >= 0) ? (CacheManager.User.BranchReidsCache.GetBranch(x.InBranchId)?.BName ?? "") : "",
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取资产我审批的分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyMyAuditedPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.AuditedEmId = userInfo.EmployeeId;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyWaitMyAuditPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count, 1, userInfo);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                x.Source,
                SourceName = x.Source.GetEnumName(),
                Time = x.Time.HasValue ? x.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                x.AuditDescription,
                Is_AuditSelf = x.Is_AuditSelf ?? 0,
                TempleteId = x.TempleteId ?? 0,
                x.DisposeType,
                DisposeTypeName = x.DisposeType.GetEnumName(),
                InBranchName = (x.InBranchId.HasValue && x.InBranchId >= 0) ? (CacheManager.User.BranchReidsCache.GetBranch(x.InBranchId)?.BName ?? "") : "",
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 获取资产审批详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyAuditDetail() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            if (demodel.UseReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递单据id");
            }

            RB_Property_UseReceive_Extend model = propertyModule.GetPropertyAuditDetail(demodel.UseReceiveId, userInfo);
            return ApiResult.Success("", new
            {
                model.Id,
                model.Remark,
                model.EmployeeId,
                model.EmName,
                ChangePerson = model.ChangePerson.HasValue ? CacheManager.User.UserReidsCache.GetEmployee(model.ChangePerson)?.EmName ?? "" : "",
                model.OrderCode,
                model.TemplateName,
                Time = model.Time.HasValue ? model.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = model.BackTime.HasValue ? model.BackTime.Value.ToString("yyyy-MM-dd") : "",
                model.AuditStatus,
                AuditStatusName = model.AuditStatus.GetEnumName(),
                model.BranchName,
                model.DepartmentName,
                model.Source,
                SourceName = model.Source.GetEnumName(),
                Is_AuditSelf = model.Is_AuditSelf ?? 0,
                Is_CanAudit = model.Is_CanAudit ?? 0,
                model.DisposeType,
                DisposeTypeName = model.DisposeType.GetEnumName(),
                InBranchName = (model.InBranchId.HasValue && model.InBranchId >= 0) ? (CacheManager.User.BranchReidsCache.GetBranch(model.InBranchId)?.BName ?? "") : "",
                model.DisposeMoney,
                model.ImageList,
                DetailList = model.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName,
                    z.PropertyModel
                }),
                AuditSteps = model.AuditSteps?.Select(y => new
                {
                    y.Id,
                    y.Sort,
                    y.SpecialNode,
                    y.AuditWay,
                    AuditWayName = EnumHelper.GetEnumName(y.AuditWay),
                    y.NextStep,
                    AuditStatus = EnumHelper.GetEnumName(y.Stauts),
                    Status = y.Stauts,
                    y.AuditDescription,
                    AuditRecordList = y.AuditRecordList?.Select(z => new { ID = z.Id, AuditName = z.AuditEmName, AduitDate = z.UpdateTime.HasValue ? z.UpdateTime.Value.ToString("yyyy-MM-dd HH:mm:ss") : "", z.CareOfRemarks, Remark = z.Description, AuditStatusStr = z.AuditStatus != null ? EnumHelper.GetEnumName(z.AuditStatus) : "", z.AuditStatus, ImageList = z.imgList?.Select(q => Config.GetFileUrl(q)), SignImage = z.SignImage ?? "" })
                })
            });
        }

        /// <summary>
        /// 审批通过 或者 驳回
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult AuditOrRefund() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            if (demodel.UseReceiveId <= 0) {
                return ApiResult.ParamIsNull("请传递单据id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(demodel.UseReceiveId ?? 0);
            if (model == null) {
                return ApiResult.Failed("单据不存在，请核实后再试");
            }
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该申请单不在审核中，无法审核");
            }

            if (demodel.AuditType == 1)
            {
                //通过
                return propertyModule.AuditPass(model, demodel.ImageList, demodel.AuditDescription, demodel.SignImage, userInfo);
            }
            else {
                if (string.IsNullOrEmpty(demodel.AuditDescription)) {
                    return ApiResult.Failed("请输入驳回理由");
                }
                //驳回
                return propertyModule.AuditReject(model, demodel.ImageList, demodel.AuditDescription, userInfo);
            }
        }

        /// <summary>
        /// 审批转交
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult AuditCareOf() {
            var requestParm = GetRequestParm();
            List<Rb_Workflow_Auditrecord_Extend> list = JsonConvert.DeserializeObject<List<Rb_Workflow_Auditrecord_Extend>>(requestParm.msg.ToString());
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            foreach (var item in list)
            {
                if (item.AuditEmId == 0)
                {
                    return ApiResult.ParamIsNull("参数EmId值有误，请核实后再试！");
                }
                if (item.WorkFlowId == 0)
                {
                    return ApiResult.ParamIsNull("参数WorkFlowId值有误，请核实后再试！");
                }
                if (string.IsNullOrEmpty(item.Description))
                {
                    return ApiResult.ParamIsNull("请填写转交备注！");
                }
                if ((item.TransferEmpId ?? 0) > 0)
                {
                    if (item.AuditEmId == item.TransferEmpId)
                    {
                        return ApiResult.Failed("不能转交给原审核人");
                    }
                }
                else
                {
                    if (item.AuditEmId == userInfo.EmployeeId)
                    {
                        return ApiResult.Failed("不能转交给原审核人");
                    }
                }
            }
            return propertyModule.Transfer(list, userInfo);
        }

        /// <summary>
        /// 获取审批日志
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetAuditChangeLogPageList() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            Model.Entity.RB_Workfolw_Change_Log demodel = JsonConvert.DeserializeObject<Model.Entity.RB_Workfolw_Change_Log>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetAuditChangeLogPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.LogId,
                x.Content,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd HH:mm:ss")
            });
            return ApiResult.Success("", pagelist);
        }

        #endregion

        #region 领用签字
        /// <summary>
        /// 获取资产待我签名的分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyWaitMySignPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.AuditEmId = userInfo.EmployeeId;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyWaitMySignPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count, 1, userInfo);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                x.Source,
                SourceName = x.Source.GetEnumName(),
                Time = x.Time.HasValue ? x.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.AuditStatus,
                AuditStatusName = "待签名",
                x.BranchName,
                x.DepartmentName,
                x.AuditDescription,
                Is_AuditSelf = x.Is_AuditSelf ?? 0,
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }


        /// <summary>
        /// 获取资产我已签名的分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyMySignedPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.AuditedEmId = userInfo.EmployeeId;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyWaitMySignPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count, 1, userInfo);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                x.Source,
                SourceName = x.Source.GetEnumName(),
                Time = x.Time.HasValue ? x.Time.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.AuditStatus,
                AuditStatusName = x.AuditStatus == OrderStateEnum.Auditing ? "待签名" : x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                x.AuditDescription,
                Is_AuditSelf = x.Is_AuditSelf ?? 0,
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }
        #endregion

        #region 资产借用
        /// <summary>
        /// 获取资产借用分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyBorrowPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.Borrow;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                x.OrderCode,
                Time = x.Time.Value.ToString("yyyy-MM-dd"),
                BackTime = x.BackTime.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.BranchName,
                x.DepartmentName,
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 借用下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyBorrowPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借用公司"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借用部门"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借用人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借取时间"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预计归还时间"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.Borrow;
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.BranchName){Rowspan=Count },
                                    new ExcelColumn(value:item.DepartmentName){Rowspan=Count },
                                    new ExcelColumn(value:item.EmName){Rowspan=Count },
                                    new ExcelColumn(value:item.Time.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:item.BackTime.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产借用.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产借用.xls");
            }

        }

        /// <summary>
        /// 资产借用
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyBorrow()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if (demodel.EmployeeId <= 0)
            {
                return ApiResult.ParamIsNull("请选择借用人");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择借用时间");
            }
            if (!demodel.BackTime.HasValue)
            {
                return ApiResult.ParamIsNull("请选择归还时间");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Borrow;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;
            demodel.DepartmentId = emodel.RB_Department_Id;
            demodel.Status = 0;

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "")
            {
                return ApiResult.Failed("资产编码" + vmsg + " 不是空闲状态，无法借用");
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产借用
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyBorrow()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产借用
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyBorrow()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        #endregion

        #region 资产归还
        /// <summary>
        /// 获取资产归还分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyGiveBackPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.GiveBack;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyCancellingStocksPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                Time = x.BackTime.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                x.OrderCode,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName,
                    z.CancelStockEmployeeName
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 归还下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyGiveBackPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"归还时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"归还前使用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.GiveBack;
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyCancellingStocksPageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.BackTime.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:item.Remark){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.CancelStockEmployeeName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.CancelStockEmployeeName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产归还.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产归还.xls");
            }

        }

        /// <summary>
        /// 资产归还
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyGiveBack()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if (!demodel.BackTime.HasValue)
            {
                return ApiResult.ParamIsNull("请选择退库时间");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.GiveBack;
            demodel.Status = 0;
            demodel.BranchId = -1;
            demodel.DepartmentId = 0;

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "")
            {
                return ApiResult.Failed("资产编码" + vmsg + " 不是借用状态，无法归还");
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该申请无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产归还
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyGiveBack()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产归还
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyGiveBack()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }
        #endregion

        #region 资产调拨
        /// <summary>
        /// 获取资产调拨分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyAllotPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.Allot;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.OrderCode,
                Time = x.Time.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                x.InBranchName,
                x.InBranchId,
                TempleteId = x.TempleteId ?? 0,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName,
                    z.HistoryBranchName
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 调拨下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyAllotPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"调入公司"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"调拨时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"调拨前公司"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.Allot;
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.InBranchName){ Rowspan=Count},
                                    new ExcelColumn(value:item.Time.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:item.Remark){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.HistoryBranchName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:citem.HistoryBranchName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产调拨.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产调拨.xls");
            }

        }

        /// <summary>
        /// 资产调拨
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyAllot()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if ((demodel.InBranchId ?? -1) <= -1)
            {
                return ApiResult.ParamIsNull("请选择调入公司");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择调拨日期");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Allot;
            demodel.BranchId = -1;
            demodel.DepartmentId = 0;
            demodel.Status = 0;

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "")
            {
                return ApiResult.Failed("资产编码" + vmsg + " 不是空闲状态，无法调拨");
            }

            foreach (var item in PList) {
                if (item.BranchId == demodel.InBranchId) {
                    return ApiResult.Failed("资产：" + item.PropertyNum + " 所在公司为调入公司！");
                }
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产调拨
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyAllot()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产调拨
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyAllot()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        #endregion

        #region 资产维修
        /// <summary>
        /// 获取资产维修分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyMaintainPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.Maintain;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.OrderCode,
                x.EmployeeId,
                x.EmName,
                Time = x.Time.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                TempleteId = x.TempleteId ?? 0,
                x.ImageList,
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName,
                    MendMoney = z.MendMoney ?? 0
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 维修下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyMaintainPageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"报修人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"维修时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.Maintain;
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.EmName){Rowspan=Count },
                                    new ExcelColumn(value:item.Time.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:item.Remark){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产维修.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产维修.xls");
            }

        }

        /// <summary>
        /// 资产维修
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyMaintain()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if ((demodel.EmployeeId ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择报修人");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择报修日期");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
                if ((item.MendMoney ?? 0) <= 0) {
                    return ApiResult.ParamIsNull("请输入维修金额");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Maintain;
            var emodel = CacheManager.User.UserReidsCache.GetEmployee(demodel.EmployeeId);
            demodel.BranchId = emodel.RB_Branch_id;
            demodel.DepartmentId = emodel.RB_Department_Id;
            demodel.Status = 0;
            demodel.ImageList = demodel.ImageList ?? new List<string>();

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            //if (vmsg != "")  // 维修不用状态
            //{
            //    return ApiResult.Failed("资产编码" + vmsg + " 不是空闲状态，无法调拨");
            //}

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产维修
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyMaintain()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产维修
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyMaintain()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        #endregion

        #region 资产处置
        /// <summary>
        /// 获取资产处置分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyDisposePageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.Source = PropertySourceEnum.Dispose;
            List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Remark,
                x.OrderCode,
                Time = x.Time.Value.ToString("yyyy-MM-dd"),
                x.AuditStatus,
                AuditStatusName = x.AuditStatus.GetEnumName(),
                TempleteId = x.TempleteId ?? 0,
                x.DisposeMoney,
                x.DisposeType,
                DisposeTypeName = x.DisposeType.GetEnumName(),
                DetailList = x.DetailList.Select(z => new
                {
                    z.Id,
                    z.CategoryName,
                    z.PropertyName,
                    z.PropertyNum,
                    z.PropertyId,
                    z.BrandName
                }),
                x.CreateBy,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 处置下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyDisposePageListToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"单号"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"处置类型"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"处置时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"处置金额"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"备注"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编号"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.Source = PropertySourceEnum.Dispose;
                List<RB_Property_UseReceive_Extend> list = propertyModule.GetPropertyReceivePageList(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 1;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var citem in item.DetailList)
                    {
                        int Count = item.DetailList.Count();
                        if (Num == 1)
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:item.AuditStatus.GetEnumName()){ Rowspan=Count },
                                    new ExcelColumn(value:item.OrderCode){Rowspan=Count },
                                    new ExcelColumn(value:item.DisposeType.GetEnumName()){Rowspan=Count },
                                    new ExcelColumn(value:item.Time.Value.ToString("yyyy-MM-dd")){Rowspan=Count },
                                    new ExcelColumn(value:(item.DisposeMoney??0).ToString()){Rowspan=Count },
                                    new ExcelColumn(value:item.Remark){Rowspan=Count },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:(CacheManager.User.UserReidsCache.GetEmployee(item.UpdateBy)?.EmName ?? "") + (item.IsSelfApplication == 1 ? "(自助申请)" : "")){Rowspan=Count },
                                    new ExcelColumn(value:item.UpdateDate.Value.ToString("yyyy-MM-dd")){Rowspan=Count }
                                }
                            };
                            slist.Add(datarow);
                        }
                        else
                        {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:citem.PropertyNum){ },
                                    new ExcelColumn(value:citem.PropertyName){ },
                                    new ExcelColumn(value:citem.CategoryName){ },
                                    new ExcelColumn(value:citem.BrandName){ },
                                    new ExcelColumn(value:""){ },
                                    new ExcelColumn(value:""){ }
                                }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产处置.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产处置.xls");
            }

        }

        /// <summary>
        /// 资产处置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyDispose()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_UseReceive_Extend demodel = JsonConvert.DeserializeObject<RB_Property_UseReceive_Extend>(requestParm.msg.ToString());

            if ((demodel.DisposeType ?? 0) <= 0)
            {
                return ApiResult.ParamIsNull("请选择处置类型");
            }
            if (!demodel.Time.HasValue)
            {
                return ApiResult.ParamIsNull("请选择处置日期");
            }
            if (demodel.DetailList == null || !demodel.DetailList.Any())
            {
                return ApiResult.ParamIsNull("请选择资产");
            }
            foreach (var item in demodel.DetailList)
            {
                if ((item.PropertyId ?? 0) <= 0)
                {
                    return ApiResult.ParamIsNull("请传递资产id");
                }
            }
            if (demodel.DetailList.Count() != demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct().Count())
            {
                return ApiResult.ParamIsNull("资产有重复选择，请核实后再试");
            }

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.Source = PropertySourceEnum.Dispose;
            demodel.BranchId = -1;
            demodel.DepartmentId = 0;
            demodel.Status = 0;
            demodel.ImageList = demodel.ImageList ?? new List<string>();
            demodel.DisposeMoney = demodel.DisposeMoney ?? 0;

            List<RB_Property_Info_Extend> PList = new List<RB_Property_Info_Extend>();
            string vmsg = propertyModule.ValidatePropertyStatus(demodel.Id, demodel.Source, string.Join(",", demodel.DetailList.Select(x => x.PropertyId ?? 0).Distinct()), demodel.RB_Group_Id ?? 0, out PList);
            if (vmsg != "")
            {
                return ApiResult.Failed("资产编码" + vmsg + " 不是空闲状态，无法处置");
            }

            if (demodel.Id > 0)
            {
                //验证是否可以修改
                var model = propertyModule.GetUseReceiveEntity(demodel.Id);
                if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
                {
                    return ApiResult.Failed("该单据无法修改");
                }
            }
            return propertyModule.SetPropertyUseReceiveCommon(demodel, userInfo, PList);
        }

        /// <summary>
        /// 终止资产处置 （审核中终止  已完成取消）
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult CancelPropertyDispose()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Auditing)
            {
                return ApiResult.Failed("该单据无法终止");
            }

            bool flag = propertyModule.CancelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除资产处置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyDispose()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int ReceiveId = parms.GetInt("ReceiveId", 0);
            if (ReceiveId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证状态
            var model = propertyModule.GetUseReceiveEntity(ReceiveId);
            if (model.AuditStatus != OrderStateEnum.Reject && model.AuditStatus != OrderStateEnum.Terminate)
            {
                return ApiResult.Failed("该单据无法删除");
            }

            bool flag = propertyModule.DelPropertyReceive(ReceiveId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取资产处置枚举列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyDisposeEnumList() {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyDisposeEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        #endregion

        #region 资产折旧
        /// <summary>
        /// 获取资产折旧 分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyDepreciationPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Depreciation_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Depreciation_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyDepreciationPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.Type,
                x.ResidualsRate,
                CategoryList = x.CategoryList.Select(z => new
                {
                    z.Id,
                    z.Name
                }),
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 设置折旧详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyDepreciationInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Depreciation_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Depreciation_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入折旧名称");
            }
            if (demodel.Type != 1 && demodel.Type != 2)
            {
                return ApiResult.ParamIsNull("请传递类型");
            }
            if (demodel.ResidualsRate <= 0) {
                return ApiResult.ParamIsNull("请输入残值率");
            }
            if (demodel.CategoryList == null || !demodel.CategoryList.Any()) {
                return ApiResult.ParamIsNull("请选择分类");
            }
            foreach (var item in demodel.CategoryList) {
                if (item.Id <= 0) {
                    return ApiResult.ParamIsNull("请传递分类");
                }
            }
            //parentId如果更换  查询是否跟换到其下级目录    未做

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.CategoryIdString = string.Join(",", demodel.CategoryList.Select(x => x.Id));
            bool flag = propertyModule.SetPropertyDepreciationInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除折旧
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyDepreciationInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int DepreciationId = parms.GetInt("DepreciationId", 0);
            if (DepreciationId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = propertyModule.DelPropertyDepreciationInfo(DepreciationId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }
        #endregion

        #region 资产盘点

        /// <summary>
        /// 获取资产盘点 分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyCheckPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Check_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Check_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyCheckPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.CheckState,
                x.CategoryIdList,
                x.BranchId,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                StartDate = x.StartDate.HasValue ? x.StartDate.Value.ToString("yyyy-MM-dd") : "",
                EndDate = x.EndDate.HasValue ? x.EndDate.Value.ToString("yyyy-MM-dd") : "",
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 设置盘点详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyCheckInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Check_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Check_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入盘点单名称");
            }
            if (demodel.CategoryIdList == null || !demodel.CategoryIdList.Any())
            {
                return ApiResult.ParamIsNull("请选择分类");
            }

            demodel.Status = 0;
            demodel.CheckState = 1;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            demodel.CategoryIds = string.Join(",", demodel.CategoryIdList);
            
            bool flag = propertyModule.SetPropertyCheckInfo(demodel);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 删除盘点
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult DelPropertyCheckInfo()
        {
            RequestParm requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }

            bool flag = propertyModule.DelPropertyCheckInfo(CheckId, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 设置盘点单状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyCheckStatus()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckId = parms.GetInt("CheckId", 0);
            int Type = parms.GetInt("Type", 0);
            if (CheckId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            if (Type == 1) {
                //取消盘点单已完成
                if (propertyModule.IsPropertyCheckDeal(CheckId, userInfo.RB_Group_id)) {
                    return ApiResult.Failed("盘点单内的资产已进行盈亏处理，无法取消复核");
                }
            }
            bool flag = propertyModule.SetPropertyCheckStatus(CheckId, Type, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }


        /// <summary>
        /// 获取资产盘点详情 分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyCheckDetailPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_CheckDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Property_CheckDetail_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyCheckDetailPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.AuditStatus,
                x.CheckStatus,
                CheckStatusName = x.CheckStatus.GetEnumName(),
                x.Remark,
                x.PropertyNum,
                x.PropertyId,
                x.PropertyName,
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.CategoryName,
                x.BrandName,
                x.ImageList,
                x.AuditEmpId,
                AuditEmpName = x.AuditEmpId.HasValue && x.AuditEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(x.AuditEmpId)?.EmName : "",
                AuditDate = x.AuditDate.HasValue ? x.AuditDate.Value.ToString("yyyy-MM-dd") : "",
                x.CheckEmpId,
                CheckEmpName = x.CheckEmpId.HasValue && x.CheckEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(x.CheckEmpId)?.EmName : "",
                CheckDate = x.CheckDate.HasValue ? x.CheckDate.Value.ToString("yyyy-MM-dd") : "",
                DealStatus = x.DealStatus??0,
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 设置盘点单明细状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyCheckDetailStatus() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckDetailId = parms.GetInt("CheckDetailId", 0);
            int Type = parms.GetInt("Type", 0);
            string Remark = parms.GetStringValue("Remark");
            if (CheckDetailId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            //验证盘点单是否已整单完成
            if (Type == 1 || Type == 2 || Type == 3 || Type == 6)
            {
                if (propertyModule.ValidataPropertyCheckDetail(CheckDetailId)) {
                    return ApiResult.Failed("该盘点单已整单完成，无法操作");
                }
            }
            bool flag = propertyModule.SetPropertyCheckDetailStatus(CheckDetailId, Type, Remark, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 设置盘点图片
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyCheckImage() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int CheckDetailId = parms.GetInt("CheckDetailId", 0);
            string ImageList = parms.GetStringValue("ImageList");
            if (CheckDetailId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            
            if (string.IsNullOrEmpty(ImageList))
            {
                return ApiResult.ParamIsNull("请上传图片");
            }
            List<string> ImagesList = JsonConvert.DeserializeObject<List<string>>(ImageList);

            //验证盘点单是否已整单完成
            if (propertyModule.ValidataPropertyCheckDetail(CheckDetailId))
            {
                return ApiResult.Failed("该盘点单已整单完成，无法操作");
            }

            bool flag = propertyModule.SetPropertyCheckImage(CheckDetailId, ImagesList, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取盘点方式枚举
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyCheckStatusEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyCheckStatusEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        /// <summary>
        /// 盘点详情下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyCheckDetailToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_CheckDetail_Extend demodel = JsonConvert.DeserializeObject<RB_Property_CheckDetail_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"审核状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点备注"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产编码"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产品牌"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点人"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"盘点时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"审核人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"审核时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }
                         },
            };
            slist.Add(header);
            try
            {

                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.GetPropertyCheckDetailList(demodel);

                #region 组装数据
                foreach (var item in list)
                {
                    string AuditStatusName = "";
                    switch (item.AuditStatus) {
                        case 1:
                            AuditStatusName = "未审核";break;
                        case 2:
                            AuditStatusName = "已审核"; break;
                        case 3:
                            AuditStatusName = "审核驳回"; break;
                    }
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:AuditStatusName){ },
                            new ExcelColumn(value:item.CheckStatus.GetEnumName()){ },
                            new ExcelColumn(value:item.Remark){ },
                            new ExcelColumn(value:item.PropertyNum){ },
                            new ExcelColumn(value:item.PropertyName){ },
                            new ExcelColumn(value:item.CategoryName){ },
                            new ExcelColumn(value:item.BrandName){ },
                            new ExcelColumn(value:item.CheckEmpId.HasValue && item.CheckEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(item.CheckEmpId)?.EmName : ""){ },
                            new ExcelColumn(value:item.CheckDate.HasValue ? item.CheckDate.Value.ToString("yyyy-MM-dd") : ""){ },
                            new ExcelColumn(value:item.AuditEmpId.HasValue && item.AuditEmpId > 0 ? CacheManager.User.UserReidsCache.GetEmployee(item.AuditEmpId)?.EmName : ""){ },
                            new ExcelColumn(value:item.AuditDate.HasValue ? item.AuditDate.Value.ToString("yyyy-MM-dd") : ""){ }
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion


                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "盘点单详情.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "盘点单详情.xls");
            }

        }

        /// <summary>
        /// 获取资产是否在盘点
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIsHaveCheck()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int PropertyId = parms.GetInt("PropertyId", 0);
            if (PropertyId <= 0) {
                return ApiResult.Failed("请传递资产id");
            }

            bool flag = propertyModule.GetPropertyIsHaveCheck(PropertyId, userInfo);
            return ApiResult.Success("", flag);
        }

        /// <summary>
        /// 设置app盘点单明细状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetPropertyAPPCheckDetailStatus()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int PropertyId = parms.GetInt("PropertyId", 0);
            int Type = parms.GetInt("Type", 0);
            string Remark = parms.GetStringValue("Remark");
            if (PropertyId <= 0)
            {
                return ApiResult.ParamIsNull("请传递参数id");
            }
            if (Type != 1 && Type != 2 && Type != 3)
            {
                return ApiResult.ParamIsNull("type参数有误");
            }

            return propertyModule.SetPropertyAPPCheckDetailStatus(PropertyId, Type, Remark, userInfo);
        }

        /// <summary>
        /// 获取资产二维码列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyQRCodeList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyQRCodeList(demodel);

            return ApiResult.Success("", list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.QRCode
            }));
        }
        #endregion

        #region 统计报表

        /// <summary>
        /// 获取资产折旧统计列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetDepreciationStatisticsPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.EndTime)) {
                return ApiResult.ParamIsNull("请传递月份");
            }
            demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
            demodel.IsSelectDepreciation = 1;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetDepreciationStatisticsPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                BuyDate = x.BuyDate.HasValue ? x.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                x.BuyType,
                BuyTypeName = x.BuyType.GetEnumName(),
                x.Units,
                x.UseStatus,
                UseStatusName = x.UseStatus.GetEnumName(),
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.PropertyModel,
                x.ImageList,
                x.SerialNumber,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                GetTime = x.GetTime.HasValue ? x.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.SupplierId,
                x.SupplierName,
                x.BranchId,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                x.DepreciationType,
                x.ResidualsRate,
                MonthDepreciation = Math.Round(x.MonthDepreciation ?? 0, 2, MidpointRounding.AwayFromZero),
                TotalDepreciation = Math.Round(x.TotalDepreciation ?? 0, 2, MidpointRounding.AwayFromZero),
                x.UsedMonth,
                x.UseMonths
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 折旧统计下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetDepreciationStatisticsToExcel() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"设备序列号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"所在公司"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"折旧类型"){CellWidth=20,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"本月折旧"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"累计折旧"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"残值率"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预计使用月份"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"已经使用月份"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                if (string.IsNullOrEmpty(demodel.EndTime))
                {
                    var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                    return File(byteData1, "application/octet-stream", "资产折旧统计.xls");
                }

                demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
                demodel.IsSelectDepreciation = 1;
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.GetDepreciationStatisticsList(demodel);

                #region 组装数据
                decimal TMonthDepreciation = 0, TTotalDepreciation = 0;
                foreach (var item in list)
                {
                    TMonthDepreciation += (item.MonthDepreciation ?? 0);
                    TTotalDepreciation += (item.TotalDepreciation ?? 0);
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:item.PropertyNum){ },
                            new ExcelColumn(value:item.Name){ },
                            new ExcelColumn(value:item.CategoryName){ },
                            new ExcelColumn(value:item.BrandName){ },
                            new ExcelColumn(value:item.SerialNumber){ },
                            new ExcelColumn(value:item.EmName){ },
                            new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                            new ExcelColumn(value:item.DepreciationType==1?"购置/起租日当月":"购置/起租日次月"){ },
                            new ExcelColumn(value:(Math.Round(item.MonthDepreciation??0,2,MidpointRounding.AwayFromZero)).ToString("#0.00")){ },
                            new ExcelColumn(value:(Math.Round(item.TotalDepreciation??0,2,MidpointRounding.AwayFromZero)).ToString("#0.00")){ },
                            new ExcelColumn(value:(item.ResidualsRate??0).ToString()){ },
                            new ExcelColumn(value:(item.UseMonths??0).ToString()){ },
                            new ExcelColumn(value:(item.UsedMonth??0).ToString()){ }
                        }
                    };
                    slist.Add(datarow);
                }
                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计（共"+list.Count()+"条）"){ Colspan=8},
                            new ExcelColumn(value:TMonthDepreciation.ToString("#0.00")){},
                            new ExcelColumn(value:TTotalDepreciation.ToString("#0.00")){},
                            new ExcelColumn(value:""){ Colspan=3},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "资产折旧统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产折旧统计.xls");
            }
            
        }

        /// <summary>
        /// 资产汇总查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyGroupQuery() {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyGroupQuery(demodel);

            //处理数据
            var BranchList = list.Select(x => x.BranchId ?? 0).Distinct().ToList();

            List<object> RList = new List<object>();
            foreach (var bitem in BranchList) {
                List<object> R2List = new List<object>();
                var blist = list.Where(x => x.BranchId == bitem).ToList();
                //再根据分类分
                var CategoryList = blist.Select(x => x.CategoryId ?? 0).Distinct().ToList();
                foreach (var citem in CategoryList) {
                    List<object> R3List = new List<object>();
                    var clist = blist.Where(x => x.CategoryId == citem).ToList();
                    //再根据分类分
                    List<int> pslist = clist.Select(x => (int)(x.PropertyStatus ?? 0)).Distinct().ToList();
                    foreach (var psitem in pslist) {
                        //查询资产了
                        var plist = clist.Where(x => x.PropertyStatus == (PropertyStatusEnum)psitem).ToList();
                        R3List.Add(new
                        {
                            PropertyStatus = psitem,
                            PropertyStatusName = ((PropertyStatusEnum)psitem).GetEnumName(),
                            Number = plist.Sum(x => x.PropertyNumber ?? 0),
                            Money = plist.Sum(x => x.Money ?? 0)
                        });
                    }
                    var cmodel = clist.FirstOrDefault();
                    R2List.Add(new
                    {
                        CategoryId = citem,
                        CategoryName = cmodel?.CategoryName ?? "",
                        PropertyStatusList = R3List
                    });
                }
                RList.Add(new
                {
                    BranchId = bitem,
                    BranchName = CacheManager.User.BranchReidsCache.GetBranch(bitem)?.BName ?? "",
                    CategoryList = R2List
                });
            }

            return ApiResult.Success("", RList);
        }

        /// <summary>
        /// 资产汇总下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GettPropertyGroupQueryToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"公司名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"分类名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"状态"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"数量"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.GetPropertyGroupQuery(demodel);

                #region 组装数据
                //处理数据
                var BranchList = list.Select(x => x.BranchId ?? 0).Distinct().ToList();

                int bid = 0, cid = 0;
                foreach (var bitem in BranchList)
                {
                    bid = 1;                    
                    var blist = list.Where(x => x.BranchId == bitem).ToList();
                    var bcount = blist.Count();
                    //再根据分类分
                    var CategoryList = blist.Select(x => x.CategoryId ?? 0).Distinct().ToList();
                    foreach (var citem in CategoryList)
                    {
                        cid = 1;
                        var clist = blist.Where(x => x.CategoryId == citem).ToList();
                        var cmodel = clist.FirstOrDefault();
                        //再根据分类分
                        List<int> pslist = clist.Select(x => (int)(x.PropertyStatus ?? 0)).Distinct().ToList();
                        foreach (var psitem in pslist)
                        {
                            //查询资产了
                            var plist = clist.Where(x => x.PropertyStatus == (PropertyStatusEnum)psitem).ToList();
                            if (bid == 1 && cid == 1)
                            {
                                int pcount = pslist.Count();
                                ExcelDataSource datarow = new ExcelDataSource()
                                {
                                    ExcelRows = new List<ExcelColumn>(30) {
                                        new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(bitem)?.BName ?? ""){ Rowspan=bcount },
                                        new ExcelColumn(value:cmodel?.CategoryName ?? ""){ Rowspan=pcount },
                                        new ExcelColumn(value:((PropertyStatusEnum)psitem).GetEnumName()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.PropertyNumber ?? 0).ToString()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.Money ?? 0).ToString("#0.00")){ },
                                    }
                                };
                                slist.Add(datarow);
                            }
                            else if (bid != 1 && cid == 1)
                            {
                                int pcount = pslist.Count();
                                ExcelDataSource datarow = new ExcelDataSource()
                                {
                                    ExcelRows = new List<ExcelColumn>(30) {
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:cmodel?.CategoryName ?? ""){ Rowspan=pcount },
                                        new ExcelColumn(value:((PropertyStatusEnum)psitem).GetEnumName()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.PropertyNumber ?? 0).ToString()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.Money ?? 0).ToString("#0.00")){ },
                                    }
                                };
                                slist.Add(datarow);
                            }
                            else {
                                ExcelDataSource datarow = new ExcelDataSource()
                                {
                                    ExcelRows = new List<ExcelColumn>(30) {
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:((PropertyStatusEnum)psitem).GetEnumName()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.PropertyNumber ?? 0).ToString()){ },
                                        new ExcelColumn(value:plist.Sum(x => x.Money ?? 0).ToString("#0.00")){ },
                                    }
                                };
                                slist.Add(datarow);
                            }
                            bid++;
                            cid++;
                        }
                    }
                }

                #endregion

                ExcelDataSource Total = new ExcelDataSource()
                {
                    ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"合计"){ Colspan=3},
                            new ExcelColumn(value:list.Sum(x=>x.PropertyNumber??0).ToString()){},
                            new ExcelColumn(value:list.Sum(x=>x.Money??0).ToString("#0.00")){},
                         },
                };
                slist.Add(Total);

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);

                return File(byteData, "application/octet-stream", "资产汇总统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产汇总统计.xls");
            }

        }

        /// <summary>
        /// 资产履历查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyRecordPageQuery()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetPropertyRecordPageQuery(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                BuyDate = x.BuyDate.HasValue ? x.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                x.BuyType,
                BuyTypeName = x.BuyType.GetEnumName(),
                x.Units,
                x.UseStatus,
                UseStatusName = x.UseStatus.GetEnumName(),
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.PropertyModel,
                x.ImageList,
                x.SerialNumber,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                GetTime = x.GetTime.HasValue ? x.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.SupplierId,
                x.SupplierName,
                x.BranchId,
                x.UseMonths,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? "",
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd"),
                LogList = x.LogList.Select(z => new
                {
                    z.Id,
                    z.TypeName,
                    z.Content,
                    CreateBy = CacheManager.User.UserReidsCache.GetEmployee(z.CreateBy)?.EmName ?? "",
                    CreateDate = z.CreateDate.Value.ToString("yyyy-MM-dd")
                })
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 资产履历下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyRecordPageQueryToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产状态"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"所属公司"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作类型"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作时间"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作人"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"操作内容"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.GetPropertyRecordPageQuery(1, 99999, demodel, out long count);

                #region 组装数据
                int Num = 0;
                foreach (var item in list)
                {
                    Num = 1;
                    foreach (var logitem in item.LogList) {
                        if (Num == 1)
                        {
                            int pcount = item.LogList.Count();
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                        new ExcelColumn(value:item.PropertyNum){ Rowspan=pcount },
                                        new ExcelColumn(value:item.Name){ Rowspan=pcount },
                                        new ExcelColumn(value:item.CategoryName){ Rowspan=pcount },
                                        new ExcelColumn(value:item.PropertyStatus.GetEnumName()){ Rowspan=pcount },
                                        new ExcelColumn(value:item.BrandName){ Rowspan=pcount },
                                        new ExcelColumn(value:item.EmName){ Rowspan=pcount },
                                        new ExcelColumn(value:item.BrandName){ Rowspan=pcount },
                                        
                                        new ExcelColumn(value:logitem.TypeName){ },
                                        new ExcelColumn(value:logitem.CreateDate.Value.ToString("yyyy-MM-dd")){ },
                                        new ExcelColumn(value:CacheManager.User.UserReidsCache.GetEmployee(logitem.CreateBy)?.EmName ?? ""){ },
                                        new ExcelColumn(value:logitem.Content){ },
                                    }
                            };
                            slist.Add(datarow);
                        }
                        else {
                            ExcelDataSource datarow = new ExcelDataSource()
                            {
                                ExcelRows = new List<ExcelColumn>(30) {
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },
                                        new ExcelColumn(value:""){ },

                                        new ExcelColumn(value:logitem.TypeName){ },
                                        new ExcelColumn(value:logitem.CreateDate.Value.ToString("yyyy-MM-dd")){ },
                                        new ExcelColumn(value:CacheManager.User.UserReidsCache.GetEmployee(logitem.CreateBy)?.EmName ?? ""){ },
                                        new ExcelColumn(value:logitem.Content){ },
                                    }
                            };
                            slist.Add(datarow);
                        }
                        Num++;
                    }
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产履历统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产履历统计.xls");
            }

        }

        /// <summary>
        /// 获取日志类型枚举
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyLogTypeEnumList()
        {
            var model = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.PropertyLogStatusEnum)).ToList().FirstOrDefault();
            return ApiResult.Success("", model.Key.Split('|'));
        }

        /// <summary>
        /// 资产待归还查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyStayBackPageQuery()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.PropertyStatus = PropertyStatusEnum.Borrow;
            var list = propertyModule.GetPropertyStayBackPageQuery(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                BuyDate = x.BuyDate.HasValue ? x.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                x.BuyType,
                BuyTypeName = x.BuyType.GetEnumName(),
                x.Units,
                x.UseStatus,
                UseStatusName = x.UseStatus.GetEnumName(),
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.PropertyModel,
                x.ImageList,
                x.SerialNumber,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                GetTime = x.GetTime.HasValue ? x.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.SupplierId,
                x.SupplierName,
                x.BranchId,
                x.UseMonths,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? ""            
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 资产待归还下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyStayBackPageQueryToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"借用时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预计归还时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"序列号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"所属公司"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER }
                         },
            };
            slist.Add(header);
            try
            {
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                demodel.PropertyStatus = PropertyStatusEnum.Borrow;
                var list = propertyModule.GetPropertyStayBackPageQuery(1, 99999, demodel, out long count);

                #region 组装数据
                foreach (var item in list)
                {
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                                new ExcelColumn(value:item.PropertyNum){ },
                                new ExcelColumn(value:item.EmName){ },
                                new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                new ExcelColumn(value:item.BackTime.HasValue ? item.BackTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                new ExcelColumn(value:item.Name){ },
                                new ExcelColumn(value:item.CategoryName){ },
                                new ExcelColumn(value:item.BrandName){ },
                                new ExcelColumn(value:item.PropertyModel){ },
                                new ExcelColumn(value:item.SerialNumber){ },
                                new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                            }
                    };
                    slist.Add(datarow);
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产待归还统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产待归还统计.xls");
            }

        }

        /// <summary>
        /// 资产到期查询
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyMaturityPageQuery()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.EndTime)) {
                return ApiResult.ParamIsNull("请传递到期月份");
            }
            demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
            demodel.RB_Group_Id = userInfo.RB_Group_id;            
            var list = propertyModule.GetPropertyMaturityPageQuery(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                x.PropertyNum,
                x.CategoryId,
                x.CategoryName,
                x.BrandName,
                x.Money,
                BuyDate = x.BuyDate.HasValue ? x.BuyDate.Value.ToString("yyyy-MM-dd") : "",
                x.BuyType,
                BuyTypeName = x.BuyType.GetEnumName(),
                x.Units,
                x.UseStatus,
                UseStatusName = x.UseStatus.GetEnumName(),
                x.PropertyStatus,
                PropertyStatusName = x.PropertyStatus.GetEnumName(),
                x.PropertyModel,
                x.ImageList,
                x.SerialNumber,
                x.Remark,
                x.EmployeeId,
                x.EmName,
                GetTime = x.GetTime.HasValue ? x.GetTime.Value.ToString("yyyy-MM-dd") : "",
                BackTime = x.BackTime.HasValue ? x.BackTime.Value.ToString("yyyy-MM-dd") : "",
                x.SupplierId,
                x.SupplierName,
                x.BranchId,
                x.UseMonths,
                x.MaturityTime,
                BranchName = CacheManager.User.BranchReidsCache.GetBranch(x.BranchId)?.BName ?? ""
            });
            return ApiResult.Success("", pagelist);
        }

        /// <summary>
        /// 资产到期下载excel
        /// </summary>
        [HttpPost]
        public FileContentResult GetPropertyMaturityPageQueryToExcel()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            List<ExcelDataSource> slist = new List<ExcelDataSource>();
            ExcelDataSource header = new ExcelDataSource()
            {
                ExcelRows = new List<ExcelColumn>(30) {
                            new ExcelColumn(value:"资产编码"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产名称"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"资产分类"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"品牌"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"型号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"序列号"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用人"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"使用时间"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"购置日期"){CellWidth=15,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"金额"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"所属公司"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"预计使用期限"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                            new ExcelColumn(value:"到期时间"){CellWidth=25,HAlignmentEnum= HAlignmentEnum.CENTER,VAlignmentEnum= VAlignmentEnum.CENTER },
                         },
            };
            slist.Add(header);
            try
            {
                if (string.IsNullOrEmpty(demodel.EndTime))
                {
                    var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                    return File(byteData1, "application/octet-stream", "资产待归还统计.xls");
                }
                demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
                demodel.RB_Group_Id = userInfo.RB_Group_id;
                var list = propertyModule.GetPropertyMaturityPageQuery(1, 99999, demodel, out long count);

                #region 组装数据
                foreach (var item in list)
                {
                    ExcelDataSource datarow = new ExcelDataSource()
                    {
                        ExcelRows = new List<ExcelColumn>(30) {
                                new ExcelColumn(value:item.PropertyNum){ },
                                new ExcelColumn(value:item.Name){ },
                                new ExcelColumn(value:item.CategoryName){ },
                                new ExcelColumn(value:item.BrandName){ },
                                new ExcelColumn(value:item.PropertyModel){ },
                                new ExcelColumn(value:item.SerialNumber){ },
                                new ExcelColumn(value:item.EmName){ },
                                new ExcelColumn(value:item.GetTime.HasValue ? item.GetTime.Value.ToString("yyyy-MM-dd") : ""){ },
                                new ExcelColumn(value:item.BuyDate.HasValue ? item.BuyDate.Value.ToString("yyyy-MM-dd") : ""){ },
                                new ExcelColumn(value:(item.Money??0).ToString("#0.00")){ },
                                new ExcelColumn(value:CacheManager.User.BranchReidsCache.GetBranch(item.BranchId)?.BName ?? ""){ },
                                new ExcelColumn(value:item.UseMonths.Value.ToString()){ },
                                new ExcelColumn(value:item.MaturityTime){ },
                            }
                    };
                    slist.Add(datarow);
                }
                #endregion

                var byteData = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData, "application/octet-stream", "资产到期统计.xls");
            }
            catch (Exception ex)
            {
                LogHelper.Write(ex, string.Format("GetFileFromWebApi_requestData: {0}", JsonHelper.Serialize(requestParm)));
                var byteData1 = ExcelTempLateHelper.ToExcelExtend(slist);
                return File(byteData1, "application/octet-stream", "资产到期统计.xls");
            }

        }
        #endregion

        #region 首页统计
        /// <summary>
        /// 获取首页任务统计
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIndexTask()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            object robj = propertyModule.GetPropertyIndexTask(userInfo);
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取首页资产概况
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIndexGeneralSituation()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            object robj = propertyModule.GetPropertyIndexGeneralSituation(userInfo);
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取首页资产占比
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIndexPropertyStatusRatio()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);

            object robj = propertyModule.GetPropertyIndexPropertyStatusRatio(userInfo);
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取首页资产入库领用情况
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIndexPropertyInStock()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_Info_Extend demodel = JsonConvert.DeserializeObject<RB_Property_Info_Extend>(requestParm.msg.ToString());

            demodel.StartTime = Convert.ToDateTime(demodel.StartTime + "-01").ToString("yyyy-MM-dd");
            demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
            //if (string.IsNullOrEmpty(demodel.CategoryIds)) {
            //    return ApiResult.Failed("请传递分类ids");
            //}

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            object robj = propertyModule.GetPropertyIndexPropertyInStock(demodel);
            return ApiResult.Success("", robj);
        }

        /// <summary>
        /// 获取首页物料入库领用情况
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetPropertyIndexSuppliesInStock()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Supplies_StockIn_Extend demodel = JsonConvert.DeserializeObject<RB_Supplies_StockIn_Extend>(requestParm.msg.ToString());

            demodel.StartTime = Convert.ToDateTime(demodel.StartTime + "-01").ToString("yyyy-MM-dd");
            demodel.EndTime = Convert.ToDateTime(demodel.EndTime + "-01").AddMonths(1).AddDays(-1).ToString("yyyy-MM-dd");
            //if (string.IsNullOrEmpty(demodel.SuppliesIds)) {
            //    return ApiResult.Failed("请传递物料ids");
            //}
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            object robj = propertyModule.GetPropertyIndexSuppliesInStock(demodel);
            return ApiResult.Success("", robj);
        }

        #endregion

        #region 请购单
        /// <summary>
        /// 获取请购单分页列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetBuyingRequisitionPageList()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            ResultPageModel pagelist = JsonConvert.DeserializeObject<ResultPageModel>(requestParm.msg.ToString());
            RB_Property_BuyingRequisition_Extend demodel = JsonConvert.DeserializeObject<RB_Property_BuyingRequisition_Extend>(requestParm.msg.ToString());

            demodel.RB_Group_Id = userInfo.RB_Group_id;
            var list = propertyModule.GetBuyingRequisitionPageList(pagelist.pageIndex, pagelist.pageSize, demodel, out long count);
            pagelist.count = Convert.ToInt32(count);
            pagelist.pageData = list.Select(x => new
            {
                x.Id,
                x.Name,
                CategoryName = string.IsNullOrEmpty(x.CategoryName) ? "未收录" : x.CategoryName,
                x.Type,
                TypeName = x.Type == 1 ? "资产" : "耗材",
                x.BuyStatus,
                BuyStatusName = x.BuyStatus.GetEnumName(),
                x.Content,
                x.RejectRemark,
                UseTime = x.UseTime.HasValue ? x.UseTime.Value.ToString("yyyy-MM-dd") : "",
                BuyTime = x.BuyTime.HasValue ? x.BuyTime.Value.ToString("yyyy-MM-dd") : "",
                CreateBy = CacheManager.User.UserReidsCache.GetEmployee(x.CreateBy)?.EmName ?? "",
                UpdateBy = CacheManager.User.UserReidsCache.GetEmployee(x.UpdateBy)?.EmName ?? "",
                UpdateDate = x.UpdateDate.Value.ToString("yyyy-MM-dd")
            });
            return ApiResult.Success("", pagelist);
        }
        /// <summary>
        /// 设置请购单信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetBuyingRequisitionInfo()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            RB_Property_BuyingRequisition_Extend demodel = JsonConvert.DeserializeObject<RB_Property_BuyingRequisition_Extend>(requestParm.msg.ToString());

            if (string.IsNullOrEmpty(demodel.Name))
            {
                return ApiResult.ParamIsNull("请输入请购单名称");
            }
            if ((demodel.CategoryId ?? 0) < 0)
            {
                return ApiResult.ParamIsNull("请选择资产分类");
            }
            if (demodel.Type <= 0) 
            {
                return ApiResult.ParamIsNull("请选择类型");
            }
            if (string.IsNullOrEmpty(demodel.Content))
            {
                return ApiResult.ParamIsNull("请输入内容信息");
            }
            if (!demodel.UseTime.HasValue)
            {
                return ApiResult.ParamIsNull("请选择预计使用日期");
            }

            demodel.Status = 0;
            demodel.RB_Branch_Id = userInfo.RB_Branch_id;
            demodel.RB_Group_Id = userInfo.RB_Group_id;
            demodel.CreateBy = userInfo.EmployeeId;
            demodel.CreateDate = DateTime.Now;
            demodel.UpdateBy = userInfo.EmployeeId;
            demodel.UpdateDate = DateTime.Now;
            bool flag = propertyModule.SetBuyingRequisitionInfo(demodel, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 设置请购单状态
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult SetBuyingRequisitionStatus()
        {
            var requestParm = GetRequestParm();
            UserInfo userInfo = CacheManager.User.UserReidsCache.GetUserLoginInfo(requestParm.uid);
            JObject parms = JObject.Parse(requestParm.msg.ToString());
            int BuyingRequisitionId = parms.GetInt("BuyingRequisitionId", 0);
            int Type = parms.GetInt("Type", 0);// 1购买中  2驳回  3已购买  4撤回
            string BuyTime = parms.GetStringValue("BuyTime");
            string RejectRemark = parms.GetStringValue("RejectRemark");
            if (BuyingRequisitionId <= 0) {
                return ApiResult.ParamIsNull();
            }
            if (Type <= 0) {
                return ApiResult.ParamIsNull();
            }
            if (Type == 1 || Type == 3)
            {
                if (string.IsNullOrEmpty(BuyTime))
                {
                    return ApiResult.ParamIsNull("请传递购买日期");
                }
            }
            else if (Type == 4) { 
                //撤回
            }
            else
            {
                if (string.IsNullOrEmpty(RejectRemark))
                {
                    return ApiResult.ParamIsNull("请输入驳回理由");
                }
            }

            bool flag = propertyModule.SetBuyingRequisitionStatus(BuyingRequisitionId, Type, BuyTime, RejectRemark, userInfo);
            if (flag)
            {
                return ApiResult.Success();
            }
            else
            {
                return ApiResult.Failed();
            }
        }

        /// <summary>
        /// 获取请购单状态列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ApiResult GetBuyingRequisitionEnumList()
        {
            var list = EnumHelper.GetEnumList(typeof(REBORN.Common.Enum.BuyingRequisitionStatusEnum));
            return ApiResult.Success("", list.Select(x => new { Id = x.Value, Name = x.Key }));
        }

        #endregion
    }
}
